/*******************************************************************/
/*                                                                 */
/*      Test Program for SNDS Board ROM : DIAG.c		   */
/*                                                                 */ 
/*-----------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                    */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*      Modified, programmed by hbahn, from 1998.02.13             */
/*                                                                 */
/*******************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "std.h"
#include "ks5000.h"
#include "diag.h"
#include "memory.h"
#include "flash.h"
#include "down.h"
#include "pollio.h"
#include "uart.h"
#include "pic.h"
#include "lcd.h"
#include "dma.h"
#include "iic.h"
#include "timer.h"
#include "system.h"
#include "mac.h"
#include "hdlc.h"
#include "kslib.h"
#include "dhry.h"
#include "sysconf.h"
#include "iop.h"

extern U32 AllTestResult ;
extern void SNDS_ver(void);   //Display SNDS informations

// C Program Entry Point
void C_Entry(void)
{

        Diag();
}



void  Diag(void)
{

	while(1){
		InitSnds() ;
		PrintBanner() ;
		TestItemSelect() ;
	}
}

/* 
 * Function : InitSnds
 * Description : Initialize for Command Line Interface
 */
void InitSnds(void) 
{
	IOPMOD |= 0x1F ; // General Purpose Register Mode Select
	IOPDATA = 0x0 ;

	InitInterrupt() ;
        UARTPollInit(SERIAL_DEV0); 
        UARTPollInit(SERIAL_DEV1);
	LCDInit() ;
	ResetPhyChip() ;
	ClearPhyChip() ;
	IicSetup() ; 
	MacInitialize() ;
	HDLCA_init() ;
	HDLCB_init() ;

	// Interrupt Setup 
	SysSetInterrupt(nTIMER0_INT, tm0isr) ;
	SysSetInterrupt(nTIMER1_INT, tm1isr) ;
	SysSetInterrupt(nGDMA0_INT, GDMA0isr);
	SysSetInterrupt(nGDMA1_INT, GDMA1isr);

	IOPDATA = 0xF ;
}

void SNDS_ver(void)
{
     int line;

	Print("\n\n\n") ;
        Print("KS32C5000 SNDS B'd Ver Rev1.1, 1998. 03. 02. \r") ;
        Print("SAMSUNG ELECTRONIC CO. NETCOM  Div.\r");
        Print("PROGRAMMED BY HBAHN, DATE :1997.12.16. \r") ;
        Print("DIAGNOSTIC ROM Ver1.0(updated by Y.S.KIM, 1998.06.04). \r") ;
        Print("\n\nType any key...."); get_byte();

        for(line = 0; line < 24; line++) put_byte('\r');
}


// Print Banner
void PrintBanner(void)
{
	Print("\n\n") ;
	Print("                 DIAGNOSTIC TEST ITEMS                    \r\r") ;
	Print("-----------------------------------------------------------\r") ;
	Print("  1. User Pgm Download  -[P]   2. Flash Download     -[F]  \r") ;
	Print("  3. Cache/SRAM Test    -[C]   4. Memory Test        -[M]  \r") ;
	Print("  5. GDMA Test          -[D]   6. Timer Test         -[T]  \r") ;
	Print("  7. Interrupt Test     -[I]   8. MAC/Ethernet Test  -[L]  \r") ;
	Print("  9. HDLC Test          -[H]   10.I2C Test           -[S]  \r") ;
	Print("  11.UART Test          -[U]   12.I/O Port Functions -[G]  \r") ;
	Print("  13.All Test           -[A]   15.ETC Test           -[E]  \r") ;
	Print("  15.View Configuration -[V]   17.Benchmarking       -[B]  \r") ;
	Print("  18.LCD Only Test Mode -[O]  \r") ;
	Print("-----------------------------------------------------------\r") ;
	Print("> To enter the command line debugging mode, Type \"E\".    \r") ;
	Print("-----------------------------------------------------------\r") ;
}

// Test Item Select
void TestItemSelect(void)
{
	char ItemSel ;

		Print("\rSelect Test Item : ") ;
		lPrint("Select Key ...") ;
		ItemSel = get_key() ;
	
		switch(ItemSel)
		{
			case 'M' : case 'm' : MemTestDiag() ; break ;
			case 'C' : case 'c' : CacheTestDiag() ; break ;
			case 'I' : case 'i' : IntTestDiag() ; break ;
			case 'D' : case 'd' : DmaTestDiag() ; break ;
			case 'T' : case 't' : TimerTestDiag() ; break ;
			case 'L' : case 'l' : MacTestDiag() ; break ;
			case 'H' : case 'h' : HdlcTestDiag() ; break ;
			case 'S' : case 's' : I2CTestDiag() ; break ;
			case 'U' : case 'u' : UartTestDiag() ; break ;
			case 'E' : case 'e' : EtcTestDiag() ; break ;
			case 'A' : case 'a' : AllTestDiag() ; break ;
			case 'B' : case 'b' : BenchMarkDiag() ; break ;
			case 'P' : case 'p' : DownloadDiag() ; break ;
			case 'F' : case 'f' : FusingDiag() ; break ;
			case 'V' : case 'v' : SysConfig() ; break ;
			case 'O' : case 'o' : LcdOnlyTestMode() ; break ;
			case 'G' : case 'g' : IoFuncDiag(); break ;

			default : Print("\nNo Test Item Selected") ; 
				break ;
		}

	IOPDATA = 0xF ;
	Print("\n Press Any Key to Continue"); 
	lPrint("Press Any Key"); 
	get_key();
	return ;
}


void LcdOnlyTestMode(void)
{
	int i,SelCount = 0 ;
	U32 KeyInValue  ;
	char ch ;
	ITEM TestItemSel[] = { 
			"* All Test",
	 		"* Memory Test",
	 		"* Cache Test",
	 		"* GDMA Test",
	 		"* Timer Test",
	 		"* Interrupt Test",
	 		"* MAC Test",
	 		"* HDLC Test",
	 		"* I2C Test",
	 		"* UART Test",
	 		"* ETC Test", 
	 		"* Downloading", 
	 		"* Quit Test Mode" 
			} ;

	Print("\n >>   LCD Only Mode is Selected    <<") ;
	Print("\r >> Use Function Key in SNDS Board <<\n\n") ;

	for (i = 0; i < MAXTESTITEM; i++)
    		SelTestItemFunc[i] = DummyTestItem;

	InitTestItemSelFunction(0 ,AllTestDiag) ;
	InitTestItemSelFunction(1 ,MemTestDiag) ;
	InitTestItemSelFunction(2 ,CacheTestDiag) ; 
	InitTestItemSelFunction(3 ,DmaTestDiag) ;
	InitTestItemSelFunction(4 ,TimerTestDiag) ; 
	InitTestItemSelFunction(5 ,IntTestDiag) ;
	InitTestItemSelFunction(6 ,MacTestDiag) ;
	InitTestItemSelFunction(7 ,HdlcTestDiag) ;
	InitTestItemSelFunction(8 ,I2CTestDiag) ;
	InitTestItemSelFunction(9 ,UartTestDiag) ; 
	InitTestItemSelFunction(10 ,EtcTestDiag) ;
	InitTestItemSelFunction(11 ,DownloadDiag) ;

	while(1) {

	// ** For Test
	//Print("\r SelCount : %2d -> ",SelCount) ;
	//dbg_out("%s",TestItemSel[SelCount].TestItem) ;

	ClearLcdDisplay() ;
	lPrint(TestItemSel[SelCount].TestItem) ;

	// Wait Until Key Input Invalid
	while(1) {
		KeyInValue = (IOPDATA>>5) & 0x7 ;
		if (KeyInValue == 0x7) break ;
		} 

	// Receive Key Input
	while(1) {
	KeyInValue = (IOPDATA>>5) & 0x7 ;
		switch(KeyInValue)
			{
			case 0x6 : ch = 'U' ; break ; // P5
			case 0x5 : ch = 'D' ; break ; // P6
			case 0x3 : ch = 'S' ; break ; // P7
			default : break ;
			} 
	// Exit when Key Input is Valid
	if (KeyInValue != 0x7) break ;
	} 

	// Selected Function
	while(1) {
		switch(ch) {
		case 'U' : // P5
			if (SelCount == 0) SelCount = MAXTESTITEM ; 
			else SelCount-- ; 
			break; 
		case 'D' : // P6
			if (SelCount < MAXTESTITEM) SelCount++ ; 
			else SelCount = 0  ; 
			break; 
		case 'S' : // P7
			if (SelCount < MAXTESTITEM)
				{
				SelTestItemFunc[SelCount]() ; 
				LCDInit() ;
				break ;
				}
			else return ; 
		default : break ;
		}
		break ;

	} // End of While : Select Function

	} // End of While : Main Routine
}

// Initialize Test Item Selection Function
void InitTestItemSelFunction(int nItem , void (*TestItem)())
{
    	SelTestItemFunc[nItem] = TestItem;
}

void MemTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Memory Test ..") ;
	MemoryPartTest() ; 
	lPrint(" Finished.") ;
}

void CacheTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Cache Test ..") ;
	CacheTest() ; 
	lPrint(" Finished.") ;
}
void IntTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Interrupt Test ..") ;
	Print("\n Next Version Supported") ;
	//InterruptTest() ; 
	lPrint(" Finished.") ;
}

void DmaTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("DMA Test ..") ;
	GdmaTest() ; 
	lPrint(" Finished.") ;
}

void TimerTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Timer Test ..") ;
	TimerTest() ; 
	lPrint(" Finished.") ;
}

void MacTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("MAC Test ..") ;
	MacTest() ; 
	lPrint(" Finished.") ;
}

void HdlcTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("HDLC Test ..") ;
	HDLCTest() ; 
	lPrint(" Finished.") ;
}

void I2CTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("I2C Test ..") ;
	//Print("\n Next Version Supported") ;
	IICTest() ; 
	lPrint(" Finished.") ;
}

void UartTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("UART Test ..") ;
	UartTest() ; 
	lPrint(" Finished.") ;
}


void IoFuncDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("I/O Port & share function Test ..") ;
	ioptest(); 
	lPrint(" Finished.") ;
}


void EtcTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("ETC Test ..") ;
	//Print("\n Next Version Supported") ;
	EtcTest() ;
	lPrint(" Finished.") ;
}

void AllTestDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("ALL Test ..") ;

	// TIMER 0/1 Initialize
	tmReset(TIMER0) ; tmReset(TIMER1) ;
	ClrTimeVar() ;

	Rtc_init(TIMER0) ; Rtc_init(TIMER1) ;
        TimerStart(TIMER0); TimerStart(TIMER1);

	AllTestInternal(1) ;
	if  (AllTestResult != 0 ) 
		{
		Print("\n $$$ All Test Failed !!!!!") ;
		Print("\n $$$ Error Occured Count : %d ", AllTestResult) ;
		IOPDATA = 0x1F ;
		lPrint(" Fail.") ;
		}
	else
		{
		Print("\n $$$ All Test Passed !!!!!") ;
		lPrint(" Finished.") ;
		IOPDATA = 0x0A ;
		}

	AllTestResult = 0 ;

	ClrTimeVar() ;
	lPrint(" Finished.") ;
}

void DownloadDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Downloading ..") ;
	DownLoadPgm() ; 
	lPrint(" Finished.") ;
}

void FusingDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Fusing ..") ;
	FlashDown() ; 
	lPrint(" Finished.") ;
}

void BenchMarkDiag(void)
{
	LcdUsrModeSet() ;
	lPrint("Bench Marking ..") ;

/*
	SysSetInterrupt(nTIMER0_INT, tm0isr);
       	Enable_Int(nGLOBAL_INT); //enable global interrupt mask

        TDATA0 = 0xffffffff ;  // will be 1sec at fMCLK=33Mhz
        TCNT0 = 0x0;
        TMOD = TM0_OUT_1; // default interval, Tout high 
        IntMode |= (1<<nTIMER0_INT); // default FIQ mode
        IntMask &= ~TIMER0_INT; // Timer interrupt enable

	Dhrystone() ;

	lPrint(" Finished.") ;
*/
}

// Configuration View
void SysConfig(void)
{
	void *RO_Base 	= Image$$RO$$Base ;
	void *RO_End 	= Image$$RO$$Limit ;
	void *RW_Base 	= Image$$RW$$Base ;
	void *RW_End 	= Image$$RW$$Limit ;

	int *RO_BaseAddr, *RO_EndAddr, RO_Size ;
	int *RW_BaseAddr, *RW_EndAddr, RW_Size ;

	//int *ROM_
	LcdUsrModeSet() ;
	lPrint("System Configuration View ..") ;

	GetSizeOfRom() ;

	RO_BaseAddr = (int *)RO_Base ;
	RO_EndAddr = (int *)RO_End ;
	RO_Size = RO_EndAddr - RO_BaseAddr ;

	RW_BaseAddr = (int *)RW_Base ;
	RW_EndAddr = (int *)RW_End ;
	RW_Size = RW_EndAddr - RW_BaseAddr ;

        //CacheConfig(); // Cache & SRAM control stauts

	Print("\r===============================") ;
	Print("\r>> System Manager Status") ;
	Print("\r===============================") ;
	if (SPSTR & STALL_ENABLE) Print("\r   - Stall Enabled") ;
	else Print("\r   - Stall Disabled") ;

	if (SPSTR & CACHE_ENABLE) Print("\r   - Cache Enabled") ;
	else Print("\r   - Cache Disabled") ;

	if ((SPSTR & SRAM_CACHE) == 0x00 ) 
			Print("\r%6c4K SRAM, 4K CACHE",' ') ;
	else if ((SPSTR & SRAM_CACHE) == 0x10 ) 
			Print("\r%6c0K SRAM, 8K CACHE",' ') ;
	else if ((SPSTR & SRAM_CACHE) == 0x20 ) 
			Print("\r%6c8K SRAM, 0K CACHE",' ') ;
	else Print("\r%6cSRAM/CACHE Status : %x ",' ',(SPSTR & SRAM_CACHE) ) ; 

	if (SPSTR & WRITE_BUF_EN) Print("\r   - Write Buffer Enabled") ;
	else Print("\r   - Write Buffer Disabled") ;

	Print("\r   - SRAM Base Addr  : %8x",((SPSTR & 0xFFC0) << 10) ) ;
	Print("\r===============================") ;

	Print("\r>> Read Only Memory Used") ;
	Print("\r===============================") ;
	Print("\r   - Read Only Base  : %8x",RO_BaseAddr) ;
	Print("\r   - Read Only End   : %8x",RO_EndAddr) ;
	Print("\r   - Read Only Size  : %8d",RO_Size) ;
	Print("\r===============================") ;
	Print("\r>> Read/Write Memory Used") ;
	Print("\r===============================") ;
	Print("\r   - Read/Write Base : %8x",RW_BaseAddr) ;
	Print("\r   - Read/Write End  : %8x",RW_EndAddr) ;
	Print("\r   - Read/Write Size : %8d",RW_Size) ;
	Print("\r===============================") ;

}


void GetSizeOfRom(void)
{
	Print("\r===============================") ;
	Print("\r>> EEPROM Data Bus Size is ") ;
	if ( (EXTDBWTH & 0x3) == B0SIZE_BYTE ) 
			Print("BYTE") ;
	else if ( (EXTDBWTH & 0x3) == B0SIZE_SHORT ) 
			Print("SHORT") ;
	else if ( (EXTDBWTH & 0x3) == B0SIZE_WORD ) 
			Print("WORD") ;
}


void EtcTest0(void) 
{
	U32 *RomBase1, *DramBase1, *ExtIoBase2 ;
	Print("\n >> Etc Test <<") ;

	RomBase1 = (U32 *)ROMBasePtr1 ;
	DramBase1 = (U32 *)DRAMBasePtr1 ;
	ExtIoBase2 = (U32 *)EXTIOBasePtr2 ;

	Print("\n +- External Bank2 Read/Write : Press Any Key to Exit\r") ;
	Print("\r EXTACON1 : %x",EXTACON1) ;
	EXTACON1 = ETCOS2 | ETACS2 | ETCOH2 | ETACC2 ; 
	Print("\r EXTACON1 : %x\r",EXTACON1) ;

	do {
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
		*ExtIoBase2= *(ExtIoBase2+1) ;
	}while( !kbd_hit() ) ;

	Print("\n +- ROM Bank1 Read/Write : Press Any Key to Exit\r") ;
	do {
		*RomBase1 = *(RomBase1+1) ;
	}while( !kbd_hit() ) ;

	Print("\r DRAMCON0 : %x, DRAMCON1 : %x",DRAMCON0,DRAMCON1) ;
	Print("\r REFEXTCON : %x",REFEXTCON) ;
	Print("\n +- DRAM Bank1 Read/Write : Press Any Key to Exit\r") ;
	do {
		*DramBase1 = *(DramBase1+1) ;
	}while( !kbd_hit() ) ;

	Print("\n +- I/O Port 8 Write 1 ") ;
	IOPMOD = 0xF1F ;
	do {
		IOPDATA = 0xFF0 ;
	}while( !kbd_hit() ) ;

	Print("\n +- I/O Port 8 Write 0 ") ;
	do {
		IOPDATA = 0x001 ;
		IOPDATA = 0xF02 ;
		IOPDATA = 0x001 ;
		IOPDATA = 0xF02 ;
		IOPDATA = 0x001 ;
		IOPDATA = 0xF02 ;
		IOPDATA = 0x001 ;
		IOPDATA = 0xF02 ;
	}while( !kbd_hit() ) ;

}


