/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/ds2152.c                                          */
/*   DATE:    98/06/19                                                 */
/*   PURPOSE: ds2152(T1 single chip tranceiver) Source                 */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by hbahn                                            */
/***********************************************************************/
#include  "ks5000.h"
#include "sysconf.h"
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "iop.h"
#include "memory.h"
#include "system.h"
#include "hdlc.h"
#include "ds2152.h"
#include "kslib.h"

// Global variable
extern U32	gRxDB_B ;
extern volatile U32	gHdlcBRxDone ;
extern pHdlcTxStatus 	gBHdlcTxStatus ;
extern pHdlcRxStatus 	gBHdlcRxStatus ;

volatile U32	gHdlcBTxDone  = 0 ;
/*
 * Function : Ds2152Test
 * Description : HDLC Channel B and DS2152 Test
 */
void Ds2152Test(void) 
{
	char TestItemSel ;

	InitDs2152() ;


	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|        ***** DS2152 Test Function *****           |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.06.19)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| Transfer a Single Frame  to DS2152          - [T] |\r") ;
	Print("| Loopback Test With DS2152 Framer            - [L] |\r") ;
	Print("| Loopback External DS2152 (Loopback Cable)   - [E] |\r") ;
	Print("| Quit DS2152 Transfer Test                   - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'T' : case 't' : TxFrame2Ds2152() ; break ;
		case 'L' : case 'l' : LoopbackDs2152() ; break ;
		case 'E' : case 'e' : LoopbackExternalDs2152() ; break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	// ViewHdlcErrorRpt() ;
	IOPDATA = 0xF ;

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}

	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}

/*
 * Function : TxFrame2Ds2152
 * Description : Transfer a Frame to DS2152
 */
void TxFrame2Ds2152(void)
{
	U8 *TxDBB, *RxDBB ;
	char TransferPacket ;
	U32 DBSize = 0x100 ;

	// Enable DS2152 Framer Loopback (For Test)
	//CCR1 |= FLB ; // Frame Loopback
	//CCR5 |= LLB ; // Local Loop back

	// Transfer Frame Buffer Area
	TxDBB 	= (U8 *)0x1100000 ;

	do 
	{
	Print("\n >>> Transfer HDLC Frame to DS2152 (T or Q) ? " ) ;
	TransferPacket = get_byte() ;

	if ( (TransferPacket == 'T') || (TransferPacket == 't') )
		{

		FIMR = ITNF ;

		// Initialize Tx HDMA
		HDLCTxDataInit(TxDBB, DBSize) ; 
		HDMAB_Tx_init((U32)TxDBB, DBSize) ;
		PrintHdlcData(TxDBB, DBSize) ;
		RxDBB	= (U8 *)gRxDB_B ;
	
		// Transfer Enable & receive enable
		HDLCBTxGo() ;

		while(!gHdlcBTxDone); gHdlcBTxDone = 0 ;
		Print("  -- Done. ") ;

		while(!gHdlcBRxDone); gHdlcBRxDone = 0 ;
		PrintHdlcData(RxDBB, DBSize) ;
	
		// Compare loopback data
		if ( !CompareHdlcLoopbackData(TxDBB, RxDBB, DBSize) )
			Print("\r HDLC Channel B Loopback Test Fail !") ;
		else Print("\r HDLC Channel B Loopback Test Ok !!!!") ;

		FIMR = ITMEND ;
		}
	} while ( (TransferPacket != 'Q') && (TransferPacket != 'q') ) ;

}

/*
 * Function : LoopbackDs2152
 * Description : Loopback test
 */
int LoopbackDs2152(void) 
{
	U8 *TxDBB, *RxDBB ;
	U32 DBSize = 0x2000 ;
	int lcount = 20 ;

	// Enable DS2152 Framer Loopback
	CCR1 |= FLB ;
	Print("\n >>> Loopback Test With DS2152 Framer .....  " ) ;

	// Transfer Frame Buffer Area
	TxDBB 	= (U8 *)0x1100000 ;
	HDLCTxDataInit(TxDBB, DBSize) ; 

	if ( !HdlcBLoopback(TxDBB, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;
}


/*
 * Function : LoopbackExternalDs2152
 * Description : External DS2152 Loopback Test
 */
int LoopbackExternalDs2152(void) 
{
	U8 *TxDBB, *RxDBB ;
	U32 DBSize = 0x2000 ;
	int lcount = 20 ;

	Print("\n >>> Loopback External DS2152(Use Loopback Cable) ...  " ) ;

	// Transfer Frame Buffer Area
	TxDBB 	= (U8 *)0x1100000 ;
	HDLCTxDataInit(TxDBB, DBSize) ; 

	if ( !HdlcBLoopback(TxDBB, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;
}

/*
 * Function : InitDs2152
 * Description : Enhanced T1 Single Chip Tranceiver initialize
 */
void InitDs2152(void)
{
	HCON1B = F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
		BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_TXC | RxCLK_RXC ;

	ResetDs2152() ;
	Ds2152IntEnable() ;
}

/*
 * Function : ResetDs2152
 * Description : Reset DS2152
 */
void ResetDs2152(void)
{
	U8 *RegBase ;
	U32 i ;

	// Clear All Register
	RegBase = (U8 *)DS2152_Base ;
	for (i=0; i<0x90 ; i++) *RegBase++ = 0x00 ;

	// Reset Line Interface, BOC/HDLC & Elastore 
	CCR7 = LIRST ;
	FDLC = RBR | RHR | THR ;
	//CCR3 = ESR ;

	// Setup for FDL, BOC or HDLC
	TBOC = HBEN ;
	TFDL = 0x7E ;
	RFDL = 0x7E ;
	//FIMR = IRBOC | IRPE | IRPS | IRNE | ITNF | ITMEND ;

	// Setup for Transmit and Received side function
	RCR2 = RCS | RSIO ;
	TCR1 = TFPT | TCPT | TFDLS ;
	CCR2 = TFDL | RFDL ;
	//CCR2 = TFM | RFM | TB8ZS | RB8ZS | TFDL | RFDL ;

	// Line Interface Control Register
	LICR = L2 | L0 ; // -7.5dB
}

/*
 * Function : DispDs2152RegVal
 * Description : Display Ds2152 Register Value
 */
void DispDs2152RegVal(void)
{
	U8 *RegBase ;
	U32 i ;
	

	RegBase = (U8 *)DS2152_Base ;
	Print("\n >>> DS2152 Register display <<< \r") ;
	for (i=0; i<0x90 ; i++)
	{
		if (!(i%0x10) ) Print("\r %08x : ",RegBase) ;
		Print(" %02x ",*RegBase++) ;
	}
}

/*
 * Function : Ds2152IntEnable
 * Description : Enable Ds2152 Interrupt for External Interrupt #0
 */
void Ds2152IntEnable(void)
{
	// Setup Port8 to External Interrupt 0
	// Filter On, Falling-Edge, Active Low
	IOPMOD &= ~0x100 ;
	IOPCON &= ~0x1F ;
	IOPCON |= xIRQ0_ON | FILTER_ON | EDGE_FALL ;

	SysSetInterrupt(nHDLCB_INT, IsrForDs2152_isr );
	SysSetInterrupt(nEXT0_INT, Ds2152Int_isr);
        Enable_Int(nEXT0_INT); //enable EXT0 interrupt mask
	Enable_Int(nGLOBAL_INT); 
}

/*
 * Function : Ds2152Int_isr
 * Description : Ds2152 Interrupt Service Routine
 */
void Ds2152Int_isr(void)
{
     Clear_PendingBit(nEXT0_INT);
     Print("\r> External interrupt 0 !!!\r");
}

/*
 *  Function : IsrForDs2152_isr
 *  Description : HDMA Channel B Interrupt Service Routine
 */
void IsrForDs2152_isr(void)
{
	U32 IntHdlcStatus ;

	IntHdlcStatus = HSTATB ;
	HSTATB = IntHdlcStatus ;

	// Clear Interrupt Pending Bit
	Clear_PendingBit(nHDLCB_INT);

	// HDMA Rx Abort, but not Rx abort and Rx overrun
	if ( IntHdlcStatus & DRxABT )
	{
		HDMARXCNTB = HdmaRxSize_B ;
		HDLCBRxEnable() ; 
		gBHdlcRxStatus.DMARxABT++ ;
	}

	// HDMA Rx Stop, When Rx Done
	if ( IntHdlcStatus & DRxSTOP )
	{
		gBHdlcRxStatus.DMARxStop++ ;
		if ( !GetRxHdlcData_B(IntHdlcStatus) ) 
			{
			Print("\r HDLC Channel B Rx Fail !!! ") ;
			ViewHdlcErrorRpt() ;
			}
	}

	if ( IntHdlcStatus & ( TxFC | DTxSTOP) )
	{
		// HDLC Tx Complete
		if ( IntHdlcStatus & TxFC )
		{
			//HCON0B &= ~TxRTS ;
			gHdlcBTxDone  = 1 ;
		}
	} else if ( IntHdlcStatus & ( TxU | DTxABT ) )
	{
		// HDLC Tx Underrun Error
		if ( IntHdlcStatus & TxU )
		{
			gBHdlcTxStatus.TxUnderrun++ ;
		}
	}

}

