/**************************************************/
/*       Flash ROM Download and Program           */
/*    Programd by young-sun kim(at first 1998.4.9)*/
/**************************************************/
//#include <stdio.h>
#include <math.h>
#include "ks5000.h"
#include "flash.h"
#include "std.h"
#include "uart.h"
#include "pollio.h"
#include "down.h"

// Global variable for EEPROM Write address setting
volatile U32 gRomDstAddr ;

/*********************************************************************
 *  PROCEDURE : Program_SST29EE         
 *        
 *  This procedure can be used to write a total of 128bytes at one
 *  write cycle to the SST's 29EE512/29LE512/29VE512, 29EE010/29LE010
 *  /29VE010, 29EE020/29LE020/29VE020.
 *        
 *********************************************************************/


void Program_SST29EE(void *Src, void *Dst,int RcvDataSize)
{
	int RomPageSize;
	int i;
	pB0SIZE *RomSrc;
	pB0SIZE *RomDst; 

        RomSrc = (pB0SIZE *)Src;
        RomDst = (pB0SIZE *)Dst;
	gRomDstAddr = (U32)Dst ;

	SST29EE_DisableProtection() ;
	SST29EE_EraseEEPROM();
		
	//RomPageSize = (int)((float)RcvDataSize/(float)ROW_SIZE_29EE) + 1 ;
	RomPageSize = (int)ceil((float)RcvDataSize/(float)ROW_SIZE_29EE);

	Print("\rProgramming EEPROM 0x%08x to 0x%08x ..... ",RomSrc,RomDst);
	for (i=0 ; i<RomPageSize ; i++)
	{
		SST29EE_WriteEEPROM(RomSrc,RomDst);
		RomSrc += ROW_SIZE;
		RomDst += ROW_SIZE;
	}
	Print(" Finished.") ;

	SST29EE_EnableProtection() ;
}


void Program_SST29SF(void *Src, void *Dst,int RcvDataSize)
{
	pB0SIZE *SourceBuf;
	pB0SIZE *DestBuf; 
        int RomPageSize;
        int i;

        SourceBuf = (pB0SIZE *)Src;
        DestBuf =   (pB0SIZE *)Dst;

        /* CHECK SST ID FOR 28SF040 */
        SST29SF_EnableProtection(); 
        if(Check_SST_28SF040()) Print("\rID = SST 28SF040 EEPROM!!.");
        else {
                Print("\rNot Available EEPROM type.");
                return;
        }

          
	/* ERASE OPERATION */
	SST29SF_DisableProtection() ;
        //RomPageSize = (int)ceil((float)SST29SF_SIZE/(float)ROW_SIZE_29SF) ;
        RomPageSize = (int)ceil((float)RcvDataSize/(float)ROW_SIZE_29SF);
	Print("\rErasing EEPROM ..... ") ;

	for(i=0; i < RomPageSize; i++) { 
             if(!SST29SF_EraseEEPROM(DestBuf)) {
                 //Print("Fail.");
                 //return;   Return with error 
             }

             DestBuf += ROW_SIZE_29SF;
        }
	Print("Finished.") ;

       /* PROGRAM OPERATION */ 
        SourceBuf = (pB0SIZE *)Src;
        DestBuf =   (pB0SIZE *)Dst;

	/* Write EEPROM Data */
        Print("\rProgramming EEPROM 0x%08x to 0x%08x ....",SourceBuf,DestBuf);

	for (i=0 ; i<RomPageSize ; i++)
	{
		if(!SST29SF_WriteEEPROM(SourceBuf,DestBuf,RcvDataSize)) {
		    Print("Fail.");
		    //return; /* return with error */
		}
		SourceBuf += ROW_SIZE_29SF;
		DestBuf += ROW_SIZE_29SF;
	}
	Print("Finished.") ;

        SST29SF_EnableProtection(); 
}

/*********************************************************************
 *  PROCEDURE : SST29EE_DisableProtection()
 *        
 *  This procedure DISABLES the data protection feature. After calling       
 *  this routine, the chip can be written without any additional commands.        
 *        
 *********************************************************************/

void SST29EE_DisableProtection(void)
{
	pB0SIZE *Temp ;

	Print("\rDisable EEPROM Data Protection ..... ") ;

	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xAAAAAAAA ;	;	
	Temp =  (pB0SIZE *)( (0x2AAA << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x55555555 ; 	
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x80808080  ; 		
	
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xAAAAAAAA  ;	
	Temp =  (pB0SIZE *)( (0x2AAA << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x55555555  ; 	
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x20202020  ; 	

	Delay_Milli_Second(50000) ;
	Print("Finished.") ;
}


void SST29SF_DisableProtection()
{
	pB0SIZE *Temp ;
	pB0SIZE TempBuff;

	Print("\rDisable EEPROM Data Protection ..... ") ;

	Temp = (pB0SIZE *)(0x00001823) ; 
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00001820) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00001822) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00000418) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x0000041B) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00000419) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x0000041A) ;
	TempBuff = *Temp;	        

	Print("Finished.") ;
}
/*********************************************************************
 *  PROCEDURE : SST29EE_EraseEEPROM();
 *        
 *  Erase EEPROM operation        
 *        
 *********************************************************************/
void SST29EE_EraseEEPROM(void)
{
	pB0SIZE *Temp ;

	Print("\rErasing EEPROM ..... ") ;

	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE) 0xAAAAAAAA  ;			
	Temp =  (pB0SIZE *)( (0x2AAA << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x55555555  ; 	
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x80808080  ; 

	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xAAAAAAAA  ;			
	Temp =  (pB0SIZE *)( (0x2AAA << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x55555555  ; 	
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x10101010  ; 

	// Waiting for 20 ms
	Delay_Milli_Second(1000000) ;
	Print("Finished.") ;
}

int SST29SF_EraseEEPROM(pB0SIZE *Dst)
{
       pB0SIZE *DestBuf;
       pB0SIZE Data;
       pB0SIZE SourceData;
       int Count;
       int Continue;

       DestBuf = Dst; 
       Count = 0;
       Continue = TRUE;
       Data = (pB0SIZE)0xFFFFFFFF;      

       *DestBuf = (pB0SIZE)AUTO_PG_ERASE1;
       *DestBuf = (pB0SIZE)AUTO_PG_ERASE2;
       Delay_Milli_Second(1000000) ;
       //Check_Toggle_Ready(DestBuf);

       
       while((Count<ROW_SIZE_29SF) && (Continue)) {
            SourceData = *DestBuf++;
            if(SourceData == Data) Count++;
            else Continue = FALSE;
       }

       if(!Continue) return(Continue);  /* Error return */
}
/*********************************************************************
 *  PROCEDURE : SST29EE_WriteEEPROM(RomSrc,RomDst);
 * 
 * Write a total of 128 bytes at once write cycle          
 * Src : SOURCE Address containing the data                
 * Dst : DESTINATION Address which will be writen with the 
 *       data passed in from ds:si            
 *
 *********************************************************************/

void SST29EE_WriteEEPROM(pB0SIZE *Src, pB0SIZE *Dst) 
{
	pB0SIZE *Temp ;
	pB0SIZE *SourceBuf ;
	pB0SIZE *DestBuf ;
	unsigned int index ;

	SourceBuf = (pB0SIZE *)Src;
	DestBuf   = (pB0SIZE *)Dst ;
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xAAAAAAAA;		
	Temp =  (pB0SIZE *)( (0x2AAA << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x55555555; 		
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xA0A0A0A0; 		

	//for (index=0; index<ROW_SIZE_29EE; index++)
	for (index=0; index<ROW_SIZE; index++)
		*DestBuf++ = *SourceBuf++ ;

	Delay_Milli_Second(50000) ;

	Check_Toggle_Ready(Dst) ;
}


int SST29SF_WriteEEPROM(pB0SIZE *Src, pB0SIZE *Dst, int RcvDataSize) 
{
      pB0SIZE *SourceBuf;
      pB0SIZE *DestBuf;
      pB0SIZE SourceData;
      pB0SIZE ProgramedData;
      pB0SIZE Data;
      int Index;
      int PgmSize;

      SourceBuf = Src;
      DestBuf = Dst;
      Data = (pB0SIZE)0xFFFFFFFF; 

      if(RcvDataSize < ROW_SIZE_29SF) PgmSize = RcvDataSize+1;
      else PgmSize = ROW_SIZE_29SF; 
          

       // Write EEPROM Data
      for(Index = 0;Index<PgmSize;Index++)
      {
          SourceData = *SourceBuf++;
          *DestBuf = AUTO_PGRM;        /*issue AUTO PROGRAM command*/
          *DestBuf = SourceData;       /*program the data*/
          //Check_Toggle_Ready(DestBuf); /*wait for toggle bit ready*/
	  Delay_Milli_Second(10000);   /*Byte program cycle time 30us*/
          ProgramedData = *DestBuf++;
          if(SourceData != ProgramedData) {
             return(FALSE);  /* return with error */
          }
      }
}

/*********************************************************************
 *  PROCEDURE : SST29EE_EnableProtection() ;
 *        
 * After calling this routine, the chip cannot be written  
 * unless preceded by the three Byte-Load sequence.       
 *        
 *********************************************************************/
void SST29EE_EnableProtection()
{
	pB0SIZE *Temp ;

	Print("\rEnable EEPROM Data Protection ..... ") ;

	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xAAAAAAAA  ;
	Temp =  (pB0SIZE *)( (0x2AAA << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0x55555555;	
	Temp =  (pB0SIZE *)( (0x5555 << SST_ALIGN) | gRomDstAddr ) ;
	*Temp = (pB0SIZE)0xA0A0A0A0;	

	Delay_Milli_Second(50000) ;
	Print("Finished.") ;
}


void SST29SF_EnableProtection()
{
	pB0SIZE *Temp ;
	pB0SIZE TempBuff;

	Print("\rEnable EEPROM Data Protection ..... ") ;

	Temp = (pB0SIZE *)(0x00001823) ; 
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00001820) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00001822) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00000418) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x0000041B) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x00000419) ;
	TempBuff = *Temp;	        
	Temp = (pB0SIZE *)(0x0000040A) ;
	TempBuff = *Temp;	        
         
	Print("Finished.") ;
}
/*********************************************************************
 *  PROCEDURE : Check_Toggle_Ready(void *Dst)
 * 
 * During the internal write cycle, any consecutive read   
 * operation on DQ6 will produce alternating 0's and 1's.  
 * When the Write cycle is completed, DQ6 of the data will 
 * stop toggling After the DQ6 data bit stops toggling,    
 * the device is ready for next operation                  
 *
 *********************************************************************/

void Check_Toggle_Ready(pB0SIZE *Dst)
{
     int Loop = TRUE ;
     pB0SIZE  PreData ;
     pB0SIZE  CurrData ;
     unsigned long TimeOut = 0 ;

     PreData = *Dst;
     PreData = PreData & (pB0SIZE)0x40404040  ;


     while((TimeOut<0x07FFFFFF) && Loop) {
	    CurrData = *Dst;
            CurrData = CurrData & (pB0SIZE)0x40404040;
            if (PreData == CurrData) Loop = FALSE;
            PreData = CurrData;
            TimeOut++;
     }
}


/*********************************************************************
 *  PROCEDURE : Check_Data_Polling
 * 
 * Durring the internal write cycle, any attemp to read DQ7 of the byte
 * loading during the page/byte load cycle will receive the complement
 * of the data. Once the write cycle completed, DQ7 show true data.
 * TrueData : this is the original (true) data
 *
 *********************************************************************/

void Check_Data_Polling(pB0SIZE *Dst, pB0SIZE TrueData)
{
	int Loop = TRUE ;
	pB0SIZE CurrData ;
	unsigned int TimeOut = 0 ;

	TrueData = TrueData & (pB0SIZE)0x80808080  ;
	
	while (( TimeOut < 0x07FFFFFF) && Loop)
	{
		CurrData = *Dst ;
		CurrData = CurrData & (pB0SIZE)0x80808080  ;
		if (TrueData == CurrData) Loop = FALSE ;
		TimeOut++ ; 
        }
}


/*********************************************************************
 *  PROCEDURE : Check_SST_28SF040
 * 
 *  This procedure decides whether a physical hardware device has a 
 *  SST 28SF040 4 Mbit SuperFlash EEPROM installed or not...
 *
 *  Input: None
 *
 *  Output: 
 *          return  0 : indicates no SST 28SF040 installed
 *          return  1 : indicates a SST 28SF040 is installed
 *********************************************************************/

int Check_SST_28SF040(void)
{
      pB0SIZE *Temp;
      int SST_id1;
      int SST_id2;
      int ReturnStatus;
      pB0SIZE OriginalByte;

      Temp = (pB0SIZE *)(0x00000000);
      OriginalByte = *Temp;
      *Temp = RESET;
      *Temp = READ_ID;
      Delay_Milli_Second(130); 

      Temp = (pB0SIZE *)(0x00000000);
      SST_id1 = (int)*Temp;
      Temp = (pB0SIZE *)(0x00000001);
      SST_id2 = (int)*Temp;

      if((SST_id1 == (int)SST_ID) && (SST_id2 == (int)SST_28SF040))
            ReturnStatus = 1;
      else
            ReturnStatus = 0;

      Temp = (pB0SIZE *)(0x00000000);
      *Temp = RESET;

      if(ReturnStatus == 0) *Temp = OriginalByte;

      return(ReturnStatus);
}


/*********************************************************************
 *  PROCEDURE : Delay_Milli_Second
 *********************************************************************/
void Delay_Milli_Second(int count) 
{
	int i ;
	for (i=0; i<count ; i++) ;
}

