/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/gcsu.c                                            */
/*   DATE:    98/06/19                                                 */
/*   PURPOSE: GCSU Interface Source                                    */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by hbahn                                            */
/***********************************************************************/
#include  "ks5000.h"
#include "sysconf.h"
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "iop.h"
#include "memory.h"
#include "system.h"
#include "hdlc.h"
#include "gcsu.h"
#include "kslib.h"

// Global variable
extern U32	gRxDB_A ;
extern volatile U32	gHdlcARxDone ;
extern pHdlcTxStatus 	gAHdlcTxStatus ;
extern pHdlcRxStatus 	gAHdlcRxStatus ;

extern volatile U32	gHdlcATxDone  ;

/*
 * Function : GCSUTest
 * Description : GCSU1000 T1 Interface Test Program
 */
void GCSUTest(void) 
{
	char TestItemSel ;

	InitDs2152() ;
	//HCON0A &= ~TxRTS ;
	//HCON0A &= ~TxDTR ;
	// Set Baud Rate
	HBRGTCA	= fMCLK / (2 * 64000) - 1   ;
	HCON1A = F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
		BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_RXC | TxOCLK_BRG1  ;

		//BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_TXC | RxCLK_RXC ;

	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|          ***** GCSU Test Function *****           |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.06.24)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| Transfer a Single Frame  to GCSU 1000       - [T] |\r") ;
	Print("| Loopback Test With GCSU 1000                - [L] |\r") ;
	Print("| Quit GCSU Test                              - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'T' : case 't' : TxFrame2GCSU() ; break ;
		case 'L' : case 'l' : LoopbackGCSU() ; break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	ViewHdlcErrorRpt() ;
	IOPDATA = 0xF ;

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}

	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}

/*
 * Function : TxFrame2GCSU
 * Description : Transfer a Frame to GCSU 1000
 */
void TxFrame2GCSU(void)
{
	U8 *TxDBA, *RxDBA ;
	char TransferPacket ;
	U32 DBSize = 0x100 ;
	int i = 0 ;

	// Transfer Frame Buffer Area
	TxDBA 	= (U8 *)0x1100000 ;

	do 
	{
	Print("\n >>> Transfer HDLC Frame to GCSU (T or Q) ? " ) ;
	TransferPacket = get_byte() ;

	if ( (TransferPacket == 'T') || (TransferPacket == 't') )
		{

		// Initialize Tx HDMA
		HDLCTxDataInit(TxDBA, DBSize) ; 
		HDMAA_Tx_init((U32)TxDBA, DBSize) ;
		//PrintHdlcData(TxDBA, DBSize) ;
		RxDBA	= (U8 *)gRxDB_A ;
	
		// Transfer Enable 
		HDLCATxGo() ;

		//ViewHdlcErrorRpt() ;
		while(!gHdlcATxDone); gHdlcATxDone = 0 ;
		Print("  -- Done. ") ;
		ViewHdlcErrorRpt() ;


/*
		while(!gHdlcARxDone); gHdlcARxDone = 0 ;
		PrintHdlcData(RxDBA, DBSize) ;
	
		// Compare loopback data
		if ( !CompareHdlcLoopbackData(TxDBA, RxDBA, DBSize) )
			Print("\r HDLC Channel A Loopback Test Fail !") ;
		else Print("\r HDLC Channel A Loopback Test Ok !!!!") ;
*/
	
		}
	} while ( (TransferPacket != 'Q') && (TransferPacket != 'q') ) ;

}

/*
 * Function : LoopbackGCSU
 * Description : Loopback test
 */
int LoopbackGCSU(void) 
{
	U8 *TxDBA, *RxDBA ;
	U32 DBSize = 0x1000 ;
	int lcount = 20 ;

	Print("\n >>> Loopback Test With GCSU 1000 .....  " ) ;

	// Transfer Frame Buffer Area
	TxDBA 	= (U8 *)0x1100000 ;
	HDLCTxDataInit(TxDBA, DBSize) ; 

	if ( !HdlcBLoopback(TxDBA, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;
}
