/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/hdlc.c                                            */
/*   DATE:    98/02/12                                                 */
/*   PURPOSE: hdlc(high level data link control) function define.      */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by hbahn                                            */
/*                                                                     */
/***********************************************************************/
#include  "ks5000.h"
#include "sysconf.h"
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "memory.h"
#include "system.h"
#include "hdlc.h"
#include "ds2152.h"
#include "gcsu.h"
#include "kslib.h"

extern U32 gRxDB_A ;
extern volatile U32 gHdlcARxDone, gHdlcBRxDone ;

/*
 * Function : HDLCTest
 * Description : HDLC Test Main Function
 */
void HDLCTest() 
{
	char TestItemSel ;


	// Initialize HDLC channel A,B
	HDLCA_init() ;
	HDLCB_init() ;
	//InitDs2152() ;

	// Select Test Item
	do {

	// All LED On
	IOPDATA = 0x0 ; 

	Print("\n+---------------------------------------------------+\r") ;
	Print("|   >>>>> HDLC Block Diagonistic Program <<<<<      |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.06.15)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| MAC Loopback Test                           - [L] |\r") ;
	Print("| Transfer HDLC Frame                         - [T] |\r") ;
	Print("| Receive HDLC Frame                          - [R] |\r") ;
	Print("| Change & View HDLC Configuration            - [V] |\r") ;
	Print("| DS2152 Test                                 - [D] |\r") ;
	Print("| GCSU-1000 Test                              - [G] |\r") ;
	Print("| Quit HDLC Test                              - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;

	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'L' : case 'l' : HdlcLoopbackTest() ; break ;
		case 'T' : case 't' : HdlcFrameTx() ; break ;
		case 'R' : case 'r' : break ;
		case 'V' : case 'v' : ViewHdlcErrorRpt() ; break ;
		case 'D' : case 'd' : Ds2152Test() ; break ;
		case 'C' : case 'c' : GCSUTest() ; break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	// LED Off
	IOPDATA = 0x0F ; 
	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue HDLC Test") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
	
}

/*
 * Function : HdlcLoopbackTest
 * Description : HDLC Loopback Test Function
 */
void HdlcLoopbackTest(void) 
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|         ***** HDLC LoopBack Test *****            |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.06.15)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| HDLC Internal LoopBackTest                  - [I] |\r") ;
	Print("| HDLC Channel-A External LoopBackTest        - [A] |\r") ;
	Print("| HDLC Channel-B External LoopBackTest        - [B] |\r") ;
	Print("| HDLC Channel A and B External LoopBackTest  - [E] |\r") ;
	Print("| All LoopBackTest                            - [L] |\r") ;
	Print("| Quit HDLC LoopBack Test                     - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'I' : case 'i' : HdlcInternalLoopback() ; break ;
		case 'A' : case 'a' : HdlcAExternLoopback() ; break ;
		case 'B' : case 'b' : HdlcBExternLoopback() ; break ;
		case 'E' : case 'e' : HdlcABExternLoopback() ; break ;
		case 'L' : case 'l' : break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	ViewHdlcErrorRpt() ;

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}

	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}


/*
 * Function : HdlcFrameTx
 * Description : HDLC Frame Transfer
 */
void HdlcFrameTx(void) 
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|        ***** HDLC Packet Transfer *****           |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.06.19)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| HDLC Channel-A Transfe by Single Mode       - [a] |\r") ;
	Print("| HDLC Channel-B Transfe by Single Mode       - [b] |\r") ;
	Print("| HDLC Channel-A Transfe by DMA Mode          - [A] |\r") ;
	Print("| HDLC Channel-B Transfe by DMA Mode          - [B] |\r") ;
	Print("| HDLC Channel-A Transfe to CSU               - [C] |\r") ;
	Print("| HDLC Channel-B Transfe to DS2152            - [D] |\r") ;
	Print("| Quit HDLC Transfer Test                     - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'a' : break ;
		case 'b' : break ;
		case 'A' : break ;
		case 'B' : break ;
		case 'C' : case 'c' : break ;
		case 'D' : case 'd' : TxFrame2Ds2152() ; break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	ViewHdlcErrorRpt() ;

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}

	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}
