/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/hdlc.h                                            */
/*   DATE:    98/02/05                                                 */
/*   PURPOSE: hdlc(high level data link control) header                */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by hbahn                                            */
/***********************************************************************/
#ifndef _HDLC_H
#define _HDLC_H

#define	HdlcReturnOk	1 
#define	HdlcReturnFail 	0

#define HdmaRxSize_A 	512
#define HdmaRxSize_B 	512 
#define RxDBA_Start 	0x5200000 
#define RxDBB_Start 	0x5210000 
#define RxDBA_Limit 	0x5210000 
#define RxDBB_Limit 	0x5220000 

#define	HDLC_BAUDRATE	1544000
#define	HDLC_PREAMBLE	0xAA

// HDLC Control Register 0 (HCON0) 
#define    TxRS            0x00000001            // bit 0  
#define    RxRS            0x00000002            // bit 1  
#define    DTxRS           0x00000004            // bit 2  
#define    DRxRS           0x00000008            // bit 3  
#define    BRGEN           0x00000020            // bit 5  
#define    DPLLEN          0x00000040            // bit 6  
#define    TxEN            0x00000100            // bit 8  
#define    RxEN            0x00000200            // bit 9  
#define    DTxEN           0x00000400            // bit 10  
#define    DRxEN           0x00000800            // bit 11  
#define    TxWA            0x00003000            // bit 12:13  
#define    TxWA_MASK       0xFFFFCFFF            // bit 12:13  
#define    TxWA1           0x00001000            // bit 12:13  
#define    TxWA2           0x00002000            // bit 12:13  
#define    TxWA3           0x00003000            // bit 12:13  
#define    RxWA            0x0000C000            // bit 14:15  
#define    RxWA_MASK       0xFFFF3FFF            // bit 14:15  
#define    RxWA1           0x00004000            // bit 14:15  
#define    RxWA2           0x00008000            // bit 14:15  
#define    RxWA3           0x0000C000            // bit 14:15  
#define    TxSTCL          0x00010000            // bit 16  
#define    RxSTCL          0x00020000            // bit 17  
#define    DPLLTMCL        0x00040000            // bit 18  
#define    DPLLOMCL        0x00080000            // bit 19  
#define    TxABT           0x01000000            // bit 24  
#define    TxRTS           0x02000000            // bit 25  
#define    TxDTR           0x04000000            // bit 26  
#define    RxDISCON        0x08000000            // bit 27  
#define    TxNOCRC         0x10000000            // bit 28  
#define    RxNOCRC         0x20000000            // bit 29  


// HDLC Control Register 1 (HCON1)

#define    F4WM            0x00000001            // bit 0  
#define    TxLittle        0x00000002            // bit 1  
#define    RxLittle        0x00000004            // bit 2  
#define    DTxMADEC        0x00000008            // bit 3  
#define    DRxMADEC        0x00000010            // bit 4  
#define    DTxMAINC        0x00000000            // bit 3  
#define    DRxMAINC        0x00000000            // bit 4  
#define    DRxSTOPTC       0x00000020            // bit 5  
#define    TxFLAGIDLE      0x00000040            // bit 6  
#define    TxPREAMBLE      0x00000080            // bit 7  
#define    TxSDFL          0x00000100            // bit 8  
#define    TxABTEXT        0x00000200            // bit 9  
#define    TxLOOP          0x00000400            // bit 10  
#define    RxASEN          0x00000800            // bit 11  
#define    RxAEXT          0x00001000            // bit 12  
#define    RxECHO          0x00002000            // bit 13  
#define    RxFDSEN         0x00004000            // bit 14  
#define    DF              0x00070000            // bit 16:17:18  
#define    DF_MASK         0xFFF8FFFF            // bit 16:17:18  
#define    DF_NRZ          0x00000000            // bit 16:17:18  
#define    DF_NRZI         0x00010000            // bit 16:17:18  
#define    DF_FM0          0x00020000            // bit 16:17:18  
#define    DF_FM1          0x00030000            // bit 16:17:18  
#define    DF_Manchester   0x00040000            // bit 16:17:18  
#define    BRGCLK_MASK     0xFFFEFFFF
#define    BRGCLK_MCLK     0x00080000            // bit 19  
#define    BRGCLK_RXC      0x00000000
#define    DPLLCLK         0x00700000            // bit 20:21:22  
#define    DPLLCLK_MASK    0xFF8FFFFF            // bit 20:21:22  
#define    DPLLCLK_TXC     0x00000000            // bit 20:21:22  
#define    DPLLCLK_RXC     0x00100000            // bit 20:21:22  
#define    DPLLCLK_MCLK    0x00200000            // bit 20:21:22  
#define    DPLLCLK_BRG1    0x00300000            // bit 20:21:22  
#define    DPLLCLK_BRG2    0x00400000            // bit 20:21:22  
#define    DPLLCLK_BRG3    0x00500000            // bit 20:21:22  
#define    TxCLK           0x03800000            // bit 23:24:25  
#define    TxCLK_MASK      0xFC7FFFFF            // bit 23:24:25  
#define    TxCLK_TXC       0x00000000            // bit 23:24:25  
#define    TxCLK_RXC       0x00800000            // bit 23:24:25  
#define    TxCLK_DPLL      0x01000000            // bit 23:24:25  
#define    TxCLK_BRG1      0x01800000            // bit 23:24:25  
#define    TxCLK_BRG2      0x02000000            // bit 23:24:25  
#define    TxCLK_BRG3      0x02800000            // bit 23:24:25  
#define    RxCLK           0x1C000000            // bit 26:27:28  
#define    RxCLK_MASK      0xE3FFFFFF            // bit 26:27:28  
#define    RxCLK_TXC       0x00000000            // bit 26:27:28  
#define    RxCLK_RXC       0x04000000            // bit 26:27:28  
#define    RxCLK_DPLL      0x08000000            // bit 26:27:28  
#define    RxCLK_BRG1      0x0C000000            // bit 26:27:28  
#define    RxCLK_BRG2      0x10000000            // bit 26:27:28  
#define    RxCLK_BRG3      0x14000000            // bit 26:27:28  
#define    TxOCLK_TXC      0x00000000            // bit 31:30:29  
#define    TxOCLK_RXC      0x20000000            // bit 31:30:29  
#define    TxOCLK_BRG1     0x40000000            // bit 31:30:29  
#define    TxOCLK_BRG2     0x60000000            // bit 31:30:29  
#define    TxOCLK_BRG3     0x80000000            // bit 31:30:29  


// HDLC Status Register (HSTAT)
#define    RxRB            0x0000000F            // bit3 - bit0  
#define    TxFC            0x00000100            // bit8  
#define    TxFA            0x00000200            // bit9  
#define    TxCTS           0x00000400            // bit10  
#define    TxSCTS          0x00000800            // bit11  
#define    TxU             0x00001000            // bit12  
#define    RxFA            0x00002000            // bit13  
#define    RxFAP           0x00004000            // bit14  
#define    RxFD            0x00008000            // bit15  
#define    RxDCD           0x00010000            // bit16  
#define    RxSDCD          0x00020000            // bit17  
#define    RxAERR          0x00040000            // bit18  
#define    RxFV            0x00080000            // bit19  
#define    RxIDLE          0x00100000            // bit20  
#define    RxABT           0x00200000            // bit21  
#define    RxFERR          0x00400000            // bit22  
#define    RxOV            0x00800000            // bit23  
#define    DTxSTOP         0x01000000            // bit24  
#define    DTxABT          0x02000000            // bit25  
#define    DRxSTOP         0x04000000            // bit26  
#define    DRxABT          0x08000000            // bit27  
#define    DPLLTM          0x10000000            // bit28  
#define    DPLLOM          0x20000000            // bit29  


// HDLC Interrupt Enable Register (HINTEN)
#define    HDLCIE          0x00000001            // bit0  
#define    TxIE            0x00000002            // bit1  
#define    RxIE            0x00000004            // bit2  
#define    DTxIE           0x00000008            // bit3  
#define    DRxIE           0x00000010            // bit4  
#define    DPLLIE          0x00000020            // bit5  
#define    TxFCIE          0x00000100            // bit8  
#define    TxFAIE          0x00000200            // bit9  
#define    TxCTSIE         0x00000800            // bit11  
#define    TxUIE           0x00001000            // bit12  
#define    RxFAIE          0x00002000            // bit13  
#define    RxFAPIE         0x00004000            // bit14  
#define    RxFDIE          0x00008000            // bit15  
#define    RxDCDIE         0x00020000            // bit17  
#define    RxAERRIE        0x00040000            // bit18  
#define    RxFVIE          0x00080000            // bit19  
#define    RxIDLEIE        0x00100000            // bit20  
#define    RxABIE          0x00200000            // bit21  
#define    RxFERRIE        0x00400000            // bit22  
#define    RxOVIE          0x00800000            // bit23  
#define    DTxSTOPIE       0x01000000            // bit24  
#define    DTxABIE         0x02000000            // bit25  
#define    DRxSTOPIE       0x04000000            // bit26  
#define    DRxABIE         0x08000000            // bit27  
#define    DPLLTMIE        0x10000000            // bit28  
#define    DPLLOMIE        0x20000000            // bit29  

// HDLC Tx Status Structure
typedef struct HdlcTxStatus {
	U32 TxFrameComplete ;
	U32 TxFIFOAvailable ;
	U32 TxLevelOfCTS ;
	U32 TxTransitionOfCTS ;
	U32 TxUnderrun ;
	U32 DMATxStop ;
	U32 DMATxABT ;
} pHdlcTxStatus ;

// HDLC Rx Status Structure
typedef struct HdlcRxStatus {
	U32 RxFIFOAvalable ;
	U32 RxFIFOAddrPresent ;
	U32 RxFlagDetected ;
	U32 RxLevelOfDCD ;
	U32 RxTransitionOfDCD ;
	U32 RxAddressError ;
	U32 RxLastFrameValid ;
	U32 RxIdle ;
	U32 RxAbort ;
	U32 RxFrameError ;
	U32 RxOverrun ;
	U32 DMARxStop ;
	U32 DMARxABT ;
} pHdlcRxStatus ;

// HDLC DPLL Status Structure
typedef struct HdlcDPLLStatus {
	U32 DPLLOneClkMiss ;
	U32 DPLLTwoClkMiss ;
} pHdlcDPLLStatus ;


/*------------------------------------
 * Function Prototype for HDLC.c
 *------------------------------------*/
extern void HDLCTest(void)  ;
/*
 * Function : HDLCTest
 * Description : HDLC Test Main Function
 */

extern void HdlcLoopbackTest(void) ;
/*
 * Function : HdlcLoopbackTest
 * Description : HDLC Loopback Test Function
 */

extern void HdlcFrameTx(void) ;
/*
 * Function : HdlcFrameTx
 * Description : HDLC Frame Transfer
 */

/*------------------------------------
 * Function Prototype for HDLCinit.c
 *------------------------------------*/
extern void HDLCA_init(void) ;
/*
 *  Function : HDLCA_init
 *  Description : HDLC Channel A controller initialize
 */

extern void HDLCB_init(void) ;
/*
 *  Function : HDLCB_init
 *  Description : HDLC Channel B controller initialize
 */

extern void HDMAA_Tx_init(U32 TxDB_A, U32 TxD_Size_A) ;
/*
 *  Function : HDMAA_Tx_init
 *  Description : HDMA Channel A Tx initialize
 */

extern void HDMAA_Rx_init(void) ;
/*
 *  Function : HDMAA_Rx_init
 *  Description : HDMA Channel A Rx initialize
 */

extern void HDMAB_Tx_init(U32 TxDB_B, U32 TxD_Size_B) ;
/*
 *  Function : HDMAB_Tx_init
 *  Description : HDMA Channel B Tx initialize
 */

extern void HDMAB_Rx_init(void) ;
/*
 *  Function : HDMAB_Rx_init
 *  Description : HDMA Channel B Rx initialize
 */

extern void HDLCA_isr(void) ;
/*
 *  Function : HDLCA_isr
 *  Description : HDMA Channel A Interrupt Service Routine
 */

extern void HDLCB_isr(void) ;
/*
 *  Function : HDLCB_isr
 *  Description : HDMA Channel B Interrupt Service Routine
 */

/*------------------------------------
 * Function Prototype for HDLClib.c
 *------------------------------------*/
extern void HDLCATxGo(void) ;
/*
 * Function : HDLCATxGo
 * Description : HDLC Channel A Tx Enable
 */

extern void HDLCARxEnable(void) ;
/*
 * Function : HDLCARxEnable
 * Description : HDLC Channel A Rx Enable
 */

extern void HDLCBTxGo(void) ;
/*
 * Function : HDLCBTxGo
 * Description : HDLC Channel B Tx Enable
 */

extern void HDLCBRxEnable(void) ;
/*
 * Function : HDLCBRxEnable
 * Description : HDLC Channel B Rx Enable
 */

extern void HDLCALoopbackEn(void) ;
/*
 * Function : HDLCALoopbackEn
 * Description : HDLC Channel A Internal Loopback Test Enable
 */

extern void HDLCALoopbackDis(void) ;
/*
 * Function : HDLCALoopbackDis
 * Description : HDLC Channel A Internal Loopback Test Disable
 */

extern void HDLCBLoopbackEn(void) ;
/*
 * Function : HDLCBLoopbackEn
 * Description : HDLC Channel B Internal Loopback Test Enable
 */

extern void HDLCBLoopbackDis(void) ;
/*
 * Function : HDLCBLoopbackDis
 * Description : HDLC Channel B Internal Loopback Test Disable
 */

extern int GetRxHdlcData_A(U32 IntHdlcStatus) ;
/* 
 * Function : GetRxHdlcData_A
 * Description : HDLC Channel A Data Receive
 */

extern int GetRxHdlcData_B(U32 IntHdlcStatus) ;
/* 
 * Function : GetRxHdlcData_B
 * Description : HDLC Channel B Data Receive
 */

extern void HDLCTxDataInit(U8 *TxDB, U32 TxD_Size) ;
/*
 * Function : HDLCTxDataInit
 * Description : HDLC Tx Data Initialize
 */

extern void PrintHdlcData(U8 *DBPtr, U32 Size) ;
/*
 * Function : PrintHdlcData
 * Description : Print HDLC Tx/Rx Data 
 */

extern int CompareHdlcLoopbackData(U8 *TxDB, U8 *RxDB, U32 Size) ;
/*
 * Function : CompareHdlcLoopbackData
 * Description : Compare Loopback test data
 * Return : Compare Ok or Fail
 */

extern int HdlcInternalLoopback(void) ;
/*
 * Function : HdlcInternalLoopback
 * Description : HDLC Internal Loopback Function
 */

extern int HdlcAExternLoopback(void) ;
/*
 * Function : HdlcAExternLoopback
 * Description : HDLC Internal Loopback Function
 */

extern int HdlcBExternLoopback(void) ;
/*
 * Function : HdlcBExternLoopback
 * Description : HDLC Channel B External Loopback Function
 */

extern int HdlcABExternLoopback(void) ;
/*
 * Function : HdlcA2BExternLoopback
 * Description : HDLC A to B External Loopback Function
 */

extern int HdlcALoopback(U8 *TxDBA, int tsize, int lcount) ;
/*
 * Function : HdlcALoopback
 * Description : HDLC Channel A Loopback Function
 */

extern int HdlcBLoopback(U8 *TxDBB, int tsize, int lcount) ;
/*
 * Function : HdlcBLoopback
 * Description : HDLC Channel B Loopback Function
 */

extern int HdlcA2BLoopback(U8 *TxDBA, int tsize, int lcount) ;
/*
 * Function : HdlcA2BLoopback
 * Description : HDLC A to B Loopback Function
 */

extern int HdlcB2ALoopback(U8 *TxDBB, int tsize, int lcount) ;
/*
 * Function : HdlcB2ALoopback
 * Description : HDLC B to A Loopback Function
 */

extern void ViewHdlcErrorRpt(void) ;
/* 
 * Function : ViewHdlcErrorRpt
 * Description : View HDLC Error Report and Clear
 */

#endif  /* _HDLC_H */ 
