/***********************************************************************/
/*                                                                     */
/*   MODULE:  hdlcinit.c                                               */
/*   DATE:    98/02/12                                                 */
/*   PURPOSE: hdlc(high level data link control) function define.      */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by hbahn                                            */
/*                                                                     */
/***********************************************************************/
#include  "ks5000.h"
#include "sysconf.h"
#include "hdlc.h"
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "memory.h"
#include "system.h"
#include "kslib.h"

// Global variable for HDLC Channel A test
volatile U32	gHCon0_A = BRGEN | DPLLEN | TxRTS | TxDTR ;
volatile U32	gHCon1_A = F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
			BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_BRG1 ;
volatile U32	gHintEn_A = HDLCIE | TxIE | RxIE | DTxIE | DRxIE | DPLLIE | \
			TxFCIE | TxCTSIE | TxUIE | RxFAIE | RxFDIE | \
			RxAERRIE | RxABIE | RxFERRIE | RxOVIE | DTxSTOPIE | \
			DTxABIE | DRxSTOPIE | DRxABIE | DPLLOMIE | DPLLTMIE ;


// Global variable for HDLC Channel B test
volatile U32	gHCon0_B = BRGEN | DPLLEN | TxRTS | TxDTR ;
volatile U32	gHCon1_B = F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
			BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_BRG1 ;
volatile U32	gHintEn_B = HDLCIE | TxIE | RxIE | DTxIE | DRxIE | DPLLIE | \
			TxFCIE | TxCTSIE | TxUIE | RxFAIE | RxFDIE | \
			RxAERRIE | RxABIE | RxFERRIE | RxOVIE | DTxSTOPIE | \
			DTxABIE | DRxSTOPIE | DRxABIE | DPLLOMIE | DPLLTMIE ;

// Global variable for HDLC Tx/Rx Status report
pHdlcTxStatus 	gAHdlcTxStatus = {0,0,0,0,0,0,0} ;
pHdlcRxStatus 	gAHdlcRxStatus = {0,0,0,0,0,0,0,0,0,0,0,0,0} ;
pHdlcDPLLStatus gAHdlcDPLLStatus = {0,0} ;
pHdlcTxStatus 	gBHdlcTxStatus = {0,0,0,0,0,0,0} ;
pHdlcRxStatus 	gBHdlcRxStatus = {0,0,0,0,0,0,0,0,0,0,0,0,0} ;
pHdlcDPLLStatus gBHdlcDPLLStatus = {0,0} ;

// Global variable for HDLC Rx Buffer
U32 gRxDB_A 	= RxDBA_Start ;
U32 gRxDB_B 	= RxDBB_Start ;

// Global variable for Tx Interrupt
volatile U32 gHdlcATxDone = 0 ;


/*
 *  Function : HDLCA_init
 *  Description : HDLC Channel A controller initialize
 */
void HDLCA_init(void)
{
	// Reset HDLC Controller and HDLC DMA
	HCON0A	= TxRS | RxRS | DTxRS | DRxRS ;

	// Set Station address
	HSADRA	= 0x13245678 ;

	// Set Baud Rate
	HBRGTCA	= fMCLK / (2 * HDLC_BAUDRATE) - 1   ;

	// Set Preamble value for NRZ code
	HPRMBA	=  HDLC_PREAMBLE ;

	// Set HDLC Control Register 1 ;
	HCON1A	= gHCon1_A ;
	
	// HDLC Controller 0 Initialize
	// Baud-rate Generator Enable, DPLL Enable, Tx/Rx Enable
	HCON0A	= gHCon0_A ;

	// Set HDLC Interrupt enable register
	HINTENA = gHintEn_A ;

	// Enable HDLC Rx
	HDMAA_Rx_init() ;
	HDLCARxEnable() ;

	// Setup Interrupt for HDLC Channel A
	SysSetInterrupt(nHDLCA_INT, HDLCA_isr );
	Enable_Int(nHDLCA_INT);
	Enable_Int(nGLOBAL_INT); 
}

/*
 *  Function : HDLCB_init
 *  Description : HDLC Channel B controller initialize
 */
void HDLCB_init(void)
{
	// Reset HDLC Controller and HDLC DMA
	HCON0B	= TxRS | RxRS | DTxRS | DRxRS ;

	
	HSADRB	= 0x13245678 ;

	// Set Baud Rate
	HBRGTCB	= fMCLK / (2 * HDLC_BAUDRATE) - 1   ;

	// Set Preamble value for NRZ code
	HPRMBB	=  HDLC_PREAMBLE ;

	// Set HDLC Control Register 1 ;
	HCON1B	= gHCon1_B ;
	
	// HDLC Controller 0 Initialize
	// Baud-rate Generator Enable, DPLL Enable, Tx/Rx Enable
	HCON0B	= gHCon0_B ;

	// Set HDLC Interrupt enable register
	HINTENB = gHintEn_B ;

	// Enable HDLC Rx
	HDMAB_Rx_init() ;
	HDLCBRxEnable() ;

	// Setup Interrupt for HDLC Channel B
	SysSetInterrupt(nHDLCB_INT, HDLCB_isr );
	Enable_Int(nHDLCB_INT);
	Enable_Int(nGLOBAL_INT); 
}

/*
 *  Function : HDMAA_Tx_init
 *  Description : HDMA Channel A Tx initialize
 */
void HDMAA_Tx_init(U32 TxDB_A, U32 TxD_Size_A)
{
	HDMATXMAA 	= TxDB_A ;
	HDMATXCNTA 	= TxD_Size_A ;
}

/*
 *  Function : HDMAA_Rx_init
 *  Description : HDMA Channel A Rx initialize
 */
void HDMAA_Rx_init(void)
{
	HDMARXBCNTA	= 0x0  ;
	HDMARXMAA 	= gRxDB_A ;
	HDMARXCNTA 	= HdmaRxSize_A	;
}

/*
 *  Function : HDMAB_Tx_init
 *  Description : HDMA Channel B Tx initialize
 */
void HDMAB_Tx_init(U32 TxDB_B, U32 TxD_Size_B)
{
	HDMATXMAB 	= TxDB_B ;
	HDMATXCNTB 	= TxD_Size_B ;
}

/*
 *  Function : HDMAB_Rx_init
 *  Description : HDMA Channel B Rx initialize
 */
void HDMAB_Rx_init(void)
{
	HDMARXBCNTB	= 0x0  ;
	HDMARXMAB 	= gRxDB_B ;
	HDMARXCNTB 	= HdmaRxSize_B	;
}


/*
 *  Function : HDLCA_isr
 *  Description : HDMA Channel A Interrupt Service Routine
 */
void HDLCA_isr(void)
{
	U32 IntHdlcStatus ;

	IntHdlcStatus = HSTATA ;

	// Clear Interrupt Pending Bit
	Clear_PendingBit(nHDLCA_INT);

	// HDMA Rx Abort, but not Rx abort and Rx overrun
	//if ( IntHdlcStatus & DRxABT & !RxABT & !RxOV)
	if ( IntHdlcStatus & DRxABT )
	{
		HSTATA &= ~DRxABT ;
		HDMARXCNTA = HdmaRxSize_A ;
		HDLCARxEnable() ; 
		gAHdlcRxStatus.DMARxABT++ ;
	}

	// HDMA Rx Stop, When Rx Done
	if ( IntHdlcStatus & DRxSTOP )
	{
		gAHdlcRxStatus.DMARxStop++ ;
		if ( !GetRxHdlcData_A(IntHdlcStatus) ) 
			Print("\r HDLC Channel A Rx Fail !!! ") ;

		HSTATA &= ~DRxSTOP ;

	}

	// HDLC Tx Complete
	if ( IntHdlcStatus & ( TxFC | DTxSTOP) )
	{
		// HDLC Tx Complete
		if ( IntHdlcStatus & TxFC )
		{
			//HCON0A &= ~TxRTS ;
			gAHdlcTxStatus.TxFrameComplete++ ;
			gHdlcATxDone = 1 ;
			HSTATA &= ~TxFC ;
		}
	
		// HDMA Tx Complete
		if ( IntHdlcStatus & DTxSTOP )
		{
			gAHdlcTxStatus.DMATxStop++ ;
			HSTATA &= ~DTxSTOP ;
		}
	}

	// HDLC Rx Error
	if ( IntHdlcStatus & ( RxFERR | RxOV | RxABT | RxAERR) )
	{
		// HDLC Rx CRC Error Occured
		if ( IntHdlcStatus & RxFERR )
		{
			gAHdlcRxStatus.RxFrameError++ ;
			HSTATA &= ~RxFERR ;
		}
	
		// HDLC Rx OverRun Error
		if ( IntHdlcStatus & RxOV )
		{
			gAHdlcRxStatus.RxOverrun++ ;
			HSTATA &= ~RxOV ;
		}
	
		// HDLC Rx Abort
		if ( IntHdlcStatus & RxABT )
		{
			gAHdlcRxStatus.RxAbort++ ;
			HSTATA &= ~RxABT ;
		}

		// HDLC Rx Address Error
		if ( IntHdlcStatus & RxAERR )
		{
			gAHdlcRxStatus.RxAddressError++ ;
			HSTATA &= ~RxAERR ;
		}
	}

	// HDLC Tx Error
	if ( IntHdlcStatus & ( TxU | DTxABT ) )
	{
		// HDLC Tx Underrun Error
		if ( IntHdlcStatus & TxU )
		{
			gAHdlcTxStatus.TxUnderrun++ ;
			HSTATA &= ~TxU ;
		}

		// HDMA Tx Abort
		if ( IntHdlcStatus & DTxABT )
		{
			gAHdlcTxStatus.DMATxABT++ ;
			HSTATA &= ~DTxABT ;
		}
	}

	// HDLC DPLL Error
	if ( IntHdlcStatus & ( DPLLOM | DPLLTM ) )
	{
		// DPLL One clock missing
		if ( IntHdlcStatus & DPLLOM )
		{
			gAHdlcDPLLStatus.DPLLOneClkMiss++ ;
			HSTATA &= ~DPLLOM ;
		}

		// DPLL Two clock missing
		if ( IntHdlcStatus & DPLLTM )
		{
			gAHdlcDPLLStatus.DPLLTwoClkMiss++ ;
			HSTATA &= ~DPLLTM ;
		}
	}


	// Other HDLC Status
	if ( IntHdlcStatus & ( TxFA | TxCTS | TxSCTS ) )
	{
		if (IntHdlcStatus & TxFA) 
			{
			gAHdlcTxStatus.TxFIFOAvailable++ ;
			HSTATA &= ~TxFA ;
			}
		if (IntHdlcStatus & TxCTS) 
			{
			gAHdlcTxStatus.TxLevelOfCTS++ ;
			HSTATA &= ~TxCTS ;
			}
		if (IntHdlcStatus & TxSCTS) 
			{
			gAHdlcTxStatus.TxTransitionOfCTS++ ;
			HSTATA &= ~TxSCTS ;
			}
	}


	if ( IntHdlcStatus & ( RxFA | RxFAP | RxFD | RxDCD | \
				RxSDCD | RxFV | RxIDLE) ) ;
	{
		if (IntHdlcStatus & RxFA) 
			{
			gAHdlcRxStatus.RxFIFOAvalable++ ;
			HSTATA &= ~RxFA ;
			}
		if (IntHdlcStatus & RxFAP) 
			{
			gAHdlcRxStatus.RxFIFOAddrPresent++ ;
			HSTATA &= ~RxFAP ;
			}
		if (IntHdlcStatus & RxFD) 
			{
			gAHdlcRxStatus.RxFlagDetected++ ;
			HSTATA &= ~RxFD ;
			}
		if (IntHdlcStatus & RxDCD) 
			{
			gAHdlcRxStatus.RxLevelOfDCD++ ;
			HSTATA &= ~RxDCD ;
			}
		if (IntHdlcStatus & RxSDCD) 
			{
			gAHdlcRxStatus.RxTransitionOfDCD++ ;
			HSTATA &= ~RxSDCD ;
			}
		if (IntHdlcStatus & RxFV) 
			{
			gAHdlcRxStatus.RxLastFrameValid++ ;
			HSTATA &= ~RxFV ;
			}
		if (IntHdlcStatus & RxIDLE) 
			{
			gAHdlcRxStatus.RxIdle++ ;
			HSTATA &= ~RxIDLE ;
			}
	}


}

/*
 *  Function : HDLCB_isr
 *  Description : HDMA Channel B Interrupt Service Routine
 */
void HDLCB_isr(void)
{
	U32 IntHdlcStatus ;

	IntHdlcStatus = HSTATB ;

	// Clear Interrupt Pending Bit
	Clear_PendingBit(nHDLCB_INT);

	// HDMA Rx Abort, but not Rx abort and Rx overrun
	if ( IntHdlcStatus & DRxABT )
	{
		HSTATB |= DRxABT ;
		//HSTATB &= ~DRxABT ;
		HDMARXCNTB = HdmaRxSize_B ;
		HDLCBRxEnable() ; 
		gBHdlcRxStatus.DMARxABT++ ;
	}

	// HDMA Rx Stop, When Rx Done
	if ( IntHdlcStatus & DRxSTOP )
	{
		gBHdlcRxStatus.DMARxStop++ ;
		if ( !GetRxHdlcData_B(IntHdlcStatus) ) 
			Print("\r HDLC Channel B Rx Fail !!! ") ;

		HSTATB |= DRxSTOP ;
		//HSTATB &= ~DRxSTOP ;

	}

	// HDLC Tx Complete
	if ( IntHdlcStatus & ( TxFC | DTxSTOP) )
	{
		// HDLC Tx Complete
		if ( IntHdlcStatus & TxFC )
		{
			//HCON0B &= ~TxRTS ;
			gBHdlcTxStatus.TxFrameComplete++ ;
			HSTATB |= TxFC ;
			//HSTATB &= ~TxFC ;
		}
	
		// HDMA Tx Complete
		if ( IntHdlcStatus & DTxSTOP )
		{
			gBHdlcTxStatus.DMATxStop++ ;
			HSTATB |= DTxSTOP ;
			//HSTATB &= ~DTxSTOP ;
		}
	}

	// HDLC Rx Error
	if ( IntHdlcStatus & ( RxFERR | RxOV | RxABT | RxAERR) )
	{
		// HDLC Rx CRC Error Occured
		if ( IntHdlcStatus & RxFERR )
		{
			gBHdlcRxStatus.RxFrameError++ ;
			HSTATB |= RxFERR ;
			//HSTATB &= ~RxFERR ;
		}
	
		// HDLC Rx OverRun Error
		if ( IntHdlcStatus & RxOV )
		{
			gBHdlcRxStatus.RxOverrun++ ;
			HSTATB |= RxOV ;
			//HSTATB &= ~RxOV ;
		}
	
		// HDLC Rx Abort
		if ( IntHdlcStatus & RxABT )
		{
			gBHdlcRxStatus.RxAbort++ ;
			HSTATB |= RxABT ;
			//HSTATB &= ~RxABT ;
		}

		// HDLC Rx Address Error
		if ( IntHdlcStatus & RxAERR )
		{
			gBHdlcRxStatus.RxAddressError++ ;
			HSTATB |= RxAERR ;
			//HSTATB &= ~RxAERR ;
		}
	}

	// HDLC Tx Error
	if ( IntHdlcStatus & ( TxU | DTxABT ) )
	{
		// HDLC Tx Underrun Error
		if ( IntHdlcStatus & TxU )
		{
			gBHdlcTxStatus.TxUnderrun++ ;
			HSTATB |= TxU ;
			//HSTATB &= ~TxU ;
		}

		// HDMA Tx Abort
		if ( IntHdlcStatus & DTxABT )
		{
			gBHdlcTxStatus.DMATxABT++ ;
			HSTATB |= DTxABT ;
			//HSTATB &= ~DTxABT ;
		}
	}

	// HDLC DPLL Error
	if ( IntHdlcStatus & ( DPLLOM | DPLLTM ) )
	{
		// DPLL One clock missing
		if ( IntHdlcStatus & DPLLOM )
		{
			gBHdlcDPLLStatus.DPLLOneClkMiss++ ;
			HSTATB |= DPLLOM ;
			//HSTATB &= ~DPLLOM ;
		}

		// DPLL Two clock missing
		if ( IntHdlcStatus & DPLLTM )
		{
			gBHdlcDPLLStatus.DPLLTwoClkMiss++ ;
			HSTATB |= DPLLTM ;
			//HSTATB &= ~DPLLTM ;
		}
	}


	// Other HDLC Status
	if ( IntHdlcStatus & ( TxFA | TxCTS | TxSCTS ) )
	{
		if (IntHdlcStatus & TxFA) 
			{
			gBHdlcTxStatus.TxFIFOAvailable++ ;
			HSTATB |= TxFA ;
			//HSTATB &= ~TxFA ;
			}
		if (IntHdlcStatus & TxCTS) 
			{
			gBHdlcTxStatus.TxLevelOfCTS++ ;
			HSTATB |= TxCTS ;
			//HSTATB &= ~TxCTS ;
			}
		if (IntHdlcStatus & TxSCTS) 
			{
			gBHdlcTxStatus.TxTransitionOfCTS++ ;
			HSTATB |= TxSCTS ;
			//HSTATB &= ~TxSCTS ;
			}
	}


	if ( IntHdlcStatus & ( RxFA | RxFAP | RxFD | RxDCD | \
				RxSDCD | RxFV | RxIDLE) ) ;
	{
		if (IntHdlcStatus & RxFA) 
			{
			gBHdlcRxStatus.RxFIFOAvalable++ ;
			HSTATB |= RxFA ;
			//HSTATB &= ~RxFA ;
			}
		if (IntHdlcStatus & RxFAP) 
			{
			gBHdlcRxStatus.RxFIFOAddrPresent++ ;
			HSTATB |= RxFAP ;
			//HSTATB &= ~RxFAP ;
			}
		if (IntHdlcStatus & RxFD) 
			{
			gBHdlcRxStatus.RxFlagDetected++ ;
			HSTATB |= RxFD ;
			//HSTATB &= ~RxFD ;
			}
		if (IntHdlcStatus & RxDCD) 
			{
			gBHdlcRxStatus.RxLevelOfDCD++ ;
			HSTATB |= RxDCD ;
			//HSTATB &= ~RxDCD ;
			}
		if (IntHdlcStatus & RxSDCD) 
			{
			gBHdlcRxStatus.RxTransitionOfDCD++ ;
			HSTATB |= RxSDCD ;
			//HSTATB &= ~RxSDCD ;
			}
		if (IntHdlcStatus & RxFV) 
			{
			gBHdlcRxStatus.RxLastFrameValid++ ;
			HSTATB |= RxFV ;
			//HSTATB &= ~RxFV ;
			}
		if (IntHdlcStatus & RxIDLE) 
			{
			gBHdlcRxStatus.RxIdle++ ;
			HSTATB |= RxIDLE ;
			//HSTATB &= ~RxIDLE ;
			}
	}
}
