/***********************************************************************/
/*                                                                     */
/*   MODULE:  hdlclib.c                                                */
/*   DATE:    98/02/12                                                 */
/*   PURPOSE: hdlc(high level data link control) function define.      */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by hbahn                                            */
/*                                                                     */
/***********************************************************************/
#include  "ks5000.h"
#include "sysconf.h"
#include "hdlc.h"
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "memory.h"
#include "system.h"
#include "kslib.h"

extern U32	gHCon0_A , gHCon1_A, gHintEn_A ;
extern U32	gHCon0_B , gHCon1_B, gHintEn_B ;
extern U32	gRxDB_A, gRxDB_B ;
extern volatile U32	gHdlcARxDone = 0, gHdlcBRxDone = 0 ;
// Global variable for HDLC Tx/Rx Status report
extern pHdlcTxStatus 	gAHdlcTxStatus ;
extern pHdlcRxStatus 	gAHdlcRxStatus ;
extern pHdlcDPLLStatus gAHdlcDPLLStatus ;
extern pHdlcTxStatus 	gBHdlcTxStatus ;
extern pHdlcRxStatus 	gBHdlcRxStatus ;
extern pHdlcDPLLStatus gBHdlcDPLLStatus ;


/*
 * Function : HDLCATxGo
 * Description : HDLC Channel A Tx Enable
 */
void HDLCATxGo(void) 
{
	//HCON0A |= TxRTS ;
	HCON0A |= TxEN | DTxEN ;
}


/*
 * Function : HDLCARxEnable
 * Description : HDLC Channel A Rx Enable
 */
void HDLCARxEnable(void) 
{
	HCON0A |= RxEN | DRxEN ;
}

/*
 * Function : HDLCBTxGo
 * Description : HDLC Channel B Tx Enable
 */
void HDLCBTxGo(void) 
{
	//HCON0B |= TxRTS ;
	HCON0B |= TxEN | DTxEN ;
}


/*
 * Function : HDLCBRxEnable
 * Description : HDLC Channel B Rx Enable
 */
void HDLCBRxEnable(void) 
{
	HCON0B |= RxEN | DRxEN ;
}


/*
 * Function : HDLCALoopbackEn
 * Description : HDLC Channel A Internal Loopback Test Enable
 */
void HDLCALoopbackEn(void) 
{
	HCON1A	|= TxLOOP ;
}

/*
 * Function : HDLCALoopbackDis
 * Description : HDLC Channel A Internal Loopback Test Disable
 */
void HDLCALoopbackDis(void) 
{
	HCON1A	&= ~TxLOOP ;
}

/*
 * Function : HDLCBLoopbackEn
 * Description : HDLC Channel B Internal Loopback Test Enable
 */
void HDLCBLoopbackEn(void) 
{
	HCON1B	|= TxLOOP ;
}

/*
 * Function : HDLCBLoopbackDis
 * Description : HDLC Channel B Internal Loopback Test Disable
 */
void HDLCBLoopbackDis(void) 
{
	HCON1B	&= ~TxLOOP ;
}


/* 
 * Function : GetRxHdlcData_A
 * Description : HDLC Channel A Data Receive
 */
int GetRxHdlcData_A(U32 IntHdlcStatus)
{
	U32 RxDataByteSize ;
	U8 *RxFDBuffer ;

	RxDataByteSize = HDMARXBCNTA ;
	//Print("\r Channel A Receve Data Size : %x ",RxDataByteSize) ;
	//Print("\r Channel A Receive Status : %x ",IntHdlcStatus) ;
	if ( IntHdlcStatus & ( RxFERR | RxOV | RxABT | RxAERR) )
		return HdlcReturnFail ;

	// Copy received frame data to user data area
	RxFDBuffer = (U8 *)malloc(RxDataByteSize) ;
	bcopy((U8 *)gRxDB_A, RxFDBuffer, RxDataByteSize) ; 
	free(RxFDBuffer) ;
	gHdlcARxDone = 1 ;

	// HDMA initialize for next use
	if ( gRxDB_A < (RxDBA_Limit-RxDataByteSize) ) 
				gRxDB_A += RxDataByteSize ;
	else gRxDB_A = RxDBA_Start ;
	HDMAA_Rx_init() ;
	HDLCARxEnable() ;
	
	return HdlcReturnOk ;
}
 
/* 
 * Function : GetRxHdlcData_B
 * Description : HDLC Channel B Data Receive
 */
int GetRxHdlcData_B(U32 IntHdlcStatus)
{
	U32 RxDataByteSize ;
	U8 *RxFDBuffer ;

	RxDataByteSize = HDMARXBCNTB ;
	//Print("\r Channel B Receive Data Size : %x ",RxDataByteSize) ;
	//Print("\r Channel B Receive Status : %x ",IntHdlcStatus) ;
	gHdlcBRxDone = 1 ;
	if ( IntHdlcStatus & ( RxFERR | RxOV | RxABT | RxAERR) )
		return HdlcReturnFail ;

	// Copy received frame data to user data area
	RxFDBuffer = (U8 *)malloc(RxDataByteSize) ;
	bcopy((U8 *)gRxDB_B, RxFDBuffer, RxDataByteSize) ; 
	free(RxFDBuffer) ;

	// HDMA initialize for next use
	if ( gRxDB_B < (RxDBB_Limit-RxDataByteSize) ) 
				gRxDB_B += RxDataByteSize ;
	else gRxDB_B = RxDBB_Start ;
	HDMAB_Rx_init() ;
	HDLCBRxEnable() ;
	
	return HdlcReturnOk ;
}


/*
 * Function : HDLCTxDataInit
 * Description : HDLC Tx Data Initialize
 */
void HDLCTxDataInit(U8 *TxDB, U32 TxD_Size) 
{
	U8 TxD_Data = 0xa5 ;
	int i=0 ;

	while(TxD_Size--)
		{
		//*TxDB++ = TxD_Data ;
		//*TxDB++ = i++ ;
		*TxDB++ = TxD_Size ^ TxD_Data ;
		}
}

/*
 * Function : HdlcInternalLoopback
 * Description : HDLC Internal Loopback Function
 */
int HdlcInternalLoopback(void)
{
	U8 *TxDBA, *TxDBB ;
	U32 DBSize = 0x2000 ;
	int lcount = 12 ;

	// Transfer Frame Buffer Area
	TxDBA 	= (U8 *)0x1100000 ;
	TxDBB 	= (U8 *)0x1100000 ;

	// channel A Loopback enable
	HDLCALoopbackEn() ; 
	HDLCBLoopbackEn() ; 

	// Test Data initialize
	HDLCTxDataInit(TxDBA, DBSize) ;

	Print("\n >>> HDLC Channel-A Internal Loopback test ... ") ;
	if ( !HdlcALoopback(TxDBA, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;

	Print("\n >>> HDLC Channel-B Internal Loopback test ... ") ;
	if ( !HdlcBLoopback(TxDBB, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;

	// channel A Loopback Disable
	HDLCALoopbackDis() ; 
	HDLCBLoopbackDis() ; 
	IOPDATA = 0xF ;

	return  HdlcReturnOk ;
}

/*
 * Function : HdlcAExternLoopback
 * Description : HDLC Channel A External Loopback Function
 */
int HdlcAExternLoopback(void)
{
	U8 *TxDBA ;
	U32 DBSize = 0x500 ;
	int lcount = 12 ;

	// Clock Setting
	HCON1A	= F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
		BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_RXC | \
		TxOCLK_BRG1 ;

	// Set Baud Rate
	HBRGTCA	= fMCLK / (2 * 128000) - 1   ;

	// Transfer Frame Buffer Area
	TxDBA 	= (U8 *)0x1100000 ;

	// Test Data initialize
	HDLCTxDataInit(TxDBA, DBSize) ;

	Print("\n >>> HDLC Channel-A External Loopback test ... ") ;
	if ( !HdlcALoopback(TxDBA, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;

	// Return to original setting
	HCON1A	= gHCon1_A ;

	// Return to original baud rate
	HBRGTCA	= fMCLK / (2 * HDLC_BAUDRATE) - 1 ;

	// Set LED Display
	IOPDATA = 0xF ;

	return  HdlcReturnOk ;
}

/*
 * Function : HdlcBExternLoopback
 * Description : HDLC Channel B External Loopback Function
 */
int HdlcBExternLoopback(void)
{
	U8 *TxDBB ;
	U32 DBSize = 0x500 ;
	int lcount = 12 ;

	// Clock Setting
	HCON1B	= F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
		BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_RXC | \
		TxOCLK_BRG1 ;

	// Set Baud Rate
	HBRGTCB	= fMCLK / (2 * 128000) - 1   ;

	// Transfer Frame Buffer Area
	TxDBB 	= (U8 *)0x1100000 ;

	// Test Data initialize
	HDLCTxDataInit(TxDBB, DBSize) ;

	Print("\n >>> HDLC Channel-B External Loopback test ... ") ;
	if ( !HdlcBLoopback(TxDBB, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;

	// Return to original setting
	HCON1B	= gHCon1_B ;

	// Return to original baud rate
	HBRGTCB	= fMCLK / (2 * HDLC_BAUDRATE) - 1 ;

	// Set LED Display
	IOPDATA = 0xF ;

	return  HdlcReturnOk ;
}

/*
 * Function : HdlcABExternLoopback
 * Description : HDLC A to B External Loopback Function
 */
int HdlcABExternLoopback(void)
{
	U8  *TxDBA, *TxDBB ;
	U32 DBSize = 0x500 ;
	int lcount = 36 ;

	// Clock Setting
	HCON1A	= F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
		BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_RXC ;
	HCON1B	= F4WM | DTxMAINC | DRxMAINC | DRxSTOPTC | DF_NRZ | \
		BRGCLK_MCLK | DPLLCLK_MCLK | TxCLK_BRG1 | RxCLK_RXC ;

	// Set Baud Rate
	HBRGTCB	= fMCLK / (2 * 128000) - 1   ;
	HBRGTCA	= fMCLK / (2 * 128000) - 1   ;

	// Transfer Frame Buffer Area
	TxDBA 	= (U8 *)0x1100000 ;
	TxDBB 	= (U8 *)0x1100000 ;

	// Test Data initialize
	HDLCTxDataInit(TxDBA, DBSize) ;

	Print("\n >>> HDLC A to B External Loopback test ... ") ;
	if ( !HdlcA2BLoopback(TxDBA, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;

	Print("\n >>> HDLC B to A External Loopback test ... ") ;
	if ( !HdlcB2ALoopback(TxDBB, DBSize, lcount) ) 
		{
		Print(" Fail !") ;
		return  HdlcReturnFail ;
		}
	else Print(" Ok !!!") ;

	// Return to original setting
	HCON1A	= gHCon1_A ;
	HCON1B	= gHCon1_B ;

	// Return to original baud rate
	HBRGTCA	= fMCLK / (2 * HDLC_BAUDRATE) - 1 ;
	HBRGTCB	= fMCLK / (2 * HDLC_BAUDRATE) - 1 ;

	// Set LED Display
	IOPDATA = 0xF ;

	return  HdlcReturnOk ;
}

/*
 * Function : HdlcALoopback
 * Description : HDLC Channel A Loopback Function
 */
int HdlcALoopback(U8 *TxDBA, int tsize, int lcount)
{
	U8 *RxDBA ;
	U32 lcnt, tcnt ;

	lcnt = lcount ;

	while(lcount--)
	{
	   for (tcnt=0 ; tcnt < 12 ; tcnt++)
	   {

		// Initialize Tx HDMA
		HDMAA_Tx_init((U32)TxDBA, tsize) ;
	
		// Save HDMA Rx Ptr
		RxDBA = (U8 *)gRxDB_A ;
	
		// Transfer Enable & receive enable
		HDLCATxGo() ;
	
		// wait until loopback Rx Done 
		//Print("\r Waiting for Loopback data Rx finished") ;
		//ViewHdlcErrorRpt() ;
		while(!gHdlcARxDone) ;
		gHdlcARxDone = 0 ;
	
		// Compare loopback data
		if ( !CompareHdlcLoopbackData(TxDBA, RxDBA, tsize) )
			{
			Print("\r HDLC Channel A Loopback Test Fail !") ;
			return  HdlcReturnFail ;
			}

	   }
	   PrintMemTestStatus((lcnt-lcount)%4) ;
	}
	return  HdlcReturnOk ;
}

/*
 * Function : HdlcBLoopback
 * Description : HDLC Channel B Loopback Function
 */
int HdlcBLoopback(U8 *TxDBB, int tsize, int lcount)
{
	U8 *RxDBB ;
	U32 lcnt, tcnt ;

	lcnt = lcount ;

	while(lcount--)
	{
	   for (tcnt=0 ; tcnt < 12 ; tcnt++)
	   {

		// Initialize Tx HDMA
		HDMAB_Tx_init((U32)TxDBB, tsize) ;
	
		// Save HDMA Rx Ptr
		RxDBB = (U8 *)gRxDB_B ;
	
		// Transfer Enable & receive enable
		HDLCBTxGo() ;
	
		// wait until loopback Rx Done 
		//Print("\r Waiting for Loopback data Rx finished") ;
		while(!gHdlcBRxDone) ;
		gHdlcBRxDone = 0 ;
	
		// Compare loopback data
		if ( !CompareHdlcLoopbackData(TxDBB, RxDBB, tsize) )
			{
			Print("\r HDLC Channel B Loopback Test Fail !") ;
			return  HdlcReturnFail ;
			}

	   }
	   PrintMemTestStatus((lcnt-lcount)%4) ;
	}
	return  HdlcReturnOk ;
}

/*
 * Function : HdlcA2BLoopback
 * Description : HDLC A to B Loopback Function
 */
int HdlcA2BLoopback(U8 *TxDBA, int tsize, int lcount)
{
	U8 *RxDBB ;
	U32 lcnt, tcnt ;

	lcnt = lcount ;

	while(lcount--)
	{
	   for (tcnt=0 ; tcnt < 25 ; tcnt++)
	   {
		// Initialize Tx HDMA
		HDMAA_Tx_init((U32)TxDBA, tsize) ;
	
		// Save HDMA Rx Ptr
		RxDBB = (U8 *)gRxDB_B ;
	
		// Transfer Enable & receive enable
		HDLCATxGo() ;
	
		// wait until loopback Rx Done 
		//Print("\r Waiting for Loopback data Rx finished") ;
		while(!gHdlcBRxDone) ;
		gHdlcBRxDone = 0 ;
	
		// Compare loopback data
		if ( !CompareHdlcLoopbackData(TxDBA, RxDBB, tsize) )
			{
			Print("\r HDLC A to B Loopback Test Fail !") ;
			return  HdlcReturnFail ;
			}
	   }
	   PrintMemTestStatus((lcnt-lcount)%4) ;
	}
	return  HdlcReturnOk ;
}

/*
 * Function : HdlcB2ALoopback
 * Description : HDLC B to A Loopback Function
 */
int HdlcB2ALoopback(U8 *TxDBB, int tsize, int lcount)
{
	U8 *RxDBA ;
	U32 lcnt, tcnt ;

	lcnt = lcount ;

	while(lcount--)
	{
	   for (tcnt=0 ; tcnt < 25 ; tcnt++)
	   {
		// Initialize Tx HDMA
		HDMAB_Tx_init((U32)TxDBB, tsize) ;
	
		// Save HDMA Rx Ptr
		RxDBA = (U8 *)gRxDB_A ;
	
		// Transfer Enable & receive enable
		HDLCBTxGo() ;
	
		// wait until loopback Rx Done 
		//Print("\r Waiting for Loopback data Rx finished") ;
		while(!gHdlcARxDone) ;
		gHdlcARxDone = 0 ;
	
		// Compare loopback data
		if ( !CompareHdlcLoopbackData(TxDBB, RxDBA, tsize) )
			{
			Print("\r HDLC B to A Loopback Test Fail !") ;
			return  HdlcReturnFail ;
			}
	   }
	   PrintMemTestStatus((lcnt-lcount)%4) ;
	}
	return  HdlcReturnOk ;
}

/*
 * Function : PrintHdlcData
 * Description : Print HDLC Tx/Rx Data 
 */
void PrintHdlcData(U8 *DBPtr, U32 Size)
{
	int i=1 ;

	Print("\n HDLC Data :") ;
	while(Size--)
	{
		if ( !(i++ % 20) ) Print("\r%12c",' ') ;
		Print(" %02x",*DBPtr++) ;
	}
}

/*
 * Function : CompareHdlcLoopbackData
 * Description : HDLC Tx Data Initialize
 * Return : Compare Ok or Fail
 */
int CompareHdlcLoopbackData(U8 *TxDB, U8 *RxDB, U32 Size)
{
	if (!bcomp(TxDB, RxDB, Size)) return HdlcReturnFail  ;
	else return HdlcReturnOk ;
}


/* 
 * Function : ViewHdlcErrorRpt
 * Description : View HDLC Error Report and Clear
 */
void ViewHdlcErrorRpt(void)
{
	Print("\n  >> HDLC Control Register  Value  <<") ;
	Print("\r+-----------------------------------------+") ;
	Print("\r                    Channel A    Channel B  ") ;
	Print("\r+-----------------------------------------+") ;
	Print("\r HCON0              %08x     %08x  ",HCON0A, HCON0B) ; 
	Print("\r HCON1              %08x     %08x  ",HCON1A, HCON1B) ;
	Print("\r HSTAT              %08x     %08x  ",HSTATA, HSTATB) ; 
	Print("\r HINTEN             %08x     %08x  ",HINTENA, HINTENB) ; 
	Print("\r HBRGTC             %08x     %08x  ",HBRGTCA, HBRGTCB) ;
	Print("\r HDMATXCNT          %08x     %08x  ",HDMATXCNTA, HDMATXCNTB) ; 
	Print("\r HDMARXCNT          %08x     %08x  ",HDMARXCNTA, HDMARXCNTB) ; 
	Print("\r HDMARXBCNT         %08x     %08x  ",HDMARXBCNTA, HDMARXBCNTB) ; 
	Print("\r+-----------------------------------------+") ;
	Print("\n  Press Any Key "); get_byte() ;

	Print("\n  >> HDLC Status Register  Value  <<") ;
	Print("\r+-----------------------------------------+") ;
	Print("\r                   Channel A    Channel B  ") ;
	Print("\r+-----------------------------------------+") ;
	Print("\r Tx Complete           %5d        %5d  ", \
		gAHdlcTxStatus.TxFrameComplete, gBHdlcTxStatus.TxFrameComplete) ;
	Print("\r Tx FIFO Avail         %5d        %5d  ", \
		gAHdlcTxStatus.TxFIFOAvailable, gBHdlcTxStatus.TxFIFOAvailable) ;
	Print("\r Tx CTS Tran           %5d        %5d  ", \
		gAHdlcTxStatus.TxTransitionOfCTS, gBHdlcTxStatus.TxTransitionOfCTS) ;
	Print("\r Tx Underrun           %5d        %5d  ", \
		gAHdlcTxStatus.TxUnderrun, gBHdlcTxStatus.TxUnderrun) ;
	Print("\r Tx DMA Stop           %5d        %5d  ", \
		gAHdlcTxStatus.DMATxStop, gBHdlcTxStatus.DMATxStop) ;
	Print("\r Tx DMA Abort          %5d        %5d  ", \
		gAHdlcTxStatus.DMATxABT, gBHdlcTxStatus.DMATxABT) ;
	Print("\r+-----------------------------------------+") ;
	Print("\r Rx FIFO Avail         %5d        %5d  ", \
		gAHdlcRxStatus.RxFIFOAvalable, gBHdlcRxStatus.RxFIFOAvalable) ;
	Print("\r Rx FIFO Addr          %5d        %5d  ", \
		gAHdlcRxStatus.RxFIFOAddrPresent, gBHdlcRxStatus.RxFIFOAddrPresent) ;
	Print("\r Rx Flag Detect        %5d        %5d  ", \
		gAHdlcRxStatus.RxFlagDetected, gBHdlcRxStatus.RxFlagDetected) ;
	Print("\r Rx DCD Transit        %5d        %5d  ", \
		gAHdlcRxStatus.RxTransitionOfDCD, gBHdlcRxStatus.RxTransitionOfDCD) ;
	Print("\r Rx Addr Error         %5d        %5d  ", \
		gAHdlcRxStatus.RxAddressError, gBHdlcRxStatus.RxAddressError) ;
	Print("\r Rx LFrame Valid       %5d        %5d  ", \
		gAHdlcRxStatus.RxLastFrameValid, gBHdlcRxStatus.RxLastFrameValid) ;
	Print("\r Rx Idle               %5d        %5d  ", \
		gAHdlcRxStatus.RxIdle, gBHdlcRxStatus.RxIdle) ;
	Print("\r Rx Abort              %5d        %5d  ", \
		gAHdlcRxStatus.RxAbort, gAHdlcRxStatus.RxAbort) ;
	Print("\r Rx Frame Error        %5d        %5d  ", \
		gAHdlcRxStatus.RxFrameError, gBHdlcRxStatus.RxFrameError) ;
	Print("\r Rx Overrun            %5d        %5d  ", \
		gAHdlcRxStatus.RxOverrun, gBHdlcRxStatus.RxOverrun) ;
	Print("\r Rx DMA Stop           %5d        %5d  ", \
		gAHdlcRxStatus.DMARxStop, gBHdlcRxStatus.DMARxStop) ;
	Print("\r Rx DMA ABT            %5d        %5d  ", \
		gAHdlcRxStatus.DMARxABT, gBHdlcRxStatus.DMARxABT) ;
	Print("\r+-----------------------------------------+") ;
	Print("\r DPLL One Clk Miss     %5d        %5d  ", \
		gAHdlcDPLLStatus.DPLLOneClkMiss, gBHdlcDPLLStatus.DPLLOneClkMiss) ;
	Print("\r DPLL Two Clk Miss     %5d        %5d  ", \
		gAHdlcDPLLStatus.DPLLTwoClkMiss, gBHdlcDPLLStatus.DPLLTwoClkMiss) ;
	Print("\r+-----------------------------------------+") ;

	// Clear Error Report
	gAHdlcTxStatus.TxFrameComplete = 0 ;
	gAHdlcTxStatus.TxFIFOAvailable = 0 ;
	gAHdlcTxStatus.TxLevelOfCTS = 0 ;
	gAHdlcTxStatus.TxTransitionOfCTS = 0 ;
	gAHdlcTxStatus.TxUnderrun = 0 ;
	gAHdlcTxStatus.DMATxStop = 0 ;
	gAHdlcTxStatus.DMATxABT = 0 ;

	gAHdlcRxStatus.RxFIFOAvalable = 0 ;
	gAHdlcRxStatus.RxFIFOAddrPresent = 0 ;
	gAHdlcRxStatus.RxFlagDetected = 0 ;
	gAHdlcRxStatus.RxLevelOfDCD = 0 ;
	gAHdlcRxStatus.RxTransitionOfDCD = 0 ;
	gAHdlcRxStatus.RxAddressError = 0 ;
	gAHdlcRxStatus.RxLastFrameValid = 0 ;
	gAHdlcRxStatus.RxIdle = 0 ;
	gAHdlcRxStatus.RxAbort = 0 ;
	gAHdlcRxStatus.RxFrameError = 0 ;
	gAHdlcRxStatus.RxOverrun = 0 ;
	gAHdlcRxStatus.DMARxStop = 0 ;
	gAHdlcRxStatus.DMARxABT = 0 ;
	
	gAHdlcDPLLStatus.DPLLOneClkMiss = 0 ;
	gAHdlcDPLLStatus.DPLLTwoClkMiss = 0 ;
	
}

