/********************************************************************/
/*                                                                  */
/*      I2C  Test Program for KS32C5000                             */
/*                                                                  */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*                                                                  */
/*      Modified, programmed by jwchoi 			                    */
/*                                                                  */
/*      Description : 1998-2-18 first edited for KS32C5000          */
/********************************************************************/

#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "apdialog.h"
#include "ks5000.h"
#include "uart.h"
#include "iic.h"
#include "pic.h"
#include "pollio.h"

volatile U32 IICDoneFlag = 0 ;

// IIC Test
void IICTest(void) 
{
	char TestItemSel ;

	// IIC Setup
	IicSetup() ;

	do {
	Print("\n+--------------------------------+\r") ;
	Print("|     >>>>> IIC Test <<<<<       |\r") ;
        Print("| Rev 1.1 (by hbahn, 1998.04.16) |\r") ;
	Print("+--------------------------------+\r") ;
	Print("| IIC Write Test           - [W] |\r") ;
	Print("| IIC Read Test            - [R] |\r") ;
	Print("| IIC Read/Write Test      - [A] |\r") ;
	Print("| Quit IIC Test            - [Q] |\r") ;
	Print("+--------------------------------+\r") ;

	Enable_Int(nGLOBAL_INT); //enable global interrupt mask

	Print("\rSelect Test Item : ") ;
	TestItemSel = get_byte() ;

	switch(TestItemSel)
	{
		case 'W' : case 'w' : IICWriteTest() ; break ;
		case 'R' : case 'r' : IICReadTest() ; break ;
		case 'A' : case 'a' : IICAutoTest() ; break ;
		case 'Q' : case 'q' : break ;
		default : return ;
	}
	Print("\n Press Any Key to Continue IIC Test") ; 
	get_byte() ;

	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}


// IIC Write Test Program
void IICWriteTest(void)
{
	U8 TestData[30] ;
	int i, GetDataSize=0 ;

	Print("\r >>> Input Write Character(<30) : ") ;
	while ((TestData[GetDataSize] = get_byte()) != CR ) GetDataSize++ ;

	Print("\r +- Received Character is : ") ;
	for (i=0;i<GetDataSize;i++) put_byte(TestData[i]) ;
	IICWrite((U32)IicTestPtr,TestData, GetDataSize) ;
}

// IIC Read Test Program
void IICReadTest(void)
{
	U8 DataBuff[30] ;
	int ReadDataSize ;
	int i ;

	Print("\r >>> Input Read Data Size : ") ;
	ReadDataSize = get_digit() ;

	Print("\r +-- Read Data is : ") ;
	IICRead((U32)IicTestPtr,ReadDataSize,DataBuff) ;
	for (i=0;i<ReadDataSize;i++) put_byte(DataBuff[i]) ;
}

// IIC Read/Write Test Program
void IICAutoTest(void)
{
	U8 DataBuff[30] ;
	U8 TestData[] = "This is a IIC Test Program" ; 
	U8 TempData ;
	int i, TestSize, TestLoop ;

	Print("\n >>> IIC Read/Write Test <<<\r") ;

	TestLoop = TestSize = sizeof(TestData) ;

	while(TestLoop--) {
	Print("\r $$ ") ;
	IICWrite((U32)IicTestPtr,TestData, TestSize) ;
	for (i=0;i<100000;i++) ;

	IICRead((U32)IicTestPtr,TestSize,DataBuff) ;
	for (i=0;i<TestSize;i++) put_byte(DataBuff[i]) ;

	// Next Test Data Gen
	TempData = TestData[TestSize] ;
	TestData[TestSize] = TestData[0] ;
	for (i=0;i<TestSize;i++) TestData[i] = TestData[i+1] ;

	} // End of while loop
}

// IIC Loopback Test 
int IICLoopBackTest(void)
{
	U8 DataBuff[30] ;
	U8 TestData[] = "This is a IIC Test Program  " ; 
	U8 TempData ;
	int i, TestSize, TestLoop ;

	Print("\r >>> IIC Read/Write Test ...  ") ;

	TestLoop = TestSize = sizeof(TestData) ;

	while(TestLoop--) {

	
	IICWrite((U32)IicTestPtr,TestData, TestSize) ;

	for (i=0;i<100000;i++) ;

	IICRead((U32)IicTestPtr,TestSize,DataBuff) ;

	// compare
	for (i=0;i<TestSize;i++) 
		{
		if (TestLoop%4 ==0 ) PrintStatus(0xE,DataBuff[i]) ;
		else if (TestLoop%4 ==1 ) PrintStatus(0xD,DataBuff[i]) ;
		else if (TestLoop%4 ==2 ) PrintStatus(0xB,DataBuff[i]) ;
		else PrintStatus(0x7,DataBuff[i]) ;
		//put_byte(DataBuff[i]) ;
		if (TestData[i] != DataBuff[i]) return 0 ;
		}

	// Next Test Data Gen
	TempData = TestData[TestSize] ;
	TestData[TestSize] = TestData[0] ;
	for (i=0;i<TestSize;i++) TestData[i] = TestData[i+1] ;

	} // End of while loop

	return 1 ;
}

// I2C Setup Routine
void IicSetup(void)
{
	// Reset IIC Controller
	IICCON = IicReset ;
	// Set Prescale Value
	IICPS = Pre_Val ;

	// Enable  IIC Interrupt
	SysSetInterrupt(nIIC_INT,IICisr) ;
	Enable_Int(nIIC_INT) ;			
}

// I2C Interrupt Service Routine
void IICisr(void)
{	
	IICDoneFlag = 1 ;
	Clear_PendingBit(nIIC_INT);
}


// I2C Write Program
void IICWrite(U32 WrAddrPtr, U8 *WriteData, int SizeOfData)
{
	int WriteDataCnt ;	

	// Step1. Setup IICCON Register
	IICCON = Start | ACK | IEN ;

	// Step2. Send Slave Address and Write Command
	IICBUF = SlaveAddr | S_Write ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;

	// Step3. Send Slave Address Pointer 1
	// Addr Ptr 1 : EEPROM Address 1
	IICBUF = S_AddrPtr1 ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;
	
	// Step4. Send Slave Address Pointer 2
	// Addr Ptr 2 : EEPROM Address 2
	IICBUF = WrAddrPtr ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;
 		
	// Step5. Send Multiple Data
	for(WriteDataCnt=0;WriteDataCnt<SizeOfData;WriteDataCnt++)
	{
		IICBUF = WriteData[WriteDataCnt] ;
		while( !IICDoneFlag ) ; IICDoneFlag = 0 ;
	}

	// Step6. Stop IIC Controller
	IICCON = Stop;
}

// IIC Read Program
void IICRead(U32 RdAddrPtr, int ReadDataSize, U8 ReadValue[] )
{
	int ReadCnt ;
	
	// Step1. Setup IICCON Register
	IICCON = Start | ACK | IEN ;

	// Step2. Send Slave Address and Read Command
	IICBUF = SlaveAddr | S_Write ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;

	// Step3. Send Slave Address Pointer 1
	// Addr Ptr 1 : EEPROM Address 1
	IICBUF = S_AddrPtr1 ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;

	// Step4. Send Slave Address Pointer 2
	// Addr Ptr 2 : EEPROM Address 2
	IICBUF = RdAddrPtr ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;

	// Step5. Repeart Start
	IICCON = ReStart ;
	IICCON = Start | ACK | IEN ;
	IICBUF = SlaveAddr | S_Read ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;
	
	// Step6. Receive Multiple Data
	IICCON = ACK | IEN ;
	for (ReadCnt=0 ; ReadCnt < ReadDataSize - 1 ; ReadCnt++)
		{
		while( !IICDoneFlag ) ; IICDoneFlag = 0 ;
		ReadValue[ReadCnt] = IICBUF ;
		}

	// Receive Last Data and ACK Disable
	IICCON = AckDisable | IEN ;
	while( !IICDoneFlag ) ; IICDoneFlag = 0 ;
	ReadValue[ReadCnt] = IICBUF ;

	// Step7. Stop IIC Controller
	IICCON = Stop ;
} 

