;=========================================================================
; KS32C5000 SNDS Board ROM Program
; Program Edit : hbahn, from 1998,1,25 for KS32C5000
;=========================================================================
; The AREA must have the attribute READONLY, otherwise the linker will not
; place it in ROM.
;
; The AREA must have the attribute CODE, otherwise the assembler will not
; let us put any code in this AREA
;
; Note the '|' character is used to surround any symbols which contain
; non standard characters like '!'.
;==========================================================================
	GET memory.a
	GET ks5000.a

	AREA    Init, CODE, READONLY

; --- Define entry point
        EXPORT  __main  ; defined to ensure that C runtime system
__main                  ; is not linked in
        ENTRY
; --- Setup interrupt / exception vectors
    IF :DEF: ROM_AT_ADDRESS_ZERO
; If the ROM is at address 0 this is just a sequence of branches
        B       Reset_Handler
        B       Undefined_Handler
        B       SWI_Handler
        B       Prefetch_Handler
        B       Abort_Handler
        NOP     		; Reserved vector
        B       IRQ_Handler
        B       FIQ_Handler
    ELSE
; Otherwise we copy a sequence of LDR PC instructions over the vectors
; (Note: We copy LDR PC instructions because branch instructions
; could not simply be copied, the offset in the branch instruction
; would have to be modified so that it branched into ROM. Also, a
; branch instructions might not reach if the ROM is at an address
; > 32M).
        MOV     R8, #0
        ADR     R9, Vector_Init_Block
        LDMIA   R9!, {R0-R7}
        STMIA   R8!, {R0-R7}
        LDMIA   R9!, {R0-R7}
        STMIA   R8!, {R0-R7}

; Now fall into the LDR PC, Reset_Addr instruction which will continue
; execution at 'Reset_Handler'

Vector_Init_Block
        LDR     PC, Reset_Addr
        LDR     PC, Undefined_Addr
        LDR     PC, SWI_Addr
        LDR     PC, Prefetch_Addr
        LDR     PC, Abort_Addr
        NOP
        LDR     PC, IRQ_Addr
        LDR     PC, FIQ_Addr

Reset_Addr      DCD     Reset_Handler
Undefined_Addr  DCD     Undefined_Handler
SWI_Addr        DCD     SWI_Handler
Prefetch_Addr   DCD     Prefetch_Handler
Abort_Addr      DCD     Abort_Handler
                DCD     0       	; Reserved vector
IRQ_Addr        DCD     IRQ_Handler
FIQ_Addr        DCD     FIQ_Handler
    ENDIF

;==========================================================
; The Default Exception Handler Vector Entry Pointer Setup
;==========================================================
FIQ_Handler
	SUB	sp, sp, #4
	STMFD	sp!, {r0}
	LDR	r0, =HandleFiq
	LDR	r0, [r0]
	STR	r0, [sp, #4]
	LDMFD	sp!, {r0, pc}

IRQ_Handler
	SUB	sp, sp, #4
	STMFD	sp!, {r0}
	LDR	r0, =HandleIrq
	LDR	r0, [r0]
	STR	r0, [sp, #4]
	LDMFD	sp!, {r0, pc}

Prefetch_Handler
	SUB	sp, sp, #4
	STMFD	sp!, {r0}
	LDR	r0, =HandlePrefetch
	LDR	r0, [r0]
	STR	r0, [sp, #4]
	LDMFD	sp!, {r0, pc}

Abort_Handler
	SUB	sp, sp, #4
	STMFD	sp!, {r0}
	LDR	r0, =HandleAbort
	LDR	r0, [r0]
	STR	r0, [sp, #4]
	LDMFD	sp!, {r0, pc}

Undefined_Handler
	SUB	sp, sp, #4
	STMFD	sp!, {r0}
	LDR	r0, =HandleUndef
	LDR	r0, [r0]
	STR	r0, [sp, #4]
	LDMFD	sp!, {r0, pc}

SWI_Handler
	SUB	sp, sp, #4
	STMFD	sp!, {r0}
	LDR	r0, =HandleSwi
	LDR	r0, [r0]
	STR	r0, [sp, #4]
	LDMFD	sp!, {r0, pc}



	AREA Main, CODE, READONLY

;==========================================================
; The Reset Entry Point
;==========================================================
Reset_Handler
	; Initialize CPSR mode to User Mode
	; Because, CPSR is SVC mode, when reset is asserted,
	; And, setup each stack point
	MRS	r0, cpsr
	BIC	r0, r0, #NoInt | ModeMask
	ORR	r2, r0, #User32Mode	;Change CPSR to User Mode

	ORR	r1, r0, #NoInt | FIQ32Mode
	MSR	cpsr, r1
	MSR	spsr, r2
	LDR	sp, =FiqStack

	ORR	r1, r0, #NoInt | IRQ32Mode
	MSR	cpsr, r1
	MSR	spsr, r2
	LDR	sp, =IrqStack

	ORR	r1, r0, #NoInt | Abort32Mode
	MSR	cpsr, r1
	MSR	spsr, r2
	LDR	sp, =AbortStack

	ORR	r1, r0, #NoInt | Undef32Mode
	MSR	cpsr, r1
	MSR	spsr, r2
	LDR	sp, =UndefStack

	ORR	r1, r0, #NoInt | SVC32Mode
	MSR	cpsr, r1
	MSR	spsr, r2
	LDR	sp, =SVCStack	; Change CPSR to SVC mode

	;==================================================
	; Special Register Configuration	(KS32C5000)
	;==================================================
	LDR	    r0, =0x3FF0000		 
	LDR	    r1, =0x3FFFF91   	; SetValue = 0x3FFFF91
	STR	    r1, [r0]	  	; Cache,WB disable, Stall Enable
                                  	; Start_addr = 0x3FF00000

	;ROM and RAM Configuration(Multiple Load and Store)
    	ADRL    r0, SystemInitData
	LDMIA   r0, {r1-r12}
	LDR	r0, =0x3FF0000 + 0x3010 ; ROMCntr Offset : 0x3010
	STMIA   r0, {r1-r12}

	;=============================
	; Exception Vector Table Setup 
	;=============================
	LDR	r0, =HandleReset	; Exception Vector Table Memory Loc.
	LDR	r1, =ExceptionHandlerTable ; Exception Handler Assign
	MOV	r2, #8			; Number of Exception is 8
ExceptLoop	
	LDR	r3, [r1], #4
	STR	r3, [r0], #4
	SUBS	r2, r2, #1		; Down Count
	BNE	ExceptLoop

	;=====================================
	; LED Display
	;=====================================
	LDR	r1, =IOPMOD
	LDR	r0, =0x1F
	LDR	r0, [r1]

	LDR	r1, =IOPDATA
	LDR	r0, =0x0
	LDR	r0, [r1]

	;=====================================
	; Initialise memory required by C code
	;=====================================
    	IMPORT  |Image$$RO$$Limit|  ; End of ROM code (=start of ROM data)
    	IMPORT  |Image$$RW$$Base|   ; Base of RAM to initialise
    	IMPORT  |Image$$ZI$$Base|   ; Base and limit of area
    	IMPORT  |Image$$ZI$$Limit|  ; to zero initialise

    	LDR  r0, =|Image$$RO$$Limit| ; Get pointer to ROM data
    	LDR  r1, =|Image$$RW$$Base|  ; and RAM copy
    	LDR  r3, =|Image$$ZI$$Base|  ; Zero init base => top of initialised data
    	CMP     r0, r1               ; Check that they are different
    	BEQ     %1
0   	CMP     r1, r3               ; Copy init data
    	LDRCC   r2, [r0], #4
    	STRCC   r2, [r1], #4
    	BCC     %0
1   	LDR     r1, =|Image$$ZI$$Limit| ; Top of zero init segment
    	MOV     r2, #0
2  	CMP     r3, r1               ; Zero init
    	STRCC   r2, [r3], #4
    	BCC     %2
	
	;====================================================
	; Now change to user mode and set up user mode stack.
	;====================================================
	MRS	r0, cpsr
	BIC	r0, r0, #NoInt | ModeMask
	ORR	r1, r0, #User32Mode
    	MSR     cpsr, r0
    	LDR     sp, =UserStack

        
	;===========================
	; Now we enter the C Program
	;===========================
       IMPORT  C_Entry
       BL      C_Entry

;===========================================
; Exception Vector Function Definition
; Consist of function Call from C-Program.
;===========================================
SystemUndefinedHandler
	IMPORT	ISR_UndefHandler
	STMFD	sp!, {r0-r12}
	B	ISR_UndefHandler
	LDMFD	sp!, {r0-r12, pc}^

SystemSwiHandler
	STMFD	sp!, {r0-r12,lr}
	LDR	r0, [lr, #-4]
	BIC	r0, r0, #0xff000000
	CMP	r0, #0xff
	BEQ	MakeSVC
	LDMFD	sp!, {r0-r12, pc}^
MakeSVC
	MRS	r1, spsr
	BIC	r1, r1, #ModeMask
	ORR	r2, r1, #SVC32Mode
	MSR	spsr, r2
	LDMFD	sp!, {r0-r12, pc}^

SystemPrefetchHandler
	IMPORT	ISR_PrefetchHandler
	STMFD	sp!, {r0-r12, lr}
	B	ISR_PrefetchHandler
	LDMFD	sp!, {r0-r12, lr}
	;ADD	sp, sp, #4
	SUBS	pc, lr, #4

SystemAbortHandler
	IMPORT	ISR_AbortHandler
	STMFD	sp!, {r0-r12, lr}
	B	ISR_AbortHandler
	LDMFD	sp!, {r0-r12, lr}
	;ADD	sp, sp, #4
	SUBS	pc, lr, #8

SystemReserv
	SUBS	pc, lr, #4

SystemIrqHandler
	IMPORT	ISR_IrqHandler
	;SUB	sp, sp, #4
	STMFD	sp!, {r0-r12, lr}
	BL	ISR_IrqHandler
	LDMFD	sp!, {r0-r12, lr}
	;ADD	sp, sp, #4
	SUBS	pc, lr, #4

SystemFiqHandler
	IMPORT	ISR_FiqHandler
	;SUB	sp, sp, #4
	STMFD	sp!, {r0-r12, lr}
	BL	ISR_FiqHandler
	LDMFD	sp!, {r0-r12, lr}
	;ADD	sp, sp, #4
	SUBS	pc, lr, #4


	AREA ROMDATA, DATA, READONLY

;===========================================
; Memory System Initialize Data	(KS32C5000)
;===========================================
SystemInitData
	DCD 0x0FFFB7FB	; DRAM1(Half), ROM5(Byte), ROM1(Half), else 32bit
	DCD 0x02000040	; 0x0000000 ~ 0x01FFFFF, ROM0,4Mbit,2cycle
	DCD rROMCON1	; 
	;DCD 0x04008040	; 0x0200000 ~ 0x03FFFFF, ROM1
        DCD 0x06010040	; 0x0400000 ~ 0x05FFFFF, ROM2
	DCD 0x08018040	; 0x0600000 ~ 0x07FFFFF, ROM3
	DCD 0x0A020040	; 0x0800000 ~ 0x09FFFFF, ROM4
	DCD rROMCON5	; 
	;DCD 0x0C028040	; 0x0A00000 ~ 0x0BFFFFF, ROM5
	DCD rDRAMCON0   ; 0x1000000 ~ 0x13FFFFF, DRAM0 4M,
	DCD rDRAMCON1	; 0x1400000 ~ 0x17FFFFF, DRAM1 4M,
	;DCD 0xD80501AC	; 0x1400000 ~ 0x17FFFFF, DRAM1 4M,
	DCD 0x9F0601AC	; 0x1800000 ~ 0x1EFFFFF, DRAM2 16M
	DCD 0xA00701AC	; 0x1C00000 ~ 0x1FFFFFF
	DCD rREFEXTCON  ; External I/O,Refresh

;===========================================
; Exception Handler Vector Table Entry Point
;===========================================
ExceptionHandlerTable
	DCD	UserCodeArea
	DCD	SystemUndefinedHandler
	DCD	SystemSwiHandler
	DCD	SystemPrefetchHandler
	DCD	SystemAbortHandler
	DCD	SystemReserv
	DCD	SystemIrqHandler
	DCD	SystemFiqHandler

	ALIGN

	END
