/********************************************************************/
/*                                                                  */
/*      Interrupt Service Routine for KS32C5000                     */
/*                                                                  */
/*------------------------------------------------------------------*/
/*                                                                  */
/*      Copyright (C) 1997 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*                                                                  */
/*      Modified, programmed by Hbahn                               */
/*                                                                  */
/*      Description : 1997-12-21 first edited                       */ 
/********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "ks5000.h"
#include "pic.h"
#include "uart.h"
#include "pollio.h"
#include "sysconf.h"


/***************************************************************/
/* Dummy Function Prototype for Interrupt Service Vector Table */
/***************************************************************/
static void DummyIsr(void) {}

void (*InterruptHandlers[MAXHNDLRS])(void);


void global_mask_bit(int bit_value) 
{
    if(bit_value) 
          IntMask |= GLOBAL_INT;    // disable all interrupt
    else 
          IntMask &= ~GLOBAL_INT;   // enable all interrupt
}
/* 
 * Global interrupt enable or disable control fucntion 
 * case of 0, interrupt can be disabled globally.
 */

void ClrIntStatus(void)
{
    IntMask = 0x3fffff;  // All interrupt disabled include global bit
    IntPend = 0x1fffff;  // All clear pending
    IntMode = 0x1fffff;  // All FIQ mode

    /* 
     * Interrupt Priority Reset value. Need to be changed priority,
     * Set interrupt priority register. In this case, Global interrupt
     * mask bit must be disabled.
     *
     */ 
    //INTPRI0 = 0x03020100;  
    //INTPRI1 = 0x07060504;
    //INTPRI2 = 0x0b0a0908;
    //INTPRI3 = 0x0f0e0d0c;
    //INTPRI4 = 0x13121110;
    //INTPRI5 = 0x00000014;
    //INTOFFSET =  Read Only Register
}


/******************************************/
/****    Exception Handler Function    ****/
/******************************************/
void	ISR_UndefHandler(REG32 *adr)
{
	//Print("\n** Trap : Undefined Handler\n") ;
	Print("\rUndefined Address : %08x ",adr);
	Print("\rUndefined Data : %08x ",*adr);
}

void	ISR_PrefetchHandler(REG32 *adr)
{
	//Print("\n** Trap : Prefetch Abort Handler\n") ;
	Print("\rPrefetch Abort Address : %08x ",adr);
	Print("\rPrefetch Abort Data : %08x ",*adr);
}

void	ISR_AbortHandler(REG32 *adr)
{
	//Print("\n** Trap : Data Abort Handler\n") ;
	Print("\rData Abort Address : %08x ",adr);
	Print("\rData Abort Data : %08x ",*adr);
}

void	ISR_SwiHandler(void)
{
	Print("\r** Trap : SWI Handler\n") ;
}


void	ISR_IrqHandler(void)
{
       (*InterruptHandlers[INTOFFSET>>2])(); // Call interrupt service routine
	
}


void	ISR_FiqHandler(void)
{
       (*InterruptHandlers[INTOFFSET>>2])(); // Call interrupt service routine
}

/*
void	ISR_FiqHandler(void)
{
	unsigned long isr;
	unsigned long mask;
	unsigned long i;
    
	isr = IntPend & IntMode;
	mask = (~IntMask) ;
	isr &= mask;

	for(i=0; (isr) && (i<MAXHNDLRS); i++) {
		if( isr & 1 ) {
			//Clear_PendingBit(i);  // Interrupt Acknowledge
			(*InterruptHandlers[i])();
		}
		isr >>= 1;
	}

	#if DEBUG
		Print("\r>> FIQ Mode ") ;
		Print("\r   - IntMode (FIQ 1, IRQ 0) : %08x",IntMode) ;
		Print("\r   - IntPend (Pending 1)    : %08x",IntPend) ;
		Print("\r   - IntMask (Serviced 1)   : %08x",IntMask) ;
	#endif
}
*/


/***********************************************************************/
/* InitIntHandlerTable: Initialize the interrupt handler table         */
/*     NOTE(S): This should be called during system initialization     */
/***********************************************************************/
void InitIntHandlerTable(void)
{
	REG32 i;
 
	for (i = 0; i < MAXHNDLRS; i++)
    		InterruptHandlers[i] = DummyIsr;
}

/*********************************************************/
/* SysSetInterrupt: Setup Interrupt Handler Vector Table */
/*********************************************************/
void SysSetInterrupt(REG32 vector, void (*handler)())
{
	InterruptHandlers[vector] = handler;

}

/*********************************************************/
/* InitInterrupt: Initialize Interrupt		 */
/*********************************************************/
void InitInterrupt(void) 
{
        ClrIntStatus();         // Clear All interrupt 
	InitIntHandlerTable() ;
}




/*  =================================================
 *  Monitoring the interrupt related system register
 *  =================================================
 */
void IntConfig(void)
{

        Print("\n========================================\r");
        Print(">>  Interrupt Status configurations. <<\r");
        Print("========================================\r");

        Print(">>IntMode[0x%08x] = 0x%08x\r", &IntMode,IntMode);
        Print(">>IntPend[0x%08x] = 0x%08x\r", &IntPend,IntPend);
        Print(">>IntMask[0x%08x] = 0x%08x\r", &IntMask,IntMask);
        Print(">>INTPRI0[0x%08x] = 0x%08x\r", &INTPRI0,INTPRI0);
        Print(">>INTPRI1[0x%08x] = 0x%08x\r", &INTPRI1,INTPRI1);
        Print(">>INTPRI2[0x%08x] = 0x%08x\r", &INTPRI2,INTPRI2);
        Print(">>INTPRI3[0x%08x] = 0x%08x\r", &INTPRI3,INTPRI3);
        Print(">>INTPRI4[0x%08x] = 0x%08x\r", &INTPRI4,INTPRI4);
        Print(">>INTPRI5[0x%08x] = 0x%08x\r", &INTPRI5,INTPRI5);
        Print(">>INTOFFSET[0x%08x] = 0x%08x\r", &INTOFFSET,INTOFFSET);
        Print("========================================\r\r\r");
}


/*  =================================================
 *   Set interrupt priority function
 *  =================================================
 */

#define nPRIREG  6
void SetPriority(void)
{
     int pReg;
     int bCnt;
     int nPri;
     U32 rINTPRI;
     volatile unsigned int *pINTPRI[6] = {
                &INTPRI0,
                &INTPRI1,
                &INTPRI2,
                &INTPRI3,
                &INTPRI4,
                &INTPRI5
     };
     
     
     ClrIntStatus();  // Clear All interrupt 

     /* Clear Interrupt priority register */
     for(pReg=0;pReg<nPRIREG;pReg++) { 
          *pINTPRI[pReg] = 0x0;
     }

     Print("\r\rAssign Interrupt source to priority!");
     Print("\r>Interrupt source index is two hexdigit.\r\r");

     for(pReg=0,nPri=0;pReg<nPRIREG;pReg++) {
         for(bCnt=0;(bCnt<4)&&(nPri<MAXHNDLRS); bCnt++,nPri++) {
              Print("\rInterrupt Priority %d = ",nPri);
              rINTPRI = gethex2dec(2); get_byte();
              *pINTPRI[pReg] |= (rINTPRI <<(bCnt*8)); 
         }
     }
}


void SetIntMode(void)
{
    U32 rIntMask;
    U32 rIntPend;
    U32 rIntMode;
    
    IntMask = 0x3fffff;  // All interrupt disabled include global bit
    IntMode = 0x1fffff;  // All FIQ mode

    Print("\r> Set IntMode register[6 hexdigit]_");
    rIntMode = gethex2dec(6); get_byte();
    Print("\r  - IntMode input is 0x%08x\r",rIntMode);

    Print("\r> Set IntMask register[6 hexdigit]_");
    rIntMask = gethex2dec(6); get_byte();
    Print("\r  - IntMask input is 0x%08x\r",rIntMask);

    IntMode = rIntMode;
    IntPend = 0x1fffff;  // All clear pending
    IntMask = rIntMask;
    Print("\rAll interrupt pending cleared!!!\r\r");
}
