/*******************************************************************/
/*                                                                 */
/*      Test Program for SNDS Board : KSLIB.c  		   	   */
/*                                                                 */ 
/*-----------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                    */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*      Modified, programmed by hbahn from 1998.02.13              */
/*                                                                 */
/*******************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "std.h"
#include "ks5000.h"
#include "memory.h"
#include "flash.h"
#include "down.h"
#include "pollio.h"
#include "uart.h"
#include "pic.h"
#include "lcd.h"
#include "dma.h"
#include "iic.h"
#include "timer.h"
#include "system.h"
#include "mac.h"
#include "hdlc.h"
#include "kslib.h"
#include "dhry.h"

DIAGCMD gDiagCmdTable[MAX_NUMOF_COMMAND] = {
"help",		HelpCmdLi,	4, "Help of command line interface. \0",
"ls",		ListOfCmd,	2, "List of command line interface COMMAND.\0",
"mc",		MCopyCli,	2, "Memory copy source to destinination\0",
"mf",		MFillCli,	2, "Memory pattern fill to Target Address\0",
"md",		MDumpCli,	2, "Memory dump\0",
"search",	MSearchCli,	6, "Memory search\0",
"compare",	MCompCli,	7, "Memory Compare\0",
"memtest",	MemTestCli,	7, "Memory read/write test\0",
"dmatest",	DmaTestCli,	7, "DMA test for long/short/byte\0",
"dcopy",	DCopyCli,	5, "DMA copy source to destinination\0",
"cache",	CacheCmdLi,	5, "Cache ON/OFF \0",
"mac",		MacCmdLi,	3, "MAC Test Function, Use [mac help] command\0",
"hdlc",		HdlcCmdLi,	3, "HDLC Test Function, Use [hdlc help] command\0",
"iic",		IicCmdLi,	3, "I2C read/write test command\0",
"all",		AllTestCmdLi,	3, "All SNDS board function test\0",
"down",		DownPgmCmdLi,	4, "Download User Program\0",
"fuse",		FusePgmCmdLi,	4, "Fusing EEPROM\0",
"go",		GoPgmCmdLi,	2, "Start download user program\0",
"start",	StartCmdLi,	5, "Start User Program (Area : 0x1000000) \0",
"boot",		BootCmdLi,	4, "Boot pSOS at ROM Bank 1 user EEPROM\0",
"reset",	ResetCmdLi,	5, "Reset SNDS, Jump to Reset handler\0",
"exit",		ExitCmdLi,	4, "Exit command line interface to MENU mode\0"
} ;

extern TIME tm0,tm1 ;
AllErrorReport AllErrorRpt={0,0,0,0,0,0} ;
U32 AllTestResult = 0 ;


void EtcTest(void) 
{
	DiagCmdLi() ;

}

/* 
 * Function : DiagCmdLi(void)
 * Description : Command Line Interface Diag
 */
void DiagCmdLi(void) 
{
	U32 error;
	U32 comlen;
	U32 CmdOffset = 0 ;
	char commandline[MAX_LENGTHOF_COMMANDLINE]; // Up to 256 character 

	InitSnds() ;

	Print("\n >>> ------------------------------------------- <<<") ;
	Print("\r >>> SNDS Command Line Interface for Diagonistic <<<") ;
        Print("\r >>>    - KS32C5000 SNDS B'd Diag Ver 1.3 -      <<<") ;
        Print("\r >>>    Samsung Electronic Co. NETCOM  Div.      <<<") ;
        Print("\r >>>    Programmed by hbahn, from 1998.4.28      <<<") ;
	Print("\r >>> ------------------------------------------- <<<\n") ;

	while(CMDLI_TRUE)
	{
		ClearLcdDisplay() ;
		lPrint("CMD LINE I/F ... ") ;
		Print("SNDS(%d)> ",CmdOffset); // diag prompt 
		comlen = read_string(commandline);
		if((commandline[0] !=NL)&&(commandline[0]!= CR)) 
				// if only enter key input , pass ..
		{
			// if command length is over 256 bytes .
			if(comlen > MAX_LENGTHOF_COMMANDLINE) 
			{
				ReportError(ERROR_TOO_LONG_COMMAND);
				continue;
			}
			error = RunCommand(commandline,comlen);
			//if(error) ReportError(error);
			CmdOffset++ ;
		}
	}
}	

/* 
 * Function : U32 RunCommand(char *commandline, U32 comlinelen)
 * Description : Command Running
 */
U32 RunCommand(char *commandline, U32 comlinelen)
{
	U32 comindex;		
	U32 argc  ;
	char argv[MAX_NUMOF_ARGS][MAX_LENGTHOF_ARGS] = {{0,},};

	argc = ParseCommandLine(argv,commandline); // return arguments number
	comindex = LookUpCommand(argv[0]); // argv[0] is command 

	if (gDiagCmdTable[comindex].cmd_action == NULL)
		return ERROR_BAD_COMMAND; 
		// return error code, command not found 

	// Run the command & return error code
	return (gDiagCmdTable[comindex].cmd_action)(argc,argv); 
}


/*
 * Function :  LookUpCommand
 * Description : Search the command and return the index of the command.
 */
U32 LookUpCommand(char* cmd)
{
	U32 tableindex = 0;
	while( tableindex < MAX_NUMOF_COMMAND)
	{
		if ((gDiagCmdTable[tableindex].cmd_action != NULL)&&
			(!str_ncmp(cmd, gDiagCmdTable[tableindex].cmd_name , gDiagCmdTable[tableindex].cmd_length)))
			break;
		tableindex++;
	}
	return tableindex;
}

/*
 * 	Function :  ParseCommandLine
 *	Description : 
 */
U32 ParseCommandLine(char (*argv)[MAX_LENGTHOF_ARGS],char *commandline)
{
	U32 count = 1;
	U32 offset = 0;
	U32 i,j;
	char cmd;

	for(j = 0 ; j < MAX_NUMOF_ARGS ; j++)
	{
		for(i = 0 ; i < MAX_LENGTHOF_ARGS ; i++)
		{
			if((cmd = commandline[offset]) == SPACE)
			{
				offset++;
				count++;
				argv[j][i] = '\0';
				break;
			}


			switch(cmd)
			{
			case NL: argv[j][i] = '\0';
				 return count;
			case CR: argv[j][i] = '\0';
				 return count;
			default: argv[j][i] = cmd;
				 // Clear Used command line buffer
			 	 commandline[offset] = NL ;
			 	 offset++;
				 break;
			}
		}
	}

	return count ;
}


/*
 * 	Function :  ReportError
 *	Description :
 */
void ReportError(U32 err)
{
	Print("Error Code :%x\n",err);
}

/*
 * 	Function :  str_ncmp
 *	Description : String Compare
 */
U32 str_ncmp(char* str1 , char* str2 , U32 n)
{
	U32 i = 0;
	while(i < n)
	{	
		if(str1[i] != str2[i]) return  1;
		i++;
	}
	return	0;
}

/*
 * 	Function :  strtoint
 *	Description : String Value to Interger Value
 */
U32 strtoint(char *str, U32 count)
{
	U32 temp;
	U32 total = 0 ;
	
	while(count)
	{
		temp = *str++ - '0'; 
		total += temp * power(10,count-1);
		count--;
	}
	return total; 
}

/*
 * 	Function :  hexstrtohex
 *	Description : String Value to HEX Value
 */
U32 hexstrtohex(char *str, U32 count)
{
	U32 temp;
	U32 total = 0 ;
		
	while(count)
	{
		count--;
		if(('0'<=(*str))&&((*str) <= '9'))
		{
			temp = *str - '0';
		    total += temp << (4*count);
		    temp = 0;
		}
		else if(('a'<=(*str))&&((*str) <= 'f'))
		{
			switch(*str)
			{
				case 'a': temp = 10; break;
				case 'b': temp = 11; break;
				case 'c': temp = 12; break;
				case 'd': temp = 13; break;
				case 'e': temp = 14; break;
				case 'f': temp = 15; break;
				default: break;
			}
			total +=temp << (4*count);
			temp = 0;
		}
		str++;
	}
	return total;
}

/*
 * 	Function :  sizeofstr
 *	Description : Size of String
 */
U32 sizeofstr(char *str)
{
	U32 size = 0;
	while((*str != '\0')&&(*str != NL)&&(*str != CR))
	{
		str++;
		size++;
	}
	return size;
}

/*
 * 	Function :  power
 *	Description : 
 */
U32 power(U32 base, U32 cha)
{
	U32 i ,temp;
	temp = 1;
	i=0;
	while(i < cha)
	{
		temp = base * temp;
		i++;
	}
	return temp;
}

/*
 * 	Function :  PrintHelpMessage
 *	Description :   
 */
void PrintHelpMessage(char *cmd)
{
	U8 index;
	index = LookUpCommand(cmd);
	put_string(gDiagCmdTable[index].cmd_help);
	Print("\r") ;
}

/*
 * 	Function :  ListOfCmd
 *	Description : List of Command line interface Command
 */
U32 ListOfCmd(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS])
{
	int index ;

	if (argc == 2)
		HelpCmdLi(1,NULL) ; 
	else
	{
		for (index = 0 ; index < MAX_NUMOF_COMMAND ; index++) 
		{
			Print("%-10s",gDiagCmdTable[index].cmd_name);
			if (index%6 == 5) Print("\r") ;
		}
		Print("\n") ;
	}

	return NO_ERROR;
}

/*
 * 	Function :  HelpCmdLi
 *	Description : Help of Command line interface 
 */
U32 HelpCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS])
{
	int index ;

	Print("\n ===========================================================") ;
	Print("\r        SNDS Command line interface HELP     ") ;
	Print("\r ===========================================================") ;
	for (index = 0 ; index < MAX_NUMOF_COMMAND ; index++) 
	{
		Print("\r %-10s :  ",gDiagCmdTable[index].cmd_name);
		Print("%s ",gDiagCmdTable[index].cmd_help);
	}
	Print("\r ===========================================================\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  MCopyCli, MFillCli,MDumpCli,MSearchCli
 *	Description : Memory Copy, Memory Fill, Memory Dump & Search 
 */
U32 MCopyCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32  src, dst,size ;
	U32 *lsrc, *ldst ;
	U16 *hsrc, *hdst ;
	U8  *bsrc, *bdst ;
	U8 bytesize ;

	if((argc != 5))
	{
		PrintHelpMessage("mc");
		Print("Useage: mc <b/h/l> <SrcAddr> <DestinAddr> <Size> \n") ;
		return NO_ERROR;
	}		

	bytesize = (U8)*argv[1] ;
	src  = hexstrtohex(argv[2], sizeofstr(argv[2]));
	dst  = hexstrtohex(argv[3], sizeofstr(argv[3]));
	size = hexstrtohex(argv[4], sizeofstr(argv[4]));

	if (bytesize == MSIZE_BYTE)
		{
		bsrc  = (U8 *)src ;
		bdst  = (U8 *)dst ;
		bcopy(bsrc, bdst, size) ;
		}
	else if (bytesize == MSIZE_HALF)
		{
		hsrc  = (U16 *)src ;
		hdst  = (U16 *)dst ;
		scopy(hsrc, hdst, size) ;
		}
	else if (bytesize == MSIZE_LONG) 
		{
		lsrc  = (U32 *)src ;
		ldst  = (U32 *)dst ;
		wcopy(lsrc, ldst, size) ;
		}
	else Print("Useage: mc <b/h/l> <SrcAddr> <DestinAddr> <Size> \n") ;

	return NO_ERROR;
}

U32 MFillCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 *src, cnt, pattern ;

	if((argc != 4))
	{
		PrintHelpMessage("mf");
		Print("Useage: mf <TargetAddr> <Size> <Pattern> \n") ;
		return NO_ERROR;
	}		

	src = (U32 *)hexstrtohex(argv[1], sizeofstr(argv[1]));
	cnt = hexstrtohex(argv[2], sizeofstr(argv[2]));
	pattern = hexstrtohex(argv[3], sizeofstr(argv[3]));
	
	while(cnt--) *src++ = pattern ;

	return NO_ERROR;
}

U32 MDumpCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 *base,*end,baseptr,size ;

	if((argc != 3))
	{
		PrintHelpMessage("md");
		Print("Useage: md <TargetAddr> <Size> \n") ;
		return NO_ERROR;
	}		

	baseptr  = hexstrtohex(argv[1], sizeofstr(argv[1]));
	base = (U32 *)baseptr ;
	size = hexstrtohex(argv[2], sizeofstr(argv[2]));
	end  = (U32 *)(baseptr + size) ;
	Print("\n $$ Memory Dump from 0x%x to 0x%x",base,end) ;

	MemDump(base, end);

	Print("\n") ;

	return NO_ERROR;
}

U32 MSearchCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 *src,*dst,pattern ;
	int cnt ;

	if((argc != 4))
	{
	PrintHelpMessage("search");
	Print("Useage: search <SrcAddr> <DestinAddr> <SearchPattern> \n") ;
	return NO_ERROR;
	}		

	src  = (U32 *)hexstrtohex(argv[1], sizeofstr(argv[1]));
	dst  = (U32 *)hexstrtohex(argv[2], sizeofstr(argv[2]));
	pattern  = hexstrtohex(argv[3], sizeofstr(argv[3]));

	cnt = (int)(dst - src) ;

	Print("\n $$ Search 0x%x, from 0x%x to 0x%x \n", \
		pattern, src,dst) ;

	while(cnt--) 
	{
	if (*src == pattern) 
		Print("\r Searched at 0x%x",src) ;
		src++ ;
	}

	Print("\n") ;

	return NO_ERROR;
}

U32 MCompCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32  src, dst,size ;
	U32 *lsrc, *ldst ;
	U16 *hsrc, *hdst ;
	U8  *bsrc, *bdst ;
	U8 bytesize ;

	if((argc != 5))
	{
	PrintHelpMessage("mc");
	Print("Useage: compare <b/h/l> <SrcAddr> <DestinAddr> <Size> \n") ;
	return NO_ERROR;
	}		

	bytesize = (U8)*argv[1] ;
	src  = hexstrtohex(argv[2], sizeofstr(argv[2]));
	dst  = hexstrtohex(argv[3], sizeofstr(argv[3]));
	size = hexstrtohex(argv[4], sizeofstr(argv[4]));

	if (bytesize == MSIZE_BYTE)
		{
		bsrc  = (U8 *)src ;
		bdst  = (U8 *)dst ;
		if (!bcomp(bsrc, bdst, size))
			{
			Print("\r $$$ Byte Compare FAIL !!!\n") ;
			return NO_ERROR;
			} 
		}
	else if (bytesize == MSIZE_HALF)
		{
		hsrc  = (U16 *)src ;
		hdst  = (U16 *)dst ;
		if (!scomp(hsrc, hdst, size))
			{
			Print("\r $$$ Short Compare FAIL !!!\n") ;
			return NO_ERROR;
			} 
		}
	else if (bytesize == MSIZE_LONG) 
		{
		lsrc  = (U32 *)src ;
		ldst  = (U32 *)dst ;
		if (!wcomp(lsrc, ldst, size))
			{
			Print("\r $$$ Word Compare FAIL !!!\n") ;
			return NO_ERROR;
			} 
		}
	else Print("Useage: compare <b/h/l> <SrcAddr> <DestinAddr> <Size> \n") ;
	Print("\n $$$ Compare OK !\n") ;

	return NO_ERROR;
}


U32 MemTestCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
        U32 *src;    // Memory source address
        U32 *dst;    // Memory destination address
        U32 tsize;   // Memory Test size
        int lsize;   // memory test loop size

	if((argc != 5))
	{
	PrintHelpMessage("memtest");
	Print("Useage: memtest <SrcAddr> <DestinAddr> <Size> <LoopCount>\n") ;
	return NO_ERROR;
	}		

	ClearLcdDisplay() ;
	lPrint("MEM Test ..") ;

	src   = (U32 *)hexstrtohex(argv[1], sizeofstr(argv[1]));
	dst   = (U32 *)hexstrtohex(argv[2], sizeofstr(argv[2]));
	tsize = hexstrtohex(argv[3], sizeofstr(argv[3]));
	lsize = strtoint(argv[4], sizeofstr(argv[4]));

	Print("\r - Source Memory Location      : 0x%08x",src) ;
	Print("\r - Destination Memory Location : 0x%08x",dst) ;
	Print("\r - Memory Test Size            : 0x%08x",tsize) ;
	Print("\r - Memory Test Loop Count      : %d\r",lsize) ;

        if ( !MemTest(src,dst,tsize,lsize) ) 
		{
		Print("\n $$$ Memory Test Fail !!!") ;
		lPrint("FAIL.") ;
		}
	else 
		{
		Print("\n $$$ Memory Test Success !!! ") ;
		lPrint("Ok.") ;
		}

	Print("\n") ;
	return NO_ERROR;
}

U32 DmaTestCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
        U32 src;    // DMA source address
        U32 dst;    // DMA destination address
        U32 tsize;  // DMA Test size
        int lsize;  // DMA test loop size

	if((argc != 5))
	{
	PrintHelpMessage("dmatest");
	Print("Useage: dmatest <SrcAddr> <DestinAddr> <Size> <LoopCount>\n") ;
	return NO_ERROR;
	}		

	ClearLcdDisplay() ;
	lPrint("DMA Test ..") ;
        GdmaReset(0); 
        GdmaReset(1); 
        GdmaIntEnable(0);
        GdmaIntEnable(1);

	src   = hexstrtohex(argv[1], sizeofstr(argv[1]));
	dst   = hexstrtohex(argv[2], sizeofstr(argv[2]));
	tsize = hexstrtohex(argv[3], sizeofstr(argv[3]));
	lsize = strtoint(argv[4], sizeofstr(argv[4]));

	Print("\r - Source Memory Location      : 0x%08x",src) ;
	Print("\r - Destination Memory Location : 0x%08x",dst) ;
	Print("\r - DMA Test Size               : 0x%08x",tsize) ;
	Print("\r - DMA Test Loop Count         : %d\r",lsize) ;

        if ( !DmaAutoTest(src,dst,tsize,lsize) ) 
		{
		Print("\n $$$ DMA Test FAIL !!!") ;
		lPrint("FAIL.") ;
		}
	else 
		{
		Print("\n $$$ DMA Test PASS !!! ") ;
		lPrint("Ok.") ;
		}

        GdmaIntDisable(0);
        GdmaIntDisable(1);
	Print("\n") ;
	return NO_ERROR;
}

U32 DCopyCli(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32  src, dst,size ;
	U8 bytesize ;

	if((argc != 5))
	{
		PrintHelpMessage("dcopy");
		Print("Useage: dcopy <b/h/l> <SrcAddr> <DestinAddr> <Size> \n") ;
		return NO_ERROR;
	}		

        GdmaReset(0); 
        GdmaIntEnable(0);

	bytesize = (U8)*argv[1] ;
	src  = hexstrtohex(argv[2], sizeofstr(argv[2]));
	dst  = hexstrtohex(argv[3], sizeofstr(argv[3]));
	size = hexstrtohex(argv[4], sizeofstr(argv[4]));

	if (bytesize == MSIZE_BYTE)
		dcopy(src, dst, size,TxBYTE) ;
	else if (bytesize == MSIZE_HALF)
		dcopy(src, dst, size,TxHWORD) ;
	else if (bytesize == MSIZE_LONG) 
		dcopy(src, dst, size,TxWORD) ;
	else Print("Useage: dcopy <b/h/l> <SrcAddr> <DestinAddr> <Size> \n") ;

        GdmaIntDisable(0);
	Print("\n") ;

	return NO_ERROR;
}

U32 CacheCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32  Reg, n ;

	if((argc < 2))
	{
	PrintHelpMessage("cache");
	Print("Useage: cache <on/off/conf> <4/8>\n") ;
	return NO_ERROR;
	}		

	n = hexstrtohex(argv[2], sizeofstr(argv[2]));

	if (!str_ncmp(argv[1],"on",sizeofstr(argv[1]))) 
		{
        	SYSCFG &= ~STALL; 	// Stall Disable
        	SYSCFG |= WRITE_BUFF;  	// Write Buffer Enable
     		SYSCFG &= ~CACHE;      	// Disable cache before 
					// cache mode change  
     		Reg = (~CACHE_MODE & SYSCFG); // clear previous CM bit
		// Cache Mode Change
		if (n==4) 
        		Reg |= (CACHE_MODE & (0<<4)); 
		else if (n==8) 
        		Reg |= (CACHE_MODE & (1<<4)); 
      		SYSCFG = Reg;
     		CacheFlush();   // Tag ram clear before cache enable
	        SYSCFG |= CACHE; 
		}

	else if (!str_ncmp(argv[1],"off",sizeofstr(argv[1]))) 
		{
        	SYSCFG &= ~WRITE_BUFF;  // Write Buffer Enable
     		SYSCFG &= ~CACHE;      	// Disable cache before 
     		Reg = (~CACHE_MODE & SYSCFG); // clear previous CM bit
		// Cache Mode Change
        	Reg |= (CACHE_MODE & (2<<4)); 
      		SYSCFG = Reg;
		}
	else if (!str_ncmp(argv[1],"conf",sizeofstr(argv[1]))) 
			CacheConfig() ;
	
	
	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  MacCmdLi
 *	Description : MAC Test Program for Command Line 
 */
U32 MacCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS])
{
	
	//U8 *FirstTestMode, *SecondTestMode ;
	char *maccmd[] = {"loop","rx","tx","capture","show","addr","help"} ;
	U32 cmdindex ;

	if((argc < 2))
	{
		PrintHelpMessage("mac");
		return NO_ERROR;
	}		

	ClearLcdDisplay() ;
	lPrint("MAC Test ..") ;
	for (cmdindex =0 ; cmdindex < 7 ; cmdindex++)
	{
		if (!str_ncmp(maccmd[cmdindex],argv[1],sizeofstr(argv[1]))) 
					break ;
	}

	switch (cmdindex)
	{
	case 0 : // Loop Test
		if (!str_ncmp(argv[2],"all",sizeofstr(argv[2]))) 
			MacAllLoopBack() ;
		else if (!str_ncmp(argv[2],"internal",sizeofstr(argv[2]))) 
			MacInternalLoopBack() ;
		else if (!str_ncmp(argv[2],"phy",sizeofstr(argv[2]))) 
			{
			PhyNoColLoopBack() ;
			PhyColLoopBack() ;
			}
		else if (!str_ncmp(argv[2],"ether",sizeofstr(argv[2]))) 
			PhysicalLoopBack() ;
		else if (!str_ncmp(argv[2],"poll",sizeofstr(argv[2]))) 
			MacPollLoopBack(); 
		break ;
	case 1 : // Rx
		RxMacFrame() ; break ;
	case 2 : // Tx
		if (!str_ncmp(argv[2],"single",sizeofstr(argv[2]))) 
			SingleTransfer(); 
		else if (!str_ncmp(argv[2],"multi",sizeofstr(argv[2]))) 
			MultiTransfer(); 
		else if (!str_ncmp(argv[2],"control",sizeofstr(argv[2]))) 
			ControlFrameTransfer(); 
		break ;
	case 3 : // Capture
		if (!str_ncmp(argv[2],"all",sizeofstr(argv[2]))) 
				CaptureAllRxFrame() ; 
		else if (!str_ncmp(argv[2],"my",sizeofstr(argv[2]))) 
				CaptureMyRxFrame() ; 
		else if (!str_ncmp(argv[2],"broad",sizeofstr(argv[2]))) 
				CaptureBroadFrame() ; 
		else if (!str_ncmp(argv[2],"control",sizeofstr(argv[2]))) 
				CaptureCntlFrame() ; 
		else if (!str_ncmp(argv[2],"destin",sizeofstr(argv[2]))) 
				CaptureMatchedDest() ; 
		break ;
	case 4 : // Show
		ShowRecevedFrameData() ; break ;
		break ;
	case 5 : // MAC Physical Address Setup
		if (!str_ncmp(argv[2],"read",sizeofstr(argv[2]))) 
				MacAddrRead() ;
		else if (!str_ncmp(argv[2],"write",sizeofstr(argv[2]))) 
				MacAddrWrite() ;
		break ;
	case 6 : // MAC Help
		Print("\nUseage: mac <Cmd 1> <Cmd 2>") ;
		Print("\r<Cmd 1>") ;
		Print("\r 1. loop    : MAC Loopback test function") ;
		Print("\r    <Cmd 2> ") ;
		Print("\r       1) all : internal,phy loopback test") ;
		Print("\r       2) internal : MAC internal loopback test") ;
		Print("\r       3) phy : MAC-to-PHY loopback test") ;
		Print("\r       4) ether : loopback by physical ethernet cable") ;
		Print("\r       5) poll : Polling Mode loopback test") ;
		Print("\r 2. rx      : receive MAC frame and display by one packet") ;
		Print("\r 3. tx      : transfer MAC frame") ;
		Print("\r    <Cmd 2> ") ;
		Print("\r       1) single : Transfer single frame") ;
		Print("\r       2) multi : Transfer single frame ") ;
		Print("\r       3) control : Transfer control frame") ;
		Print("\r 4. capture : Capture received MAC frame and store") ;
		Print("\r              you can show received frame by [show] command") ;
		Print("\r    <Cmd 2> ") ;
		Print("\r       1) all : Capture all frame") ;
		Print("\r       2) my : Capture my mac address frame ") ;
		Print("\r       3) broad : Capture broadcast frame") ;
		Print("\r       4) control : Capture control frame") ;
		Print("\r       5) destin : Capture matched destination address frame");
		Print("\r 5. show    : Showing captured frame") ;
		Print("\r 6. addr    : MAC Address change and view in IIC EEPROM") ;
		Print("\r    <Cmd 2> ") ;
		Print("\r       1) read  : Read MAC Address") ;
		Print("\r       2) write : Write MAC Address") ;
		Print("\r 7. help    : this message") ;
		break ;
	default  : 
		Print("\r Invalid MAC test item is selected") ; 
		break ;
	}


	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  HdlcCmdLi
 *	Description : Hdlc Test Program for Command Line 
 */
U32 HdlcCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS])
{
	
	char *hdlccmd[] = {"loop","rx","tx","capture","show","addr","help"} ;
	U32 cmdindex ;

	if((argc < 2))
	{
		PrintHelpMessage("hdlc");
		return NO_ERROR;
	}		

	ClearLcdDisplay() ;
	lPrint("HDLC Test ..") ;
	for (cmdindex =0 ; cmdindex < 7 ; cmdindex++)
	{
		if (!str_ncmp(hdlccmd[cmdindex],argv[1],sizeofstr(argv[1]))) 
					break ;
	}

	switch (cmdindex)
	{
	case 0 : // Loop Test
		if (!str_ncmp(argv[2],"internal",sizeofstr(argv[2]))) 
			HdlcInternalLoopback() ;
		break ;
	case 1 : // Rx
		break ;
	case 2 : // Tx
		break ;
	case 3 : // Capture
		break ;
	case 4 : // Show
		break ;
	case 5 : // HDLC Station Address Setup
		break ;
	case 6 : // HDLC Help
		Print("\nUseage: hdlc <Cmd 1> <Cmd 2>") ;
		Print("\r<Cmd 1>") ;
		Print("\r 1. loop    : HDLC Loopback test function") ;
		Print("\r    <Cmd 2> ") ;
		Print("\r       1) all : All HDLC loopback test") ;
		Print("\r       2) internal : HDLC internal loopback test") ;
		Print("\r 2. rx      : receive HDLC frame and display by one packet") ;
		Print("\r 3. tx      : transfer HDLC frame") ;
		Print("\r    <Cmd 2> ") ;
		Print("\r       1) single : Transfer single frame") ;
		Print("\r       2) multi : Transfer single frame ") ;
		Print("\r 4. capture : Capture received HDLC frame and store") ;
		Print("\r              you can show received frame by [show] command") ;
		Print("\r 5. show    : Showing captured frame") ;
		Print("\r 6. addr    : HDLC Address change and view in IIC EEPROM") ;
		Print("\r 7. help    : this message") ;
		break ;
	default  : 
		Print("\r Invalid HDLC test item is selected") ; 
		break ;
	}


	Print("\n") ;
	return NO_ERROR;
}


/*
 * 	Function :  IicCmdLi
 *	Description : Iic Test Program for Command Line 
 */
U32 IicCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS])
{
	if((argc != 2))
	{
		PrintHelpMessage("iic");
		Print("Useage: iic <read/write/auto> \n") ;
		return NO_ERROR;
	}		

	ClearLcdDisplay() ;
	lPrint("IIC Test ..") ;

	if (!str_ncmp(argv[1],"read",sizeofstr(argv[1]))) 
		IICReadTest() ; 
	else if (!str_ncmp(argv[1],"write",sizeofstr(argv[1]))) 
		IICWriteTest() ;
	else if (!str_ncmp(argv[1],"auto",sizeofstr(argv[1]))) 
		IICAutoTest() ; 

	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  AllTestCmdLi
 *	Description : All SNDS function test
 */
U32 AllTestCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 lcnt ;

	if((argc != 3))
	{
		PrintHelpMessage("all");
		Print("Useage: all test [lcnt]\n") ;
		return NO_ERROR;
	}		

	lcnt = strtoint(argv[2], sizeofstr(argv[2]));

	ClearLcdDisplay() ;
	lPrint("All Test ..") ;

	// TIMER 0/1 Initialize
	tmReset(TIMER0) ; tmReset(TIMER1) ;
	ClrTimeVar() ;

	Rtc_init(TIMER0) ; Rtc_init(TIMER1) ;
        TimerStart(TIMER0); TimerStart(TIMER1);

	AllTestInternal(lcnt) ;
	if  (AllTestResult != 0 ) 
		{
		Print("\n $$$ All Test Failed !!!!!") ;
		Print("\n $$$ Error Occured Count : %d ", AllTestResult) ;
		IOPDATA = 0x1F ;
		lPrint(" Fail.") ;
		}
	else
		{
		Print("\n $$$ All Test Passed !!!!!") ;
		lPrint(" Finished.") ;
		IOPDATA = 0x0A ;
		}

	AllTestResult = 0 ;

	ClrTimeVar() ;

	return NO_ERROR;
}

/*
 * 	Function :  AllTestInternal
 *	Description : All KS32C5000 Internal function test
 */
U32 AllTestInternal(U32 loopcnt)
{
	U32 Reg , i ;
	AllErrorReport AllErrorRptNoCache={0,0,0,0,0,0} ;
	AllErrorReport AllErrorRpt4KCache={0,0,0,0,0,0} ;
	AllErrorReport AllErrorRpt8KCache={0,0,0,0,0,0} ;


   	for (i=0; i<loopcnt ; i++ ) {

	Print("\n\n========================================") ;
	Print("\r       All Test with Cache OFF") ;
	Print("\r========================================") ;

	AllTestFunc() ;
	AllErrorRptNoCache = AllErrorRpt ;
	// Clear Error Report
	AllErrorRpt.MemError = 0 ; AllErrorRpt.UartError = 0 ;
	AllErrorRpt.TimerError = 0 ; AllErrorRpt.DmaError = 0 ;
	AllErrorRpt.IicError = 0 ; AllErrorRpt.MacError = 0 ;
	AllErrorRpt.HdlcError = 0 ; 

	Print("\n\n========================================") ;
	Print("\r       All Test with 4K Cache ON") ;
	Print("\r========================================") ;

        SYSCFG |= WRITE_BUFF;  	// Write Buffer Enable
     	SYSCFG &= ~CACHE;      	// Disable cache before 
     	Reg = (~CACHE_MODE & SYSCFG); // clear previous CM bit
        Reg |= (CACHE_MODE & (0<<4)); // Cache Mode Change
      	SYSCFG = Reg;
     	CacheFlush();   // Tag ram clear before cache enable
	SYSCFG |= CACHE; 

	AllTestFunc() ;
	AllErrorRpt4KCache = AllErrorRpt ;
	// Clear Error Report
	AllErrorRpt.MemError = 0 ; AllErrorRpt.UartError = 0 ;
	AllErrorRpt.TimerError = 0 ; AllErrorRpt.DmaError = 0 ;
	AllErrorRpt.IicError = 0 ; AllErrorRpt.MacError = 0 ;
	AllErrorRpt.HdlcError = 0 ; 

	Print("\n\n========================================") ;
	Print("\r       All Test with 8K Cache ON") ;
	Print("\r========================================") ;

        SYSCFG |= WRITE_BUFF;  	// Write Buffer Enable
     	SYSCFG &= ~CACHE;      	// Disable cache before 
     	Reg = (~CACHE_MODE & SYSCFG); // clear previous CM bit
        Reg |= (CACHE_MODE & (1<<4)); // Cache Mode Change
      	SYSCFG = Reg;
     	CacheFlush();   // Tag ram clear before cache enable
	SYSCFG |= CACHE; 

	AllTestFunc() ;
	AllErrorRpt8KCache = AllErrorRpt ;

	Print("\n\n $ All Test Finished !!!!! ") ; 

        SYSCFG |= WRITE_BUFF;  	// Write Buffer Enable
     	SYSCFG &= ~CACHE;      	// Disable cache before 
	IOPDATA = 0xF ;

	Print("\n\n========================================") ;
	Print("\r       All Test Error Report") ;
	Print("\r========================================") ;
	Print("\r $$ Cache OFF Test") ;
		if (AllErrorRptNoCache.MemError) {
			AllTestResult++ ;
			Print("\r  $$$ Memory Test FAIL !") ; }
		else if (AllErrorRptNoCache.UartError) {
			AllTestResult++ ;
			Print("\r  $$$ UART Test FAIL !") ; }
		else if (AllErrorRptNoCache.TimerError) {
			AllTestResult++ ;
			Print("\r  $$$ TIMER Test FAIL !") ; }
		else if (AllErrorRptNoCache.DmaError) {
			AllTestResult++ ;
			Print("\r  $$$ DMA Test FAIL !") ; }
		else if (AllErrorRptNoCache.IicError) {
			AllTestResult++ ;
			Print("\r  $$$ IIC Test FAIL !") ; }
		else if (AllErrorRptNoCache.MacError) {
			AllTestResult++ ;
			Print("\r  $$$ MAC Test FAIL !") ; }
		else if (AllErrorRptNoCache.HdlcError) {
			AllTestResult++ ;
			Print("\r  $$$ HDLC Test FAIL !") ; }
		else
			Print(" : PASS !!!") ;

	Print("\r----------------------------------------") ;
	Print("\r $$ 4K Cache Test") ;
		if (AllErrorRpt4KCache.MemError) {
			AllTestResult++ ;
			Print("\r  $$$ Memory Test FAIL !") ; }
		else if (AllErrorRpt4KCache.UartError) {
			AllTestResult++ ;
			Print("\r  $$$ UART Test FAIL !") ; }
		else if (AllErrorRpt4KCache.TimerError) {
			AllTestResult++ ;
			Print("\r  $$$ TIMER Test FAIL !") ; }
		else if (AllErrorRpt4KCache.DmaError) {
			AllTestResult++ ;
			Print("\r  $$$ DMA Test FAIL !") ; }
		else if (AllErrorRpt4KCache.IicError) {
			AllTestResult++ ;
			Print("\r  $$$ IIC Test FAIL !") ; }
		else if (AllErrorRpt4KCache.MacError) {
			AllTestResult++ ;
			Print("\r  $$$ MAC Test FAIL !") ; }
		else if (AllErrorRpt4KCache.HdlcError) {
			AllTestResult++ ;
			Print("\r  $$$ HDLC Test FAIL !") ; }
		else
			Print("  : PASS !!!") ;
	Print("\r----------------------------------------") ;
	Print("\r $$ 8K Cache Test") ;
		if (AllErrorRpt8KCache.MemError) {
			AllTestResult++ ;
			Print("\r  $$$ Memory Test FAIL !") ; }
		else if (AllErrorRpt8KCache.UartError) {
			AllTestResult++ ;
			Print("\r  $$$ UART Test FAIL !") ; }
		else if (AllErrorRpt8KCache.TimerError) {
			AllTestResult++ ;
			Print("\r  $$$ TIMER Test FAIL !") ; }
		else if (AllErrorRpt8KCache.DmaError) {
			AllTestResult++ ;
			Print("\r  $$$ DMA Test FAIL !") ; }
		else if (AllErrorRpt8KCache.IicError) {
			AllTestResult++ ;
			Print("\r  $$$ IIC Test FAIL !") ; }
		else if (AllErrorRpt8KCache.MacError) {
			AllTestResult++ ;
			Print("\r  $$$ MAC Test FAIL !") ; }
		else if (AllErrorRpt8KCache.HdlcError) {
			AllTestResult++ ;
			Print("\r  $$$ HDLC Test FAIL !") ; }
		else
			Print("  : PASS !!!") ;
	Print("\r========================================\n") ;

	DisplayTime(TIMER0) ; 
	Print("\r") ;
	DisplayTime('1') ;
	Print("\n") ;

   	}

	return NO_ERROR;
}


/*
 * 	Function : All Test Program
 *	Description : Test of all function
 */
U32 AllTestFunc(void)
{
        U32 *src = (U32 *)0x1100000 ;   // Memory source address
        U32 *dst = (U32 *)0x1200000 ;   // Memory destination address
        U32 tsize = 0x10000 ;   	// Memory Test size
        int lsize = 1 ;   		// memory test loop size

	// ------------------------------------------------
	// MEMORY Test
	Print("\n1. Memory Test .... ") ;
        if ( !MemTest(src,dst,tsize,lsize) ) 
		{
		AllErrorRpt.MemError = 1 ;
		Print("\n $$$ Memory Test FAIL !!! ") ;
		}
	else Print("\n $$$ Memory Test PASS !!! ") ;

	// ------------------------------------------------
	// UART Test
	Print("\n2. UART Test ....  ") ;
	if (!UARTAutoLoopBack())
		{
		AllErrorRpt.UartError = 1 ;
		Print("\n $$$ UART Test FAIL !!! ") ;
		}
	else Print("\n $$$ UART Test PASS !!! ") ;

	// ------------------------------------------------
	// GDMA Channel 0/1 Test
	Print("\n3. GDMA Channel 0/1 Test .... ") ;
        GdmaReset(0); 
        GdmaReset(1); 
        GdmaIntEnable(0);
        GdmaIntEnable(1);

        if ( !DmaAutoTest(0x1100000,0x1200000,0x10000,10) ) 
		{
		AllErrorRpt.DmaError = 1 ;
		Print("\n $$$ DMA Test FAIL !!!") ;
		}
	else 
		{
		Print("\n $$$ DMA Test PASS !!! ") ;
		}
        GdmaIntDisable(0);
        GdmaIntDisable(1);

	// ------------------------------------------------
	// IIC Test 
	Print("\n4. IIC Test .... ") ;
	if ( !IICLoopBackTest() )
		{
		AllErrorRpt.IicError = 1 ;
		Print("\n $$$ IIC Test FAIL !!! ") ;
		}
	else Print("\n $$$ IIC Test PASS !!! ") ;

	// ------------------------------------------------
	// MAC Test 
	Print("\n5. MAC Test .... ") ;
	MACCON |= MACLoop ; // set Loopback bit(4)
	if ( !InterruptLoopBackTest() ) 
		{
		AllErrorRpt.MacError = 1 ;
		Print("\n $$$ MAC Test FAIL !!! ") ;
		}
	else Print("\n $$$ MAC Test PASS !!! ") ;
	MACCON &= ~MACLoop ; // Disable Loopback bit(4)
	ClearErrReport() ;

	// ------------------------------------------------
	// HDLC Test 
	Print("\n6. HDLC Test .... ") ;
	if ( !HdlcInternalLoopback() ) 
		{
		AllErrorRpt.HdlcError = 1 ;
		Print("\n $$$ HDLC Test FAIL !!! ") ;
		}
	else Print("\n $$$ HDLC Test PASS !!! ") ;

	return 1 ;

}


/*
 * 	Function :  DownCmdLi
 *	Description : Download Commandline interface
 */
U32 DownPgmCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 RcvDataSize ;
	char *downarea ;

	if((argc != 2))
	{
		PrintHelpMessage("down");
		Print("Useage: down <Target Address>\n") ;
		return NO_ERROR;
	}		

	IOPDATA = 0x0 ;
	ClearLcdDisplay() ;
	lPrint("Download ..") ;

	downarea   = (char *)hexstrtohex(argv[1], sizeofstr(argv[1]));

	Print("\r $$ Download Target Address : 0x%08x",downarea) ;
	Print("\r $$ Waiting Download .... ") ; 

	RcvDataSize=GetDataSize(CONSOLE)  ;
	ReceiveData(downarea,RcvDataSize) ;
	if(CRCErrorCheck(downarea,RcvDataSize)) {
		Print("Ok !") ;
		Print("\r $$ Download Program Size : %d Byte",RcvDataSize) ; 
		}
        else Print("FAIL !!!");

	IOPDATA = 0xF ;
	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  FusePgmCmdLi
 *	Description : Start Download user program 
 */
U32 FusePgmCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 *Src ;
	U32 *Dst ;
	U32 Size ;

	if((argc != 4))
	{
		PrintHelpMessage("fuse");
		Print("Useage: fuse <Download Address> <Target Address> <Size>\n") ;
		return NO_ERROR;
	}		

	IOPDATA = 0x0 ;
	ClearLcdDisplay() ;
	lPrint("Fusing EEPROM ..") ;

	Src  = (U32 *)hexstrtohex(argv[1], sizeofstr(argv[1]));
	Dst  = (U32 *)hexstrtohex(argv[2], sizeofstr(argv[2]));
	Size = strtoint(argv[3], sizeofstr(argv[3]));
	Print("\n>> Source : %08x, Destination : %08x, Size : %d", \
			Src,Dst,Size) ;

	Program_SST29EE(Src,Dst,Size); 

	IOPDATA = 0xF ;

	Print("\n+--------------------------------------------+") ;
	Print("\r| Press 'Reset' Button in SNDS board ....... |") ;
	Print("\r+--------------------------------------------+") ;
	Print("\n") ;

	return NO_ERROR;
}

/*
 * 	Function :  GoCmdLi
 *	Description : Start Download user program 
 */
U32 GoPgmCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	U32 *downarea ;

	if((argc != 2))
	{
		PrintHelpMessage("go");
		Print("Useage: go <Target Address>\n") ;
		return NO_ERROR;
	}		

	downarea   = (U32 *)hexstrtohex(argv[1], sizeofstr(argv[1]));
	StartDownPgm(downarea) ;

	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  StartCmdLi
 *	Description : Start Download user program 
 */
U32 StartCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	if((argc != 1))
	{
		PrintHelpMessage("start");
		Print("Useage: go <Target Address>\n") ;
		return NO_ERROR;
	}		

	StartUserPgm() ;

	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  BootCmdLi
 *	Description : Start pSOS in User EEPROM(in ROM Bank 1)
 */
U32 BootCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	if((argc != 1))
	{
		PrintHelpMessage("boot");
		Print("Useage: boot\n") ;
		return NO_ERROR;
	}		

	StartDownPgm( (U32 *)0x200000 ) ;

	return NO_ERROR;
}


/*
 * 	Function :  ExitCmdLi
 *	Description : Start Download user program 
 */
U32 ExitCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{

	// Call Menu mode diag program
	Diag() ;

	Print("\n") ;
	return NO_ERROR;
}

/*
 * 	Function :  ResetCmdLi
 *	Description : Reset SNDS, Jump to Reset Handler
 */
U32 ResetCmdLi(U8 argc, char (*argv)[MAX_LENGTHOF_ARGS]) 
{
	if((argc != 1))
	{
		PrintHelpMessage("reset");
		Print("Useage: reset\n") ;
		return NO_ERROR;
	}

	Reset_Boot_Rom() ;
	Print("\n") ;
	return NO_ERROR;
}
