/*********************************************************************/
/*                                                                   */
/*      LCD Test Program for KS32C5000                               */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                      */
/*-------------------------------------------------------------------*/
/*                                                                   */
/*      Modified, programmed by Hbahn 			             */
/*                                                                   */
/*      Description : 1998-3-20 first edited for KS32C5000           */
/*********************************************************************/
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "ks5000.h"
#include "uart.h"
#include "pollio.h"
#include "lcd.h"
#include "sysconf.h"


//volatile U32 LCD_Addr_Base ;

/*
 * Function : LCDTest
 * Description : LCD Test
 */
void LCDTest(void) 
{
	int i ;


	Print("\r -- Waiting Key in .. ") ;
	Print("\r Received Data : ") ;
	get_key() ;
	//put_byte(get_key()) ;

/*
	Print("\r >> LCD Test <<") ;
	LcdUsrModeSet() ;
	//lPut_byte(get_byte()) ;
	Print("\r >> Press Any Key to Quit LCD Test ") ;
	do 
	{
		LTestPutString("+- KS32C5000 SNDS Demo Board .....    ")  ;
	}while( !kbd_hit() ) ;
	Print("\r >> LCD Test Finished") ;
	LCDInit() ;
	lPrint("LCD Test End..") ;
*/

}

void LTestPutString(char *ptr) 
{
    while(*ptr )
	{
        lPut_byte(*ptr++ );
	WaitLcd(5000);
	}
}


/*
 * Function : LCDInit
 * Description : Initialize LCD 
 */
void LCDInit(void) 
{
	// Initialize EXTDBWTH : 8Byte Operation
	// Initialize EXTACON0 
	EXTDBWTH &= 0xFFCFFFFF ;
	EXTDBWTH |= DSX0 ; 
	EXTACON0 = LTCOS0 | LTACS0 | LTCOH0 | LTACC0 ; 

	// Initialize
	WaitLcd(200) ;

	/* ---
	 * Function Set
	 *     Interface Data Length is : 8 bit
	 *     Display Numer of Line is 2
	 * --- */
	LInstRegWrite((U8)0x38) ; WaitLcd(100) ; 

	/* ---
	 * Display On/Off Control
	 *     Display On
	 *     Cursor Display
	 *     Cursor Blink
	 * --- */
	LInstRegWrite((U8)0x0F) ; WaitLcd(100) ; 

	/* ---
	 * Clear Display
	 * --- */
	LInstRegWrite((U8)0x01) ; WaitLcd(100) ; 

	/* ---
	 * Cursor or Display Shift
	 *     Shift Left
	 * --- */
	LInstRegWrite((U8)0x18) ; WaitLcd(100) ; 

	/* ---
	 * Entry Mode Set
	 *     Increment DD Ram Address
	 * --- */
	LInstRegWrite((U8)0x06) ; WaitLcd(100) ; 

	ClearLcdDisplay() ;
}

/*
 * Function : LInstRegWrite
 * Description : LCD Control Register Write
 */
void LInstRegWrite(U8 DATA) 
{ 
	U8 *LcdInstWrAddr ;
	LcdInstWrAddr = (U8 *)LINSTWR ;
	*LcdInstWrAddr=  DATA ; 
}

/*
 * Function : LInstRegRead, LDataRegRead
 * Description : LCD Control Register Write
 */
U8 LInstRegRead(void) 
{
	U8 ReadData ;
	ReadData = (U8)LINSTRD ;
	return ReadData ;
}


U8 LDataRegRead(void) 
{
	U8 ReadData ;
	ReadData = (U8)LDATARD ;
	return ReadData ;
}

/*
 * Function : WaitLcd
 * Description : LCD Control Register Write
 */
void WaitLcd(int milisec)
{
	int second ;

	if (!CACHE_ENABLED) second = milisec * 40 ;
	else second = milisec * 200 ;

	while(second--) ;
}

/*
 * Function : LcdBusyCheck
 * Description : Wait Until LCD Busy Flag is Cleared
 */
void LcdBusyCheck(void) 
{
	U8 ReadData ;
	ReadData = (U8)LINSTRD ;
	while (!ReadData) ;
}

/*
 * Function : Mv2LcdLine
 * Description : Move Second LCD Line
 */
void Mv2LcdLine(void)
{
	// Set DD Ram Address to 
	LInstRegWrite((U8)0xA8) ; WaitLcd(100) ; 
}

/*
 * Function : ClearLcdDisplay
 * Description : Clear LCD Displayed Character
 */
void ClearLcdDisplay(void)
{
	LInstRegWrite((U8)0x01) ; WaitLcd(100) ; 
	lPrint("-KS32C5000 SNDS-") ;
	Mv2LcdLine() ; // Change LCD Line to 2'th Line
}


/*
 * Function : lPut_byte
 * Description : LCD Byte Data Write
 */
void lPut_byte(U8 DATA) 
{ 
	U8 *LcdDataWrAddr ;
	LcdDataWrAddr = (U8 *)LDATAWR ;
	*LcdDataWrAddr=  DATA ; 
	WaitLcd(3) ; 
}

/*
 * Function : lPrint
 * Description : Wait Until LCD Busy Flag is Cleared
 */
void lPrint(char *ptr ) 
{
    while(*ptr )
        lPut_byte(*ptr++ );
}

/*
 * Function : LcdUsrModeSet
 * Description : User Mode LCD Display
 *               Shifted Display
 */
void LcdUsrModeSet() 
{
	LInstRegWrite((U8)0x30) ; WaitLcd(100) ; 
	LInstRegWrite((U8)0x01) ; WaitLcd(100) ; // Clear
	LInstRegWrite((U8)0x8F) ; WaitLcd(100) ; 
	/* ---
	 * Entry Mode Set
	 *     Increment DD Ram Address
	 *     Shift Character
	 * --- */
	LInstRegWrite((U8)0x07) ; WaitLcd(100) ; 
}


