/********************************************************************/
/*       LCD Header for KS32C5000                                   */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*      Modified, programmed by hbahn                               */
/*                                                                  */
/*      Description : 1998-2-10 first edited for KS32C5000          */ 
/********************************************************************/
#ifndef _LCD_
#define _LCD_

// Bus Width of External I/O Bank 0
#define DSX0	(0x01 << 20)

// External I/O Bank 0 Control
/*
#define TCOS0	(0x111 << 0) 
#define TACS0	(0x110 << 3) 
#define TCOH0	(0x010 << 6) 
#define TACC0	(0x111 << 9) 
*/
#define LTCOS0	(0x7 << 0) 
#define LTACS0	(0x6 << 3) 
#define LTCOH0	(0x2 << 6) 
#define LTACC0	(0x7 << 9) 

#define Lcd_Inst_Reg	0x0 
#define Lcd_Data_Reg	0x2 
#define Lcd_Reg_Write 	0x0 
#define Lcd_Reg_Read 	0x1 

// LCD Register Read/Write Addr
#define LCD_Addr_Base 	 ( (REFEXTCON & 0x3FF) << 16 )
#define LINSTWR  (LCD_Addr_Base+Lcd_Inst_Reg+Lcd_Reg_Write)
#define LINSTRD  (LCD_Addr_Base+Lcd_Inst_Reg+Lcd_Reg_Read)
#define LDATAWR  (LCD_Addr_Base+Lcd_Data_Reg+Lcd_Reg_Write)
#define LDATARD  (LCD_Addr_Base+Lcd_Data_Reg+Lcd_Reg_Read)

#define CACHE_ENABLED 	(SPSTR & 0x2) 


/*
 * Function : LCDTest
 * Description : LCD Test
 */
extern void LCDTest(void) ;
extern void LTestPutString(char *ptr)  ;

/*
 * Function : LCDInit
 * Description : Initialize LCD 
 */
extern void LCDInit(void) ;

/*
 * Function : LInstRegWrite
 * Description : LCD Control Register Write
 */
extern void LInstRegWrite(U8 DATA)  ;

/*
 * Function : LInstRegRead, LDataRegRead
 * Description : LCD Control Register Write
 */
extern U8 LInstRegRead(void)  ;
extern U8 LDataRegRead(void)  ;

/*
 * Function : WaitLcd
 * Description : LCD Control Register Write
 */
extern void WaitLcd(int milisec) ;
extern void LcdBusyCheck(void)  ;

/*
 * Function : ClearLcdDisplay
 * Description : Clear LCD Displayed Character
 */
extern void ClearLcdDisplay(void) ;

/*
 * Function : Mv2LcdLine
 * Description : Move Second LCD Line
 */
extern void Mv2LcdLine(void) ;


/*
 * Function : lPut_byte
 * Description : LCD Control Register Write
 */
extern void lPut_byte(U8 DATA)  ;

/*
 * Function : lPrint
 * Description : Wait Until LCD Busy Flag is Cleared
 */
extern void lPrint(char *ptr ) ;

/*
 * Function : LcdUsrModeSet
 * Description : User Mode LCD Display
 *               Shifted Display
 */
extern void LcdUsrModeSet(void) ;

#endif
