/*********************************************************************/
/*                                                                   */
/*      MAC Test Program for KS32C5000                               */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                      */
/*-------------------------------------------------------------------*/
/*                                                                   */
/*      Modified, programmed by Hbahn 			             */
/*                                                                   */
/*      Description : 1998-2-10 first edited for KS32C5000           */
/*********************************************************************/
/*   MAC.c  :  Main function of MAC Test Program                     */
/*********************************************************************/
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "memory.h"
#include "system.h"
#include "kslib.h"
#include "mac.h"


// Global Variable for MAC/BDMA and CAM Configuration
extern volatile U32 gMacCon ;
extern volatile U32 gMacTxCon ;
extern volatile U32 gMacRxCon ;
extern volatile U32 gBdmaTxCon ;
extern volatile U32 gBdmaRxCon ;
extern volatile U32 gCamCon ;
extern volatile U32 CntlPktTxDone ; // Control Packet Tx Done Flag
extern volatile U32 gCam0_Addr0 , gCam0_Addr1 ;

extern volatile U32 gCRxFDPtr ;
extern volatile U32 RxFrameUserAreaBase ;
extern volatile U32 RxFrameUserAreaLimit ;

volatile U32 gCapture_Addr0= 0  , gCapture_Addr1 = 0  ;

// Global Variable for Timer
extern TIME tm0 ;

/*
 * Function : void MacTest(void)
 * Description : Main function for MAC Block test
 */
void MacTest(void)
{
	char TestItemSel ;

	ResetPhyChip() ;
	ClearPhyChip() ;
	MacInitialize() ;
	ClearErrReport() ;

	do {

	// All LED On
	IOPDATA = 0x0 ; 

	Print("\n+---------------------------------------------------+\r") ;
	Print("|    >>>>> MAC Block Diagonistic Program <<<<<      |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.04.17)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| Capture & Show Ethernet Frame               - [C] |\r") ;
	Print("| MAC Loopback Test                           - [L] |\r") ;
	Print("| Transfer Ethernet Frame                     - [T] |\r") ;
	Print("| Receive Ethernet Frame                      - [R] |\r") ;
	Print("| Other MAC Function Test                     - [O] |\r") ;
	Print("| Change & View MAC Configuration             - [V] |\r") ;
	Print("| Quit Ethernet Test                          - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;

	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'C' : case 'c' : ShowFrame() ; break ;
		case 'L' : case 'l' : MacLoopBackTest() ; break ;
		case 'T' : case 't' : TxMacFrame() ; break ;
		case 'R' : case 'r' : RxMacFrame() ; break ;
		case 'O' : case 'o' : MacEtcTest() ; break ;
		case 'V' : case 'v' : MacConfig() ; break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	// LED Off
	IOPDATA = 0x0F ; 
	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
 		MacInitialize() ; // Reinitialize MAC/BDMA for Normal 
                            // Operation
		Print("\nPress any key to Continue MAC Test") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}


/*
 * Function : void ShowFrame(void)
 * Description : Show Ethernet Frame and Capture Selected Framre
 */
void ShowFrame(void)
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|    ***** Capture & ShowFrame MAC Frame *****      |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.04.17)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| Capture All MAC Frame to Buffer Full        - [A] |\r") ;
	Print("| Capture My MAC Address Frame                - [M] |\r") ;
	Print("| Capture Broadcast Frame                     - [B] |\r") ;
	Print("| Capture Control Frame                       - [C] |\r") ;
	Print("| Capture Matched Destination Address         - [D] |\r") ;
	Print("| Show Captured MAC Frame                     - [S] |\r") ;
	Print("| View & Set User Frame Area                  - [U] |\r") ;
	Print("| Quit Show & Capture                         - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'A' : case 'a' : CaptureAllRxFrame() ; break ;
		case 'M' : case 'm' : CaptureMyRxFrame() ; break ;
		case 'B' : case 'b' : CaptureBroadFrame() ; break ;
		case 'C' : case 'c' : CaptureCntlFrame() ; break ;
		case 'D' : case 'd' : CaptureMatchedDest() ; break ;
		case 'S' : case 's' : ShowRecevedFrameData() ; break ;
		case 'U' : case 'u' : SetUserFrameArea() ; break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}

/*
 * Function : void CaptureAllRxFrame(void)
 * Description : Capture All Rx Frame
 */
void CaptureAllRxFrame(void)
{
	Print("\n >>>>> Capture All Rx Frame Data <<<<< ") ;
	ClearUserFrameArea() ;
	gCapture_Addr0 = 0x0 ; gCapture_Addr1 = 0x0 ;
	gCamCon = CompEn | StationAcc | GroupAcc | BroadAcc | NegCAM ;
	CaptureRxFrame() ;
	gCamCon = CompEn ;
}

/*
 * Function : void CaptureMyRxFrame(void)
 * Description : Capture My Rx Frame
 */
void CaptureMyRxFrame(void)
{
	Print("\n >>>>> Capture My Rx Frame Data <<<<< ") ;
	ClearUserFrameArea() ;
	gCapture_Addr0 = gCam0_Addr0 ; 
	gCapture_Addr1 = gCam0_Addr1 ;
	CaptureRxFrame() ;
}

/*
 * Function : void CaptureBroadFrame(void)
 * Description : Capture Broadcast Rx Frame
 */
void CaptureBroadFrame(void)
{
	Print("\n >>>>> Capture Broadcast Rx Frame Data <<<<< ") ;
	ClearUserFrameArea() ;
	gCapture_Addr0 = gCam0_Addr0 ; 
	gCapture_Addr1 = gCam0_Addr1 ;
	gCamCon = BroadAcc ;
	CaptureRxFrame() ;
	gCamCon = CompEn ;
}

/*
 * Function : void CaptureCntlFrame(void)
 * Description : Capture Control Rx Frame
 */
void CaptureCntlFrame(void)
{
	Print("\n >>>>> Capture Control Rx Frame Data <<<<< ") ;
	ClearUserFrameArea() ;
	gCapture_Addr0 = 0x0 ; gCapture_Addr1 = 0x0 ;
	gCamCon = CompEn | StationAcc | GroupAcc | BroadAcc | NegCAM ;
	gBdmaRxCon = BRxDIE | BRxEn | BRxBig | BRxMAINC | BRxBRST ; 
	gMacRxCon = RxEn | EnAlign | EnCRCErr | EnOver | EnLongErr | \
			EnRxPar | PassCtl  ;
	CaptureRxFrame() ;
	gMacRxCon = RxEn | EnAlign | EnCRCErr | EnOver | \
			EnLongErr | EnRxPar  ;
	gBdmaRxCon = BRxDIE | BRxEn | BRxBig | BRxMAINC | BRxBRST ;
	gCamCon = CompEn ;
}

/*
 * Function : void CaptureMatchedDest(void)
 * Description : Capture Matched Destination Address Rx Frame
 */
void CaptureMatchedDest(void)
{
	U8 RcvData[12] ;
	int i, MacAddrSize = 12 ;

	Print("\n >>>>> Capture Matched Destination Rx Frame Data <<<<< ") ;
	ClearUserFrameArea() ;

	Print("\r $$ Enter Destination Address : ") ;
	i = 0 ;
	gCapture_Addr0 = 0x0 ; gCapture_Addr1 = 0x0 ;

	while ( (RcvData[i] = get_byte() ) != CR ) 
	{
		if ( (RcvData[i] >= '0') && (RcvData[i] <= '9') )
			RcvData[i] -= '0' ;
		else if ( (RcvData[i] >= 'A') && (RcvData[i] <= 'F') )
			RcvData[i] = RcvData[i] - 'A' + 10 ;
		else if ( (RcvData[i] >= 'a') && (RcvData[i] <= 'f') )
			RcvData[i] = RcvData[i] - 'a' + 10 ;
		i++ ;
	}

	// Get First of MAC Address
	for (i=0;i<MacAddrSize-4;i++) 
		gCapture_Addr0 = (gCapture_Addr0 << 4) | RcvData[i] ;

	// Get Second of MAC Address
	for (i=MacAddrSize-4;i<MacAddrSize;i++)
		gCapture_Addr1 = (gCapture_Addr1 << 4) | RcvData[i] ;
	gCapture_Addr1 = (gCapture_Addr1 << 16)  ;

	Print("\r >>> Destination Address to Receive : ") ;
	for (i=0;i<MacAddrSize;i++)  Print("%01x ",RcvData[i]) ;
	put_byte('\n') ;

	CaptureRxFrame() ;
}


/*
 * Function : void SetUserFrameArea(void)
 * Description : Set User Frame Area Base & Limit
 */
void SetUserFrameArea(void)
{
	char SetYesOrNo ;
	Print("\n >>> User Area Base  Address : %x", \
					RxFrameUserAreaBase) ; 
	Print("\r >>> User Area Limit Address : %x", \
					RxFrameUserAreaLimit + 0x10 ) ; 

	Print("\n $$$ Set User Area Base (Y/N) ? ") ;
	SetYesOrNo = get_byte() ;
	if ( (SetYesOrNo == 'Y') || (SetYesOrNo == 'y') )
	{
		Print("\r $$ Enter User Area Base : 0x") ;
		RxFrameUserAreaBase = get_num() ;
	}

	Print("\n $$$ Set User Area Limit (Y/N) ? ") ;
	SetYesOrNo = get_byte() ;
	if ( (SetYesOrNo == 'Y') || (SetYesOrNo == 'y') )
	{
		Print("\r $$ Enter User Area Limit : 0x") ;
		RxFrameUserAreaLimit = get_num() - 0x10 ;
	}
}

/*
 * Function : void MacLoopBackTest(void)
 * Description : MAC LoopBack Test
 */
void MacLoopBackTest(void)
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|          ***** MAC LoopBack Test *****            |\r") ;
        Print("|          Rev 1.0 (by hbahn, 1998.04.17)           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| MAC Internal LoopBackTest                   - [I] |\r") ;
	Print("| MAC - PHY LoopBackTest(Without Collision)   - [L] |\r") ;
	Print("| MAC - PHY LoopBackTest(WithCollision)       - [C] |\r") ;
	Print("| Physical Ethernet LoopBackTest              - [E] |\r") ;
	Print("| Polling LoopBackTest                        - [P] |\r") ;
	Print("| All LoopBackTest(Without Physical LoopBack) - [A] |\r") ;
	Print("| Quit MAC LoopBack Test                      - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'I' : case 'i' : MacInternalLoopBack(); break ;
		case 'L' : case 'l' : PhyNoColLoopBack(); break ;
		case 'C' : case 'c' : PhyColLoopBack(); break ;
		case 'E' : case 'e' : PhysicalLoopBack(); break ;
		case 'P' : case 'p' : MacPollLoopBack(); break ;
		case 'A' : case 'a' : MacAllLoopBack(); break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}

/*
 * Function : void MacInternalLoopBack(void)
 * Description : MAC LoopBack Test
 */
void MacInternalLoopBack(void)
{
	Print("\n >>> MAC Internal Loopback Test <<<") ;
	MACCON |= MACLoop ; // set Loopback bit(4)
	InterruptLoopBackTest() ;
	MACCON &= ~MACLoop ; // Disable Loopback bit(4)
	ReadErrReport() ;
	ClearErrReport() ;
}

/*
 * Function : void PhyNoColLoopBack(void)
 * Description : Loopback Test by PHY(ICS1890) without collision
 */
void PhyNoColLoopBack(void)
{
   	MiiStationWrite(PHY_CNTL_REG, PHYHWADDR, PHY_FULLDUPLEX | \
			ENABLE_LOOPBACK) ;

	Print("\n >>> MAC - PHY LoopBackTest(Without Collision) <<<") ;
	InterruptLoopBackTest() ;
	ClearPhyChip() ;
	ReadErrReport() ;
	ClearErrReport() ;
}

/*
 * Function : void PhyColLoopBack(void)
 * Description : Loopback Test by PHY(ICS1890) without collision
 */
void PhyColLoopBack(void)
{
   	MiiStationWrite(PHY_CNTL_REG, PHYHWADDR, PHY_FULLDUPLEX | \
			PHY_COL_TEST | ENABLE_LOOPBACK) ;
	Print("\n >>> MAC - PHY LoopBackTest(With Collision) <<<") ;
	InterruptLoopBackTest() ;
	ClearPhyChip() ;
	ReadErrReport() ;
	ClearErrReport() ;
}

/*
 * Function : void PhysicalLoopBack(void)
 * Description : MAC Physical LoopBack Test(Require Loopback Cable) 
 */
void PhysicalLoopBack(void)
{
	Print("\n >>> MAC Physical Loopback Test(Require Loopback Cable) <<<");
	InterruptLoopBackTest() ;
	ReadErrReport() ;
	ClearErrReport() ;
}

/*
 * Function : void MacPollLoopBack(void)
 * Description : MAC Polling LoopBack Test
 */
void MacPollLoopBack(void)
{
	int LoopCnt = 5 ;

	Disable_Int(nMAC_RX_INT) ;
	Disable_Int(nMAC_TX_INT) ;
	Disable_Int(nBDMA_TX_INT) ;	
	Disable_Int(nBDMA_RX_INT) ;	
	MACCON |= MACLoop ; // set Loopback bit(4)

	Print("\n ++ Polling Loopback Test ++") ;
	while( LoopCnt-- ) PollingLoopBackCheck() ;
	MACCON &= ~MACLoop ; // Disable Loopback bit(4)

	Clear_PendingBit(nMAC_RX_INT) ;
	Clear_PendingBit(nMAC_TX_INT) ;
	Clear_PendingBit(nBDMA_TX_INT) ;	
	Clear_PendingBit(nBDMA_RX_INT) ;	
	Enable_Int(nMAC_RX_INT) ;
	Enable_Int(nMAC_TX_INT) ;
	Enable_Int(nBDMA_TX_INT) ;	
	Enable_Int(nBDMA_RX_INT) ;	
}

/*
 * Function : void MacAllLoopBack(void)
 * Description : MAC All LoopBack Test
 */
void MacAllLoopBack(void)
{
	int LoopCnt, LoopCntValue, GetStringCnt ;
	char GetString[12] ;

	Print("\n >>> All LoopBackTest(Without Physical LoopBack) <<<") ;
	Print("\r $$$ Enter Loopback Test Count : ") ;
	
	GetStringCnt = read_string(GetString);
	LoopCntValue = strtoint(GetString,GetStringCnt) ;

	if (LoopCntValue == 0 ) LoopCntValue = 1 ;
	LoopCnt=LoopCntValue ;
	Print("\r $$$ Loopback Test Count is %d ",LoopCnt) ;

	Rtc_init(TIMER0) ;
        TimerStart(TIMER0);

	while( LoopCnt-- )
	{
	Print("\n\n+++++ %d'th Loopback Test +++++",LoopCntValue-LoopCnt) ;

	MacInternalLoopBack() ;
	PhyNoColLoopBack() ;
	PhyColLoopBack() ;
	}

	Timer0Stop() ;
	tm0.tick = 0 ; tm0.min = 0 ; tm0.hour = 0 ;
}

/*
 * Function : void TxMacFrame(void)
 * Description : Transfer MAC Frame
 */
void TxMacFrame(void)
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|          ***** Transfer MAC Frame *****           |\r") ;
        Print("|          Rev 1.0 (by hbahn,  1998.04.17)          |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| Transfer Single MAC Frame                   - [S] |\r") ;
	Print("| Transfer Multiple MAC Frame                 - [M] |\r") ;
	Print("| Transfer Control Frame                      - [C] |\r") ;
	Print("| Quit MAC Transfer Test                      - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'S' : case 's' : SingleTransfer(); break ;
		case 'M' : case 'm' : MultiTransfer(); break ;
		case 'C' : case 'c' : ControlFrameTransfer(); break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}

/*
 * Function : SingleTransfer
 * Description : Transfer a Frame Data to another Host
 */
void SingleTransfer(void) 
{
	char TransferPacket ;

	do 
	{
	Print("\r   $$ Select Transmit(T) or Quit(Q) ? ") ;
	TransferPacket = get_byte() ;

	if ( (TransferPacket == 'T') || (TransferPacket == 't') )
		{
		TxFrameDataInit() ;
		MacTxGo() ;
		}
	} while ( (TransferPacket != 'Q') && (TransferPacket != 'q') ) ;
	ReadErrReport() ;
	ClearErrReport() ;
}

/*
 * Function : MultiTransfer
 * Description : Transfer Multiple Frame Data to another Host
 */
void MultiTransfer(void) 
{
	int i,TxFrameCount ;

	do 
	{
	Print("\r   $$ Enter Transmit Frame  Count : ") ;
	TxFrameCount = get_digit() ;
	for (i=0 ; i< TxFrameCount ; i++)
		{
		TxFrameDataInit() ;
		MacTxGo() ;
		}
	} while (TxFrameCount != 0)  ;
	ReadErrReport() ;
	ClearErrReport() ;
}

/*
 * Function : ControlFrameTransfer
 * Description : Transfer Control Frame Data to another Host
 */
void ControlFrameTransfer(void) 
{
	char TransferPacket ;

	gBdmaTxCon  |= BTxCPIE ;
	ReadyMacTx() ;

	do 
	{
	Print("\r   $$ Select Transmit(T) or Quit(Q) ? ") ;
	TransferPacket = get_byte() ;
	if ( (TransferPacket == 'T') || (TransferPacket == 't') )
		{

		// CAM #0 : 0000f0110000 (Destin Addr)
		// CAM #1 : 11f000000000 (Source Addr)
		VPint(CAM_BaseAddr) = 0x0000f011 ;	
		VPint(CAM_BaseAddr + 0x4) = 0x000011f0 ;	
		VPint(CAM_BaseAddr + 0x8) = 0x00000000 ;	

		// CAM #18 : Opcode & Operand
		VPint(CAM_BaseAddr + 0x6c) = 0x88080001 ;	
		// CAM #18 : Pause Count Value
		// CAM #19 - #20 : Filled with Zero
		VPint(CAM_BaseAddr + 0x70) = 0x12340000 ;	
		VPint(CAM_BaseAddr + 0x74) = 0x00000000 ;	
		VPint(CAM_BaseAddr + 0x78) = 0x00000000 ;	
		VPint(CAM_BaseAddr + 0x7c) = 0x00000000 ;	

		// CAM Enable
		CAMEN = 0x1c0003 ; 

		// SendPause by Write to MACTXCON
		MACTXCON |= SdPause | TxEn ;

		while(!CntlPktTxDone) ;
		CntlPktTxDone  = 0 ;
		
		}
	} while ( (TransferPacket != 'Q') && (TransferPacket != 'q') ) ;

	ReadErrReport() ;
	ClearErrReport() ;

}

/*
 * Function : void RxMacFrame(void)
 * Description : Receive one MAC Frame if Destination address is mine
 */
void RxMacFrame(void)
{
	SysSetInterrupt(nBDMA_RX_INT, BdmaRxForSingleIsr) ;	

	Print("\n >>> Waiting For Frame Receive \r") ;

	while(get_byte() != 'q') ;

	ReadErrReport() ;
	ClearErrReport() ;
	SysSetInterrupt(nBDMA_RX_INT, BDMA_Rx_isr) ;	
}

/*
 * Function : void MacEtcTest(void)
 * Description : Other MAC Function
 */
void MacEtcTest(void)
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|        ***** Other MAC Function Test *****        |\r") ;
        Print("|           Rev 1.0 (by hbahn, 1998.04.17)          |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| BDMA Buffer Test                            - [B] |\r") ;
	Print("| MAC Tx Halt Request Test                    - [H] |\r") ;
	Print("| PHY(ICS1890) MII Station Management Test    - [P] |\r") ;
	Print("| Quit Other MAC Function Test                - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{

		case 'B' : case 'b' : BdmaBufferTest(); break ;
		case 'H' : case 'h' : MacTxHaltReqTest(); break ;
		case 'P' : case 'p' : PhyStationRdWrTest(); break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue MAC Test") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}

/*
 * Function : void PhyStationRdWrTest(void)
 * Description : PHY Station Management Reg Read Write Test
 */
void PhyStationRdWrTest(void)
{
	U32 RdValue ;


	Print("\n >>>>> PHY(ICS1890) Station Management Reg Read <<<<") ;
	RdValue = MiiStationRead( PHY_CNTL_REG, PHYHWADDR) ;
	Print("\r1. CNTL REG     (0x3000) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_STATUS_REG, PHYHWADDR) ;
	Print("\r2. STATUS REG   (0x7809) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ID_REG1, PHYHWADDR) ;
	Print("\r3. ID REG1      (0x0015) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ID_REG2, PHYHWADDR) ;
	Print("\r4. ID REG2      (0xF423) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ANA_REG, PHYHWADDR) ;
	Print("\r5. ANA REG      (0x01E1) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ANLPAR_REG, PHYHWADDR) ;
	Print("\r6. ANLPAR REG   (0x0000) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ANE_REG, PHYHWADDR) ;
	Print("\r7. ANE REG      (0x0000) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ECNTL_REG1, PHYHWADDR) ;
	Print("\r8. ECNTL REG    (0xXXXX) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_QPDS_REG, PHYHWADDR) ;
	Print("\r9. QPDS REG     (0xX00X) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_10BOP_REG, PHYHWADDR) ;
	Print("\r10.10BOP REG    (0xXX10) : %04x ",RdValue) ;
	RdValue = MiiStationRead( PHY_ECNTL_REG2, PHYHWADDR) ;
	Print("\r11.ECNTL REG    (0xXXXX) : %04x ",RdValue) ;
}

/*
 * Function : BdmaBufferTest                                               
 * Description : Test Function for BDMA Rx/Tx Buffer and CAM Read/Write 
 *               Used in Other LoopBack Test Function
 */
void BdmaBufferTest(void)
{
	U32 TestBufferBase ;
	U32 TestBufferSize ;
	U32 TestLoopCnt = 0xFF;
	int i, j ;

	Print("\n >>>  BDMA Tx/Rx and CAM Buffer Test <<< ");

	Print("\r >>   BDMA Tx Buffer Test ....    ");
	TestBufferBase = Base_Addr + 0x9200 ;
	TestBufferSize = 64 ;
	for (i=0 ; i < TestLoopCnt ; i++) 
	{
		for (j=0 ; j < 4 ; j++)
		{
		MemTestInit((U32 *)MemCopySrc, TestBufferSize);
		WordCopyTest((U32 *)MemCopySrc,(U32 *)TestBufferBase,\
					 TestBufferSize);
		}
	}
	Print("Ok.") ;

	// BDMA Rx Buffer Long Range : 9900
	// BDMA Rx Buffer 1bit Range : 9800
	Print("\r >>   BDMA Rx Buffer Test ....    ");
	TestBufferBase = Base_Addr + 0x9900 ;
	TestBufferSize = 64 ;
	for (i=0 ; i < TestLoopCnt ; i++) 
	{
		for (j=0 ; j < 4 ; j++)
		{
		MemTestInit((U32 *)MemCopySrc, TestBufferSize);
		WordCopyTest((U32 *)MemCopySrc,(U32 *)TestBufferBase,\
					 TestBufferSize);
		}
	}
	Print("Ok.") ;

	Print("\r >>   CAM Read/Write Test ....    ") ;
	TestBufferBase = Base_Addr + 0x9100 ;
	TestBufferSize = 32 ;
	for (i=0 ; i < TestLoopCnt ; i++) 
	{
		for (j=0 ; j < 4 ; j++)
		{
		MemTestInit((U32 *)MemCopySrc, TestBufferSize);
		WordCopyTest((U32 *)MemCopySrc,(U32 *)TestBufferBase,\
					 TestBufferSize);
		}
	}
	Print("Ok.") ;
}


/*
 * Function : MacTxHaltReqTest
 * Description : MAC Tx Halt Request and Halt Immediate Test
 */
void MacTxHaltReqTest(void) 
{

	char TransferPacket ;

	Print("\n >>>  MAC Tx Halt Request Test  <<< \r") ;
	do 
	{
	Print("\r   >> Select (Q)uit or (T)x 2 Frame  : ");
	TransferPacket = get_byte() ;
	if ( (TransferPacket == 'T') || (TransferPacket == 't') )
		{
		// Initialize MAC Frame Data
		TxFrameDataInit() ;
		MacTxGo() ; // MAC Transfer

		// Initialize MAC Frame Data
		TxFrameDataInit() ;
		MacTxGo() ; // MAC Transfer

		// Enable Halt Request
		//MACCON |= HaltReg ;
		MACCON |= HaltImm ;
		//MACTXCON |= TxHalt ;

		}
		// Disable Halt Request
		//MACCON &= ~HaltReg ;
		MACCON &= ~HaltImm ;
		//MACTXCON &= ~TxHalt ;
	} while ( (TransferPacket != 'Q') && (TransferPacket != 'q') ) ;

	ReadErrReport() ;
	ClearErrReport() ;
}


/*
 * Function : void MacConfig(void)
 * Description : Other MAC Function
 */
void MacConfig(void)
{
	char TestItemSel ;
	do {
	Print("\n+---------------------------------------------------+\r") ;
	Print("|    ***** MAC Configuration View & Change *****    |\r") ;
        Print("|           Rev 1.0 (by hbahn, 1998.04.17)          |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| MAC & BDMAC Configuration View              - [V] |\r") ;
	Print("| MAC & BDMAC Configuration Change            - [C] |\r") ;
	Print("| View & Change My MAC Address                - [A] |\r") ;
	Print("| Quit Other MAC Function Test                - [Q] |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("\rSelect Test Item : ") ; TestItemSel = get_byte() ;
	
	switch(TestItemSel)
	{
		case 'V' : case 'v' : ViewMacAndBdmaConfiguration() ;
				      break ;
		case 'C' : case 'c' : ChangeMacAndBdmaConfiguration() ;
				      break ;
		case 'A' : case 'a' : MacAddrRead() ;
				      MacAddrWrite() ;
				      MacAddrRead() ;
				      break ;
		case 'Q' : case 'q' : break ;
		default : Print("\nInvalid Test Item Selected") ; 
				break ;
	}

	if ( (TestItemSel != 'Q') && (TestItemSel != 'q') )
		{
		Print("\nPress any key to Continue") ; 
		get_byte() ;
		}
	} while  ( (TestItemSel != 'Q') && (TestItemSel != 'q') ) ;
}


/*
 * Function : ViewMacAndBdmaConfiguration
 * Description : View MAC and BDMA Configuration View
 */
void ViewMacAndBdmaConfiguration(void)
{
	U32  MacConfView ;
	U32  BdmaConfView ;
	U32  BdmaSetValue ;
	U32  PhyStatus ;

	Print("\n  >>>>>>> MAC Configuration View <<<<<<<") ;
	// ------------ MAC Global Control Register --------------
	MacConfView = MACCON ;
	Print("\r +=========================================+");
	Print("\r + MACCON : MAC Gloabal Control Register   +");
	Print("\r +-----------------------------------------+");
	if (MacConfView & FullDup) 
		Print("\r + - Full Duplex Supported                 +") ;
	if (MacConfView & EnMissRoll)
		Print("\r + - Enabled Missed error count roll over  +") ;

	// ------------ MAC Tx Control Register --------------
	MacConfView = MACTXCON ;
	Print("\r +=========================================+");
	Print("\r +  MACTXCON : MAC Tx Control Register     +");
	Print("\r +-----------------------------------------+");
	if (MacConfView & TxEn) 
		Print("\r + - MAC Tx Enabled                        +") ;	
	if (MacConfView & TxHalt) 
		Print("\r + - MAC Tx Halt Requested                 +") ;	
	if (MacConfView & NoPad) 
		Print("\r + - MAC Tx Suppress Padding               +") ;
	if (MacConfView & NoCRC) 
		Print("\r + - MAC Tx Suppress CRC                   +") ;
	if (MacConfView & FBack)
		Print("\r + - MAC Tx Fast Back-off Enabled          +") ;
	if (MacConfView & NoDef)
		Print("\r + - MAC Tx Disable Defer Counter          +") ;

	Print("\r + - MAC Tx Interrupt :                    +") ;
	if (MacConfView & EnUnder) 
		Print("\r +             Underrun                    +") ;
	if (MacConfView & EnDefer) 
		Print("\r +             Deferral                    +") ;
	if (MacConfView & EnNCarr) 
		Print("\r +             NoCarrier                   +") ; 
	if (MacConfView & EnLateColl) 
		Print("\r +             Late Collision              +") ;
	if (MacConfView & EnExColl) 
		Print("\r +             Excessive Collision         +") ;
	if (MacConfView & EnTxPar) 
		Print("\r +             FIFO Parity Error           +") ;
	if (MacConfView & EnComp) 
		Print("\r +             Packet Tx Complete          +") ;

	// ------------ MAC Rx Control Register --------------
	MacConfView = MACRXCON ;
	Print("\r +=========================================+");
	Print("\r +  MACRXCON : MAC Rx Control Register     +");
	Print("\r +-----------------------------------------+");
	if (MacConfView & RxEn)
		Print("\r + - MAC Rx Enabled                        +") ;
	if (MacConfView & RxHalt)
		Print("\r + - MAC Rx Halt Requested                 +") ;
	if (MacConfView & LongEn)
		Print("\r + - MAC Rx Long Frame(1518B) Enabled      +") ;
	if (MacConfView & ShortEn)
		Print("\r + - MAC Rx Short Frame(64B) Enabled       +") ;
	if (MacConfView & StripCRC)
		Print("\r + - MAC Rx Check CRC, But Strip CRC       +") ;
	if (MacConfView & PassCtl)
		Print("\r + - MAC Rx Pass Control Packet            +") ;
	if (MacConfView & IgnoreCRC)
		Print("\r + - MAC Rx Ignore CRC                     +") ;

	Print("\r + - MAC Rx Interrupt :                    +") ;
	if (MacConfView & EnGood) 
		Print("\r +                     Rx Good             +") ;
	if (MacConfView & EnAlign) 
		Print("\r +                     Mis-Allign          +") ;
	if (MacConfView & EnCRCErr) 
		Print("\r +                     CRC Error           +") ;
	if (MacConfView & EnOver) 
		Print("\r +                     FIFO Overflow       +") ;
	if (MacConfView & EnLongErr) 
		Print("\r +                     Over 1518B Frame    +") ;
	if (MacConfView & EnRxPar) 
		Print("\r +                     Parity Error        +") ;
	Print("\r +=========================================+");

	Print("\n  >>>>>>> BDMA Configuration View <<<<<<<") ;
	// ------------ BDMA Tx Control Register --------------
	BdmaConfView = BDMATXCON ;
	Print("\r +=========================================+");
	Print("\r + BDMATXCON : BDMA Tx Control Register    +");
	Print("\r +-----------------------------------------+");
	if (BdmaConfView & BTxEn) 
		Print("\r + -  BDMA Tx Enabled                      +") ;
	BdmaSetValue = BdmaConfView & 0x1F ;
	Print("\r + - BDMA Tx Burst Size : %d Byte          +",(BdmaSetValue+1)) ;
	if (BdmaConfView & BTxSTSKO)
		Print("\r + - BDMA Tx STOP, if Descriptor Not Owner +") ;
	else
		Print("\r + - BDMA Tx SKIP, if Descriptor Not Owner +") ;

	Print("\r + - BDMA Tx Interrupt :                   +") ;
	if (BdmaConfView & BTxCPIE) 
		Print("\r               Control Packet Tx Complete  +") ;
	if (BdmaConfView & BTxEmpty) 
		Print("\r               BDMA Tx Buffer Empty        +") ; 
	if (BdmaConfView & BTxNOIE) 
		Print("\r               BDMA Tx Not Owner           +") ; 
	BdmaSetValue = (BdmaConfView & 0x3800) >> 11 ;
		Print("\r + - BDMA Tx to MAC Tx Start Level : %d/8   +",\
					BdmaSetValue);

	// ------------ BDMA Rx Control Register --------------
	BdmaConfView = BDMARXCON ;
	Print("\r +=========================================+");
	Print("\r + BDMARXCON : BDMA Rx Control Register    +");
	Print("\r +-----------------------------------------+");
	BdmaSetValue = BdmaConfView & 0x1F ;
	if (BdmaConfView & BRxEn) 
		Print("\r + - BDMA Rx Enabled                       +") ;
	Print("\r + - BDMA Rx Burst Size : %d Byte          +",(BdmaSetValue+1)) ;

	if (BdmaConfView & BRxSTSKO)
		Print("\r + - BDMA Rx STOP, if Descriptor Not Owner +") ;
	else
		Print("\r + - BDMA Rx SKIP, if Descriptor Not Owner +") ;
	if (BdmaConfView & BRxMAINC)
		Print("\r + - BDMA Rx Increment Memory Address      +") ;
	else
		Print("\r + - BDMA Rx Decrement Memory Address      +") ;
	if (BdmaConfView & BRxLittle) 
		Print("\r + - BDMA Rx Little Endian                 +") ;
	else 
		Print("\r + - BDMA Rx Big Endian                    +") ;

		Print("\r + - BDMA Rx Interrupt :                   +") ;
	if (BdmaConfView & BRxDIE) 
		Print("\r +             Every Received Frame        +") ;
	if (BdmaConfView & BRxNLIE) 
		Print("\r +             NULL List                   +") ;
	if (BdmaConfView & BRxNOIE) 
		Print("\r +             Rx Not Owner                +") ;
	if (BdmaConfView & BRxMSOIE) 
		Print("\r +             Max Size over               +") ;
	if (BdmaConfView & RxEmpty) 
		Print("\r +             Rx Buffer Empty             +") ; 
	if (BdmaConfView & BRxEarly) 
		Print("\r +             Early notify                +") ; 

	BdmaSetValue = (BdmaConfView & 0x3000) >> 12 ;
		Print("\r + - Rx Word Allignment : %d Invalid Byte   +",\
				BdmaSetValue) ;

	// ------------ CAM Control Register --------------
	Print("\r +=========================================+");
	Print("\r + CAMCON : CAM Control Register           +");
	Print("\r +-----------------------------------------+");
	BdmaConfView = CAMCON ;
	if (BdmaConfView & StationAcc)
		Print("\r + - Accept Unicast station address packet +") ; 
	if (BdmaConfView & GroupAcc)
		Print("\r + - Accept Multicast-group address packet +") ; 
	if (BdmaConfView & BroadAcc)
		Print("\r + - Accept Broadcast station address      +") ;
	if (BdmaConfView & NegCAM)
		Print("\r + - Accept Negative CAM recognizes        +") ;
	if (BdmaConfView & CompEn)
		Print("\r + - CAM Compare Enable mode               +") ;


	// ------------ SNDS PHY Configuration --------------
	Print("\r +=========================================+");
	Print("\r + SNDS PHY(ICS1890) Configuration         +");
	Print("\r +-----------------------------------------+");
	PhyStatus = MiiStationRead(PHY_STATUS_REG, PHYHWADDR) ;
	if ( PhyStatus & (1 << 5) ) 
		Print("\r + - Auto-Negotiation Completed            +") ;
	if ( PhyStatus & (1 << 2) ) 
		Print("\r + - Link is Valid                         +") ;
	PhyStatus = MiiStationRead(PHY_ECNTL_REG2, PHYHWADDR) ;
	if ( PhyStatus & (1 << 15) ) 
		Print("\r + - Repeater Mode is Selected             +") ;
	else
		Print("\r + - NODE Mode is Selected                 +") ;
	Print("\r +=========================================+");
}


/*
 * Function : ChangeMacAndBdmaConfiguration
 * Description : Configure MAC and BDMAC
 */
void ChangeMacAndBdmaConfiguration(void)
{
	char ChangeConfYesOrNo ;
	char ChangeValue ;
	U32  SettingValue ;

	// Configure MAC Tx/Rx Control Register
	Print("\n >> Configure MAC Tx/Rx (Y/N) ? ") ;
	ChangeConfYesOrNo = get_byte() ;

	if ( ( ChangeConfYesOrNo == 'Y' ) || ( ChangeConfYesOrNo == 'y' ) )
	{

		// MAC Global Control Register Config
		Print("\n((( MAC Global Control Register Setup )))") ;
		Print("\r$$ Support Full Duplex(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacCon = FullDup  ;
		else gMacCon = 0  ;

		Print("\r$$ Use 10Mbps 7-Wire Interface(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacCon |= ConnM01  ;
		else gMacCon |= ConnM10  ;

		Print("\r$$$ Missed Error Count Over Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacCon |= EnMissRoll ;

		// MAC Tx Control Register Config
		Print("\n((( MAC Tx Control Register Setup )))") ;
		gMacTxCon = EnComp ;
		Print("\r$$ MAC Tx No Padding(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= NoPad ;

		Print("\r$$ MAC Tx No CRC(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= NoCRC ;

		Print("\r$$ MAC Tx Use Fast Back-off timers(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= FBack ;

		Print("\r$$ MAC Tx Disable the Defer Counter(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= NoDef ;

		Print("\r$$$ MAC Tx Interrupt Setting $$$") ;
		Print("\r$$ Underrun Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= EnUnder ; 
		Print("\r$$ Deferral Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= EnDefer ; 
		Print("\r$$ No Carrier Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= EnNCarr ; 
		Print("\r$$ Late Collision Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= EnLateColl ; 
		Print("\r$$ Parity Error Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= EnTxPar ;
		Print("\r$$ Excessive Collision Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacTxCon |= EnExColl ; 

		// MAC Rx Control Register Config
		Print("\n((( MAC Rx Control Register Setup )))") ;
		gMacRxCon = RxEn | EnGood ;
		Print("\r$$ MAC Rx Long Frame Enable(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= LongEn ;
		Print("\r$$ MAC Rx Short Frame Enable(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= ShortEn ;
		Print("\r$$ MAC Rx Strip CRC(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= StripCRC ;
		Print("\r$$ MAC Rx Pass Control Packet(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= PassCtl ;
		Print("\r$$ MAC Rx Ignore CRC Check(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= IgnoreCRC ;

		Print("\r$$$ MAC Rx Interrupt Setting $$$") ;
		Print("\r$$ Mis-Allign Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= EnAlign ;
		Print("\r$$ CRC Error Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= EnCRCErr ;
		Print("\r$$ FIFO Overflow Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= EnOver ;
		Print("\r$$ Long Frame Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= EnLongErr ;
		Print("\r$$ Parity Error Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gMacRxCon |= EnRxPar ;
	}

	// Configure BDMA Tx/Rx Control Register
	Print("\n\n >> Configure BDMA Tx/Rx (Y/N) ? ") ;
	ChangeConfYesOrNo = get_byte() ;
	if ( ( ChangeConfYesOrNo == 'Y' ) || ( ChangeConfYesOrNo == 'y' ) )
	{
		// BDMA Tx Control Register Config
		Print("\n((( BDMA Tx Control Register Setup )))") ;
		gBdmaTxCon = BTxEn ;
		Print("\r$$ Input BDMA Tx Burst Size(16) : ") ;
		SettingValue = get_digit() ;
		if ( SettingValue != 0 )
			{
			SettingValue = SettingValue - 1 ;
			gBdmaTxCon |= SettingValue  ;
			}
		else gBdmaTxCon |= BTxBRST ;

		Print("\r$$ BDMA Tx STOP when Descriptor Not Owner(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaTxCon |= BTxSTSKO ;

		Print("\r$$ Input BDMA Tx to MAC Tx Start Level(0) : ") ;
		SettingValue = get_digit() ;
		SettingValue = SettingValue << 11 ;
		gBdmaTxCon |= SettingValue  ;

		Print("\r$$$ BDMA Tx Interrupt Setting $$$") ;
		Print("\r$$ Control Packet Tx Complete  Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaTxCon |= BTxCPIE ;
		Print("\r$$ BDMA Tx Sescriptor Not Owner Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaTxCon |= BTxNOIE ;
		Print("\r$$ BDMA Tx Buffer Empty Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaTxCon |= BTxEmpty ;


		// BDMA Rx Control Register Config
		Print("\n((( BDMA Rx Control Register Setup )))") ;
		gBdmaRxCon = BRxEn ;
		Print("\r $$ Input BDMA Rx Burst Size(16) : ") ;
		SettingValue = get_digit() ;
		if ( SettingValue != 0 )
			{
			SettingValue = SettingValue - 1 ;
			gBdmaRxCon |= SettingValue  ;
			}
		else gBdmaRxCon |= BTxBRST ;

		Print("\r$$ BDMA Rx STOP when Descriptor Not Owner(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxSTSKO ;

		Print("\r$$ BDMA Rx Memory Increment(Y) or Decrement(N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxMAINC ;

		Print("\r$$ BDMA Rx Memory Little-Endian(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxLittle ;

		Print("\r$$$ BDMA Tx Interrupt Setting $$$") ;
		Print("\r$$ Every Received Frame Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxDIE ;
		Print("\r$$ Null List Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxNLIE ;
		Print("\r$$ Descriptor Not Owner Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxNOIE ;
		Print("\r$$ Maximum Size Over Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxMSOIE ;
		Print("\r$$ BDMA Rx Buffer Empty Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= RxEmpty ;
		Print("\r$$ Early Notify Interrupt(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gBdmaRxCon |= BRxEarly ;
	}

	// Configure CAM Tx/Rx Control Register
	Print("\n\n >> Configure CAM (Y/N) ? ") ;
	ChangeConfYesOrNo = get_byte() ;
	if ( ( ChangeConfYesOrNo == 'Y' ) || ( ChangeConfYesOrNo == 'y' ) )
	{

		gCamCon = CompEn ;
		Print("\r$$ Accept Unicast Station Address(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gCamCon |= StationAcc ;

		Print("\r$$ Accept Multicast-Group Station Address(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gCamCon |= GroupAcc ;

		Print("\r$$ Accept Broadcast Station Address(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gCamCon |= BroadAcc ;

		Print("\r$$ Accept Negative CAM Reconization(Y/N) ? ") ;
		ChangeValue = get_byte() ;
		if ( (ChangeValue == 'Y') || (ChangeValue == 'y') )
			gCamCon |= NegCAM ;
	}


	// Change Control Register
	ReadyMacRx();
	ReadyMacTx();
	CAMCON = gCamCon ;

	Print("\n >> Cache Configuration (Y/N) ? ") ;
	ChangeConfYesOrNo = get_byte() ;
	if ( ( ChangeConfYesOrNo == 'Y' ) || ( ChangeConfYesOrNo == 'y' ) )
	{
		//CacheStartUpDialog() ;	
	}

}

