/*********************************************************************/
/*                                                                   */
/*      MAC Test Program for KS32C5000                               */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                      */
/*-------------------------------------------------------------------*/
/*                                                                   */
/*      Modified, programmed by Hbahn 			             */
/*                                                                   */
/*      Description : 1998-2-10 first edited for KS32C5000           */
/*********************************************************************/
/*   MACINIT.c  :  Include All MAC & BDMAC Initialize Routine        */
/*********************************************************************/
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "uart.h"
#include "timer.h"
#include "pic.h"
#include "pollio.h"
#include "memory.h"
#include "system.h"
#include "dma.h"
#include "iic.h"
#include "mac.h"

// Global Variable for MAC/BDMA and CAM Configuration
volatile U32 gMacCon = FullDup ;
volatile U32 gMacTxCon = EnUnder | EnDefer | EnNCarr | EnExColl | \
			EnLateColl | EnTxPar | EnComp ;
volatile U32 gMacRxCon = RxEn | EnAlign | EnCRCErr | EnOver | \
			EnLongErr | EnRxPar  ;
volatile U32 gBdmaTxCon = BTxBRST | BTxMSL110 | BTxSTSKO ;
volatile U32 gBdmaRxCon = BRxDIE | BRxEn | BRxBig | BRxMAINC | BRxBRST ;
volatile U32 gCam0_Addr0 = 0 , gCam0_Addr1 = 0  ;
volatile U32 gCamCon = CompEn ;

// Global variables  used for MAC driver
volatile U32 gCTxFDPtr; // CurrentTxFrameDescriptorPointer(gCTFDPtr)
volatile U32 gCRxFDPtr; // CurrentRxFrameDescriptorPointer(gCRFDPtr) 

// Global Variables for My MAC Address
volatile U8 MyMacSrcAddr[6] ;

volatile U32 CntlPktTxDone = 0 ; // Control Packet Tx Done Flag
volatile U32 BDMARxCnt = 0 ; // Increase when BDMA Rx Done 

// Used for Frame data Copy Area
volatile U32 RxFrameUserAreaBase = 0x1200000 ;  // User Frame Area Base
volatile U32 RxFrameUserAreaLimit = 0x12FFFF0 ;  // User Frame Area Limit
extern volatile U32 *BaseofUserFrame ;  // Base of User Frame Area 

// Global Variable for Error Report 
pMACTxErr pMACTxErrCnt = {0,0,0,0,0,0,0,0,0,0} ;
pMACRxErr pMACRxErrCnt = {0,0,0,0,0,0,0,0} ;
pBDMATxErr pBDMATxErrCnt = {0,0,0} ;
pBDMARxErr pBDMARxErrCnt = {0,0,0,0,0} ;

// Global Variable for Frame Descriptor Alloctation
void *fdallocPt = (void *)FDAllocBaseAddr ;

/*
 * void MacInitialize(void)
 * Geberate Frame Descriptor Linked List
 */
void MacInitialize(void)
{
	// ------------------------------------------------
  	// Step 1. 
	// ~~~~~~~
	// 1. Setup Interrupt Vector Table 
   	// 2. Set the initial condition of the BDMA and MAC 
	// 3. Set the BDMA Rx/Tx Frame Descriptor
	// 4. Setup BDMA and MAC to can begin Rx/Tx
 	// 5. Setup CAM mode to filter incomming messages 
	// 6. Assign User Frame Area
	// ------------------------------------------------

	// 1. Setup Interrupt Vector tables  & enable interrupt
	// Setting BDMA interrupt to IRQ Mode to use DMA interrupt 
	// can be used in BDMA interrupt service routine
	// IntMode &= (~BDMA_RX_INT) ;

	SysSetInterrupt(nMAC_RX_INT, MAC_Rx_isr) ;
	SysSetInterrupt(nMAC_TX_INT, MAC_Tx_isr) ;
	SysSetInterrupt(nBDMA_RX_INT, BDMA_Rx_isr) ;	
	SysSetInterrupt(nBDMA_TX_INT, BDMA_Tx_isr) ;	

	// Interrupt setup for timer
	SysSetInterrupt(nTIMER0_INT, tm0isr) ;

	// enable global interrupt mask
	// Enable MAC/BDMA Interrupts service.
	Enable_Int(nMAC_TX_INT) ;
	Enable_Int(nMAC_RX_INT) ;
	Enable_Int(nBDMA_TX_INT) ;	
	Enable_Int(nBDMA_RX_INT) ;	
	Enable_Int(nGLOBAL_INT); 


   	// 2. Set the initial condition of the BDMA and MAC 
	BDMARXCON = BRxRS; // Reset BDMAC Receiver
	BDMATXCON = BTxRS; // Reset BDMAC Transceiver
	BDMARXLSZ = MaxRxFrameSize;

	MACCON = SwReset;
	MACCON = FullDup ;

	// 3. Set the BDMA Rx/Tx Frame Descriptor
	//    Initialization of RxFDA(Frame Descriptor Area)
	TxFDInitialize() ;
	RxFDInitialize() ;

	// 4. Setup BDMA and MAC to can begin Rx/Tx
	ReadyMacRx() ;
	ReadyMacTx() ;
	
 	// 5. Setup CAM mode to filter incomming messages 
	//    CAM Address is Received from IIC EEPROM
	GetMyMacAddr() ;
 		
 	//    CAM Enable (CAMEN)
	//    Use Only CAM#0
	CAMEN = 0x0001;  
	CAMCON = gCamCon ;


	// 6. Assign User Frame Area 
	//    This is required for special purpose, application for diagnostic
	BaseofUserFrame = (U32 *)RxFrameUserAreaBase ;
}



/*
 * void RxFDInitialize(void) 
 * Geberate Frame Descriptor Linked List
 */
void RxFDInitialize(void) 
{
	sRxFrameDescriptor	*pFrameDescriptor, *pPrevFrameDescriptor ;
	sRxFrameDescriptor	*pStartFrameDescriptor ;
	U32			FrameDataAddr, *pFDPtr ;
	U32			i ;

	// Assign Rx Frame Data Buffer Start Address
	FrameDataAddr  = (U32)RxFBABaseAddr;

	// Memory allocation for Next Frame Descriptor
	pFrameDescriptor = \
		(sRxFrameDescriptor *)nmalloc(sizeof(sRxFrameDescriptor)) ;
	pStartFrameDescriptor = pFrameDescriptor;
	pPrevFrameDescriptor = (sRxFrameDescriptor *)NULL;

 	// Set the BDMA Rx Frame Descriptor Start Register 
   	BDMARXPTR = (U32)pFrameDescriptor ;
	gCRxFDPtr = BDMARXPTR ;
	

	for(i=0; i < MaxRxFrameDescriptors; i++) {
		// Initiatial value of Rx Frame Descriptor's Owner is BDMA
		// Owner is Changed to BDMA when Rx Frame data is generated
		pFDPtr = (U32 *)&pFrameDescriptor->FrameDataPtr ;
	    	*pFDPtr = ((U32)FrameDataAddr | fOwnership_BDMA);

		// Clear Reserved & Status/Length Field
		pFDPtr = (U32 *)&pFrameDescriptor->Reserved; 
		*pFDPtr = (U32)0x0 ;
		pFDPtr = (U32 *)&pFrameDescriptor->StatusAndFrameLength;
		*pFDPtr = (U32)0x0 ;


		// Set Previous Frame Descriptor field
		pFDPtr = (U32 *)&pFrameDescriptor->PreviousFrameDescriptor ;
		*pFDPtr = (U32) pPrevFrameDescriptor ;

		// Set current frame descriptor's next frame descriptor
   		if(pPrevFrameDescriptor != NULL) {
		   pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
		   *pFDPtr = (U32)pFrameDescriptor;
		}

		// Save current frame descriptor to previous descriptor
    		pPrevFrameDescriptor = pFrameDescriptor;

#if DEBUG
	Print("\n +--- %d Frame Descriptor -- ",i) ;
	Print("\r FrameDataPtr        : %x -> %8x", 
		(U32)&pFrameDescriptor->FrameDataPtr, \
		(U32)pFrameDescriptor->FrameDataPtr ) ;
	Print("\r PrevFrameDescriptor : %x -> %8x",
		(U32)&pFrameDescriptor->PreviousFrameDescriptor, \
		(U32)pFrameDescriptor->PreviousFrameDescriptor ) ;
	Print("\r NextFrameDescriptor : %x -> %8x",
		(U32)&pFrameDescriptor->NextFrameDescriptor, \
		(U32)pFrameDescriptor->NextFrameDescriptor ) ;
#endif


		// Allocation for Next Frame Descriptor 
		pFrameDescriptor = \
			(sRxFrameDescriptor *)nmalloc(sizeof(sRxFrameDescriptor)) ;

		// Next Frame Buffer 
    		FrameDataAddr += MaxRxFrameSize;

	}// End of For loop

	// End of Frame Descriptor's Next Frame Descriptor is First
        // Frame Descriptor ( Ring Frame Buffer Configuration )
	pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
	*pFDPtr = (U32)pStartFrameDescriptor;

	// Set prevoius frame descriptor of start frame descriptor
	pFDPtr = (U32 *)&pStartFrameDescriptor->PreviousFrameDescriptor;
	*pFDPtr = (U32)pPrevFrameDescriptor;
}

/*
 * void TxFDInitialize(void) 
 * Geberate Tx Frame Descriptor Linked List
 */
void TxFDInitialize(void) 
{
	sTxFrameDescriptor	*pFrameDescriptor, *pPrevFrameDescriptor ;
	sTxFrameDescriptor	*pStartFrameDescriptor ;
	U32			FrameDataAddr, *pFDPtr ;
	U32			i ;

	// Assign Tx Frame Data Buffer Start Address
	FrameDataAddr  = (U32)TxFBABaseAddr;

	// Memory allocation for Next Frame Descriptor
	pFrameDescriptor = \
		(sTxFrameDescriptor *)nmalloc(sizeof(sTxFrameDescriptor)) ;
	pStartFrameDescriptor = pFrameDescriptor;
	pPrevFrameDescriptor = (sTxFrameDescriptor *)NULL;

 	// Set the BDMA Tx Frame Descriptor Start Register 
   	BDMATXPTR = (U32)pFrameDescriptor ;
	gCTxFDPtr = BDMATXPTR ;
	

	for(i=0; i < MaxTxFrameDescriptors; i++) {
		// Initiatial value of Tx Frame Descriptor's Owner is CPU
		// Owner is Changed to BDMA when Tx Frame data is generated
		pFDPtr = (U32 *)&pFrameDescriptor->FrameDataPtr ;
	    	*pFDPtr = ((U32)FrameDataAddr & fOwnership_CPU);

		// Clear Reserved & Status/Length Field
		pFDPtr = (U32 *)&pFrameDescriptor->Reserved; 
		*pFDPtr = (U32)0x0 ;
		pFDPtr = (U32 *)&pFrameDescriptor->StatusAndFrameLength;
		*pFDPtr = (U32)0x0 ;

		// Next Frame Descriptor Address Assign
   		if(pPrevFrameDescriptor != NULL) {
		   pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
		   *pFDPtr = (U32)pFrameDescriptor;
		}

#if DEBUG
	Print("\n +--- %d Frame Descriptor -- ",i) ;
	Print("\r FrameDataPtr        : %x -> %8x", 
		(U32)&pFrameDescriptor->FrameDataPtr, \
		(U32)pFrameDescriptor->FrameDataPtr ) ;
	Print("\r NextFrameDescriptor : %x -> %8x",
		(U32)&pFrameDescriptor->NextFrameDescriptor, \
		(U32)pFrameDescriptor->NextFrameDescriptor ) ;
#endif

    		pPrevFrameDescriptor = pFrameDescriptor;
		pFrameDescriptor = \
			(sTxFrameDescriptor *)nmalloc(sizeof(sTxFrameDescriptor)) ;
    		FrameDataAddr += MaxRxFrameSize;

	}// End of For loop

	// End of Frame Descriptor's Next Frame Descriptor is First
        // Frame Descriptor ( Ring Frame Buffer Configuration )
	pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
	*pFDPtr = (U32)pStartFrameDescriptor;
}

/*
 * void *fdalloc(unsigned nbyte) 
 * Allocation for BDMA Rx/Tx Frame Descriptor
 */
void *fdalloc(unsigned nbyte) 
{
        void *returnPt=fdallocPt;

        fdallocPt= (int *)fdallocPt+nbyte/4+((nbyte%4)>0);
        if( (int)fdallocPt > FDAllocEndAddr)return NULL;
        return returnPt;
}

/*
 * Function : ReadyMacTx 
 * Description : set Tx Registers related with BDMA & MAC to transmit
 *               packet.
 */
void ReadyMacTx(void)
{
	BDMATXCON = gBdmaTxCon ;
	MACTXCON =  gMacTxCon ;
}

/*
 * Function : ReadyMacRx 
 * Description : set Rx Registers related with BDMA & MAC to Receive packet.
 */
void ReadyMacRx(void)
{
	BDMARXCON = gBdmaRxCon ;
	MACRXCON = gMacRxCon ;
}

/*
 * Function : MacTxGo 
 * Description : MAC Transfer Start for interactive mode 
 */
void MacTxGo(void)
{
   	// Enable MAC and BDMA Transfer 
	BDMATXCON |= BTxEn ;
	MACTXCON |= TxEn  ;
}

/*
 * Function : GetMyMacAddr 
 * Description : Get MAC Address From IIC EEPROM
 */
void GetMyMacAddr(void)
{
	U8 MacAddr[6] ;
	int i, MacAddrSize ;
	
	MacAddrSize = sizeof(MacAddr) ;
	gCam0_Addr0 = gCam0_Addr1 = 0 ;

	IICRead((U32)iMacAddrPtr,MacAddrSize,MacAddr) ;

	// Get First of MAC Address
	put_byte('\r') ;
	for (i=0;i<MacAddrSize-2;i++) 
		gCam0_Addr0 = (gCam0_Addr0 << 8) | MacAddr[i] ;

	// Get Second of MAC Address
	for (i=MacAddrSize-2;i<MacAddrSize;i++)
		gCam0_Addr1 = (gCam0_Addr1 << 8) | MacAddr[i] ;
	gCam0_Addr1 = (gCam0_Addr1 << 16) ;

 	// CAM0 register of CAM registers : 0x9100~0x9103, 0x9104, 0x9105
	VPint(CAM_BaseAddr) = gCam0_Addr0 ;
	VPint(CAM_BaseAddr + 0x4) = gCam0_Addr1 ;

	for (i=0;i<MacAddrSize;i++) MyMacSrcAddr[i] = MacAddr[i] ;
}
	
/*
 * Function : MAC_Tx_isr 
 * Description : Interrupt Service Routine for MAC Tx
 */
void MAC_Tx_isr(void)
{
	U32 IntMACTxStatus ;

	IntMACTxStatus = MACTXSTAT ;
	Clear_PendingBit(nMAC_TX_INT) ;

	if (IntMACTxStatus & ( ExColl | TxDeffered | Under | Defer | \
				NCarr | SQE | LateColl | TxPar ) )
	{
		WriteErrReport(TypeIsMACRx,IntMACTxStatus) ;

		if (IntMACTxStatus & ExColl) pMACTxErrCnt.ExCollErr++ ;
		if (IntMACTxStatus & TxDeffered) pMACTxErrCnt.TxDefferedErr++ ;
		if (IntMACTxStatus & Paused) pMACTxErrCnt.sPaused++ ;
		if (IntMACTxStatus & Under) 
			{
			pMACTxErrCnt.UnderErr++ ;
			ReadyMacTx() ;
			MacTxGo() ;
			}
		if (IntMACTxStatus & Defer) pMACTxErrCnt.DeferErr++ ;
		if (IntMACTxStatus & NCarr) pMACTxErrCnt.NCarrErr++ ;
		if (IntMACTxStatus & SQE) pMACTxErrCnt.sSQE++ ;
		if (IntMACTxStatus & LateColl) pMACTxErrCnt.LateCollErr++ ;
		if (IntMACTxStatus & TxPar) pMACTxErrCnt.TxParErr++ ;
		if (IntMACTxStatus & TxHalted) pMACTxErrCnt.sTxHalted++ ;
	}

	if ( !CheckTxOwnerAndStatus() ) Print("\r MAC Tx Fail") ;
}

/*
 * Function : MAC_Rx_isr 
 * Description : Interrupt Service Routine for MAC Tx
 */
void MAC_Rx_isr(void)
{
	U32 IntMACRxStatus ;

	IntMACRxStatus = MACRXSTAT ;
	Clear_PendingBit(nMAC_RX_INT) ;

	if (IntMACRxStatus & (MAllignErr | MCRCErr | MOverflow | MLongErr | \
				MRxPar | MRxHalted) )
	{
		WriteErrReport(TypeIsMACRx,IntMACRxStatus) ;
		if (IntMACRxStatus & MCtlRecd) 
			{ 
			pMACRxErrCnt.sCtlRecd++ ;
			Print("\r +---- Control Packet Rxd : %x ",EPZCNT) ;
			}
		if (IntMACRxStatus & MRx10Stat) pMACRxErrCnt.sRx10Stat++ ;
		if (IntMACRxStatus & MAllignErr) pMACRxErrCnt.AllgnErr++ ;
		if (IntMACRxStatus & MCRCErr) pMACRxErrCnt.sCRCErr++ ;
		if (IntMACRxStatus & MOverflow) pMACRxErrCnt.OverflowErr++ ;
		if (IntMACRxStatus & MLongErr) pMACRxErrCnt.sLongErr++ ;
		if (IntMACRxStatus & MRxPar) pMACRxErrCnt.RxParErr++ ;
		if (IntMACRxStatus & MRxHalted) pMACRxErrCnt.sRxHalted++ ;
	}
}

/*
 * Function : BDMA_Tx_isr 
 * Description : Interrupt Service Routine for MAC Tx
 */
void BDMA_Tx_isr(void)
{
	U32 IntBDMATxStatus ;

	IntBDMATxStatus = BDMASTAT;
	BDMASTAT = IntBDMATxStatus ; // Clear BDMA Status Register
	Clear_PendingBit(nBDMA_TX_INT) ;

	if (IntBDMATxStatus & S_BTxCCP) 
		{
		Print("\r      +-- Control Packet Transfered : %x",ERMPZCNT) ;
		Print("\r          Tx Control Frame Status : %x",ETXSTAT) ;
		CntlPktTxDone = 1 ;
		}

	if (IntBDMATxStatus & (S_BTxNL | S_BTxNO | S_BTxEmpty) )
	{
		WriteErrReport(TypeIsBDMATx,IntBDMATxStatus) ;
		if (IntBDMATxStatus & S_BTxNL) pBDMATxErrCnt.BTxNLErr++ ;
		if (IntBDMATxStatus & S_BTxNO) pBDMATxErrCnt.BTxNOErr++ ;
		if (IntBDMATxStatus & S_BTxEmpty) pBDMATxErrCnt.BTxEmptyErr++;
	}
}

/*
 * Function : BDMA_Rx_isr 
 * Description : Interrupt Service Routine for MAC Tx
 */
void BDMA_Rx_isr(void)
{
	U32 	IntBDMARxStatus , pRxFramePointer ;

	IntBDMARxStatus = BDMASTAT;
	BDMASTAT = IntBDMARxStatus ;
	Clear_PendingBit(nBDMA_RX_INT) ;

	if (IntBDMARxStatus & S_BRxRDF) {
		BDMARxCnt++ ;
		pRxFramePointer = BDMARXPTR ;
		}

	if (IntBDMARxStatus & ( S_BRxNL | S_BRxNO | S_BRxMSO | S_BRxSEarly ) )
	{
		WriteErrReport(TypeIsBDMARx,IntBDMARxStatus) ;
		if (IntBDMARxStatus & S_BRxSEarly) 
			{ 
			pBDMARxErrCnt.sBRxSEarly++ ;
			Print("\r ----- Early Notify : %x",BDMARXLSZ ) ;
			}

		if (IntBDMARxStatus & S_BRxNL) pBDMARxErrCnt.BRxNLErr++ ;
		if (IntBDMARxStatus & S_BRxNO) pBDMARxErrCnt.BRxNOErr++ ;
		if (IntBDMARxStatus & S_BRxMSO) pBDMARxErrCnt.BRxMSOErr++ ;
		if (IntBDMARxStatus & S_BRxEmpty) pBDMARxErrCnt.BRxEmptyErr++;
	}

	else if (!CheckRxStatus(pRxFramePointer)) Print("\r MAC Rx Fail") ;
}

/*
 * Function : ResetPhyChip, ClearPhyChip  
 * Description : Reset The Phychip, Auto-Negotiation Enable 
 *               Clear All Phy Function and enable only FullDuplex Mode
 */
// Reset PHY Chip
void ResetPhyChip(void)
{
	MiiStationWrite(PHY_CNTL_REG, PHYHWADDR, RESET_PHY ) ;
	MiiStationWrite(PHY_CNTL_REG, PHYHWADDR, ENABLE_AN | RESTART_AN | \
				PHY_FULLDUPLEX ) ;
				//PHY_FULLDUPLEX | DR_100MB  ) ;
}

// Clear Phy Chip
void ClearPhyChip(void)
{
	MiiStationWrite(PHY_CNTL_REG, PHYHWADDR, PHY_FULLDUPLEX ) ;
	//MiiStationWrite(PHY_CNTL_REG,PHYHWADDR,PHY_FULLDUPLEX|DR_100MB);
}


/*********************************************************************/
/*      Description : 1998-2-10 first edited for KS32C5000           */
/*********************************************************************/
/* This Routine is provided for example of BDMA Frame Descriptor     */
/* initialize routine for basic Frame Descriptor format              */
/*-------------------------------------------------------------------*/
/* When you use basic type of Frame Descriptor                       */
/*                                                                   */
/* typedef struct FrameDescriptor {                                  */
/* 	U32 FrameDataPtr;                                            */
/* 	U32 Reserved;                                                */
/* 	U32 StatusAndFrameLength;                                    */
/* 	U32 NextFrameDescriptor;                                     */
/* } sFrameDescriptor;                                               */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* Difference between Basic format and Advanced format is use of     */
/* " PreviousFrameDescriptor " Field                                 */
/* You can use MAC Test program with basic format structure          */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* BDMA Frame Descriptor Initialize Routine for Advanced format is   */
/* void RxFDInitialize(void), void TxFDInitialize(void).             */
/* This routine use Memory Allocation routine for initialize Frame   */
/* Descriptor Area.                                                  */
/*                                                                   */
/* BDMA Frame Descriptor Initialize Routine for Basic format is      */
/* void Init_TxFDA(void), void Init_RxFDA(void). This routine use    */
/* Fixed Frame Descriptor Area.                                      */
/*                                                                   */
/*********************************************************************/
/*
 * Function : Init_TxFDA(Frame Descriptor Area-Buffers)
 * Initialize BDMA Tx Frame Descriptor                
 * Tx Frame Descriptor Start Address : 0x5103000     
 * Tx Frame Buffer Start Address : 0x11C2000    
 * Tx Frame Buffer Depth : 128                 
 * Tx Frame Buffer Size : 1520                
 */
void Init_TxFDA(void)
{
	sFrameDescriptor	*pFrameDescriptor, *pPrevFrameDescriptor ;
	sFrameDescriptor	*pStartFrameDescriptor ;
	U32			FrameDataAddr, *pFDPtr ;
	U32			i ;

	// First Frame Descriptor Address pointer
	gCTxFDPtr  = (U32)TxFDBaseAddr;
	
	pFrameDescriptor = (sFrameDescriptor *) gCTxFDPtr;
	pStartFrameDescriptor = pFrameDescriptor;
	pPrevFrameDescriptor = (sFrameDescriptor *)NULL;
	FrameDataAddr  = (U32)TxFBABaseAddr; // Start of Tx Frame Buffer

	for(i=0; i < MaxTxFrameDescriptors; i++) {
		// Initiatial value of Tx Frame Descriptor's Owner is CPU
		// Owner is Changed to BDMA when Tx Frame data is generated
		pFDPtr = (U32 *)&pFrameDescriptor->FrameDataPtr ;
	    	*pFDPtr = ((U32)FrameDataAddr & fOwnership_CPU);

		// Next Frame Descriptor Address Assign
   		if(pPrevFrameDescriptor != NULL) {
		   pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
		   *pFDPtr = (U32)pFrameDescriptor;
		}

    		pPrevFrameDescriptor = pFrameDescriptor;
    		pFrameDescriptor++;
    		FrameDataAddr += MaxRxFrameSize;
	}// End of For loop

	// End of Frame Descriptor's Next Frame Descriptor is First
        // Frame Descriptor ( Ring Frame Buffer Configuration )
	pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
	*pFDPtr = (U32)pStartFrameDescriptor;

	#if DEBUG
 	// Debugging  --------------------------------------------------
	Print("\n3. >>--- Tx Frame Descriptor Sample ") ;
	pFrameDescriptor = (sFrameDescriptor *) gCTxFDPtr ;
	for (i=0; i < 2; i++) 
		{
		pFDPtr = (U32 *)&pFrameDescriptor->FrameDataPtr;
		Print("\r - Tx Data Ptr :%08x -> %08x", pFDPtr, *pFDPtr);
		pFDPtr = (U32 *)&pFrameDescriptor->NextFrameDescriptor;
		Print("\r   Tx Next FD  :%08x -> %08x", pFDPtr, *pFDPtr);	
		pFrameDescriptor = (sFrameDescriptor *)(*pFDPtr);
		}
 	// End Debug  --------------------------------------------------
	#endif
}

/*
 * Function : Init_RxFDA(Frame Descriptor Area-Buffers) 
 * Initialize BDMA Rx Frame Descriptor  
 * Rx Frame Descriptor Start Address : 0x5100000  
 * Rx Frame Buffer Start Address : 0x1104000
 * Rx Frame Buffer Depth : 512  
 * Rx Frame Buffer Size : 1520 
 */
void Init_RxFDA(void)
{
	sFrameDescriptor	*pFrameDescriptor, *pPrevFrameDescriptor;
	sFrameDescriptor 	*pStartFrameDescriptor;
	U32			FrameDataAddr, *pFDPtr;
	U32			i;

	// Each Frame Descriptor's frame data pointer points to 
	// the corresponding Frame Data Area.

	gCRxFDPtr = (U32)RxFDBaseAddr;

	pFrameDescriptor = (sFrameDescriptor *)gCRxFDPtr; 
	pStartFrameDescriptor = pFrameDescriptor;
	pPrevFrameDescriptor = (sFrameDescriptor *)NULL;
	FrameDataAddr  = (U32)RxFBABaseAddr;

	for(i=0; i < MaxRxFrameDescriptors; i++) {
		// Assign Frame Buffer Data Address & Change Ownership to BDMA
		pFDPtr = (U32 *)&pFrameDescriptor->FrameDataPtr;
	    	*pFDPtr = ((U32)FrameDataAddr | fOwnership_BDMA);
 
		// Next Frame Descriptor Address Assign
   		if(pPrevFrameDescriptor != NULL) {
		   pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
		   *pFDPtr = (U32)pFrameDescriptor;
		}

    		pPrevFrameDescriptor = pFrameDescriptor;
    		pFrameDescriptor++;
    		FrameDataAddr += MaxRxFrameSize;
	}// End of For loop 

	// End of Frame Descriptor's Next Frame Descriptor is First
        // Frame Descriptor ( Ring Frame Buffer Configuration )
	pFDPtr = (U32 *)&pPrevFrameDescriptor->NextFrameDescriptor;
	*pFDPtr = (U32)pStartFrameDescriptor;

	#if DEBUG
 	// Debugging  --------------------------------------------------
	Print("\n2. >>--- Rx Frame Descriptor Sample ") ;
	pFrameDescriptor = (sFrameDescriptor *) gCRxFDPtr;
	for (i=0; i < 2; i++) 
		{
		pFDPtr = (U32 *)&pFrameDescriptor->FrameDataPtr;
		Print("\r - Rx Data Ptr :%08x --> %08x", pFDPtr, *pFDPtr);
		pFDPtr = (U32 *)&pFrameDescriptor->NextFrameDescriptor;
		Print("\r   Rx Next FD  :%08x --> %08x", pFDPtr, *pFDPtr);	
		pFrameDescriptor = (sFrameDescriptor *)(*pFDPtr);
		} 
 	// End Debug  --------------------------------------------------
	#endif

}

