/*********************************************************************/
/*                                                                   */
/*      MAC Test Program for KS32C5000                               */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                      */
/*-------------------------------------------------------------------*/
/*                                                                   */
/*      Modified, programmed by Hbahn 			             */
/*                                                                   */
/*      Description : 1998-2-10 first edited for KS32C5000           */
/*********************************************************************/
/*   MACLIB.c  :  Library For MAC Test Program                       */
/*********************************************************************/
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "uart.h"
#include "pollio.h"
#include "timer.h"
#include "pic.h"
#include "dma.h"
#include "iic.h"
#include "memory.h"
#include "system.h"
#include "mac.h"

// Global variables  used for MAC driver
extern volatile U32 gCTxFDPtr ; 
extern volatile U32 gCRxFDPtr ; 
extern volatile U32 gPrevTxFDPtr = 0 ; 
extern TIME tm0 ;
extern volatile U32 gCam0_Addr0 , gCam0_Addr1 ;
extern volatile U32 gCapture_Addr0 , gCapture_Addr1 ;
extern volatile U32 gCamCon ;

extern volatile U8 MyMacSrcAddr[6] ;// Global Variables for My MAC Address
volatile U32 gTxFrameSequence = 0 ; // Global Variable for MAC Tx
volatile U32 *BaseofUserFrame ; 
volatile int RxFrameSequence ;
volatile int RxFrameMisMatch=0 ;

// Global Variable for Error Report 
volatile U32 *WriteErrRptBase = (U32 *)ErrReportBaseAddr ;
volatile U32 *ReadErrBase = (U32 *)ErrReportBaseAddr ;
extern U32 BDMARxCnt ;
extern pMACTxErr pMACTxErrCnt ;
extern pMACRxErr pMACRxErrCnt ;
extern pBDMATxErr pBDMATxErrCnt ;
extern pBDMARxErr pBDMARxErrCnt ;

extern volatile U32 RxFrameUserAreaBase ;
extern volatile U32 RxFrameUserAreaLimit ;

/*
 * Function : MiiStationRead, MiiStationWrite  
 * Description : MII Interface Station Management Register Read or Write
 * Input : PhyInAddr(PHY internal register address)
 *         PhyAddr(PHY unique address)
 *         PhyWrData(When Write) 
 * Output: PhyRdData(WhenRead) 
 */
void MiiStationWrite(U32 PhyInAddr, U32 PhyAddr, U32 PhyWrData)
{
	STADATA = PhyWrData ;
	STACON = PhyInAddr | PhyAddr |  MiiBusy | PHYREGWRITE ;
	while( (STACON & MiiBusy) )  ;
}
U32 MiiStationRead(U32 PhyInAddr, U32 PhyAddr) 
{
	U32	PhyRdData ;
	STACON = PhyInAddr | PhyAddr |  MiiBusy ;
	while( (STACON & MiiBusy) )  ;
	PhyRdData = STADATA ;
	return PhyRdData ;
}

/*
 * Function : TxFrameDataInit
 * Description : Frame data Generation for Transmit
 */
void TxFrameDataInit(void) 
{
	sTxFrameDescriptor 	*psTxFD;
	sMACFrame 		psMACFrame ;
	U32 			*pFrameDataPtr ;
	volatile U32 		*pFDPtr ;
	U8  			*pFrameData ;
	U32 			GetFrameSize ;
	int 			i ;

	// 1. Check Current BDMATXPTR's Ownership
	psTxFD = (sTxFrameDescriptor *)BDMATXPTR ;
	pFDPtr = (U32 *)&psTxFD->FrameDataPtr ;
	while (*pFDPtr & fOwnership_BDMA) ;

	// 2. Get Current TX Frame Descriptor Pointer
	psTxFD = (sTxFrameDescriptor *)gCTxFDPtr ;
	gPrevTxFDPtr = gCTxFDPtr ;

	//  Tx Frame Sequence Number: Used for Frame Data and Length Gen.
	gTxFrameSequence++ ;

	// 3. Set Tx Destination Address 
	for (i=0;i<6;i++)
		psMACFrame.DestinationAddr[i] = MyMacSrcAddr[i] ;

	// 4. Set Tx Source Address 
	for (i=0;i<5;i++)
		psMACFrame.SourceAddr[i] = MyMacSrcAddr[i] ;
		psMACFrame.SourceAddr[5] = 0xFF ;


	// 5. Get Tx Frame Length 
	//    Random Frame Size is used
	GetFrameSize = PatternGen(gTxFrameSequence) & 0x7FF ;
	if (GetFrameSize > 1500) GetFrameSize = 1490 ;
	if (GetFrameSize < 54) GetFrameSize = 60 ;

	// Calculate Frame Length : Data Length + Length Field
	psMACFrame.LengthOrType[0] = (U8)( (GetFrameSize+2) & 0xFF ) ;
	psMACFrame.LengthOrType[1] = (U8)( (GetFrameSize+2) >> 8 & 0xFF ) ;

	// 6. Tx Frame Data Generation
	for (i=0; i<4 ;i++) 
		psMACFrame.LLCData[i]=(U8)( (gTxFrameSequence>>8*i) & 0xFF) ;

	for (i=4;i<GetFrameSize;i++)
		psMACFrame.LLCData[i] = i & 0xFF ;

	// copy frame data to DRAM's Frame data buffer area 
	pFrameDataPtr = (U32 *)&psTxFD->FrameDataPtr;
	pFrameData = (U8 *)*pFrameDataPtr;

	for (i=0;i<6;i++) *pFrameData++ = psMACFrame.DestinationAddr[i] ;
	for (i=0;i<6;i++) *pFrameData++ = psMACFrame.SourceAddr[i] ;
	for (i=0;i<2;i++) *pFrameData++ = psMACFrame.LengthOrType[i] ;
	for (i=0;i<GetFrameSize ; i++)
		*pFrameData++ = psMACFrame.LLCData[i] ;

        // 7. Set Current FD for BDMA : Owner Bit Change
	pFDPtr = (U32 *)&psTxFD->FrameDataPtr;
   	*pFDPtr |= fOwnership_BDMA;

	// 8. Set TX Frame flag & Length Field
	pFDPtr = (U32 *)&psTxFD->Reserved;
   	*pFDPtr = (PaddingMode | CRCMode | SourceAddrIncrement | BigEndian | \
			WidgetAlign00 | MACTxIntEn);
	pFDPtr =(U32 *)&psTxFD->StatusAndFrameLength;
   	*pFDPtr =  (U32)((GetFrameSize + 14) & 0xffff);
}


/*
 * Function : WriteErrReport
 * Description : Write Each Status Register and Time
 */
void WriteErrReport(U32 ErrType, U32 Status) 
{
	if (WriteErrRptBase >=  (U32 *)0x12F0000) 
		WriteErrRptBase = (U32 *)ErrReportBaseAddr ;
	*WriteErrRptBase++ = tm0.tick ;
	*WriteErrRptBase++ = tm0.min ;
	*WriteErrRptBase++ = ErrType ;
	*WriteErrRptBase++ = Status ;
}


/*
 * Function : ReadErrReport
 * Description : Read Error Status and Time
 */
void ReadErrReport(void) 
{
	U32 *CrlErrRptBase = (U32 *)ErrReportBaseAddr ;
	int ErrStatus ,ErrType ;
	int i, sec, min ;
    	char buff[250];

	Print("\n >>>>> Error Report <<<<< ") ;
	// Timer is Stopped
	Print("\r -> Total Running Time : %d hour %d min %d sec",\
			tm0.hour, tm0.min,tm0.tick) ;

	Print("\n +- MAC Tx Err Count -+ ") ;
	Print("\r    ExColl: %d, Defered: %d, Paused: %d, Under : %d ", \
 		pMACTxErrCnt.ExCollErr, pMACTxErrCnt.TxDefferedErr, \
 		pMACTxErrCnt.sPaused, pMACTxErrCnt.UnderErr) ;
	Print("\r    Defer : %d, NoCarri: %d, SQE   : %d, LatCol: %d ", \
 		pMACTxErrCnt.DeferErr, pMACTxErrCnt.NCarrErr, \
 		pMACTxErrCnt.sSQE, pMACTxErrCnt.LateCollErr) ;
	Print("\r    TxPar : %d, TxHalt : %d ", \
 		pMACTxErrCnt.TxParErr, pMACTxErrCnt.sTxHalted) ;

	Print("\r +- MAC Rx Err Count -+ ") ;
	Print("\r    CtlRcd: %d, 10Stat : %d, Allign: %d, CRCErr: %d ", \
 		pMACRxErrCnt.sCtlRecd, pMACRxErrCnt.sRx10Stat, \
 		pMACRxErrCnt.AllgnErr, pMACRxErrCnt.sCRCErr ) ;
	Print("\r    OverFl: %d, LongErr: %d, RxPar : %d, RxHalt: %d ", \
 		pMACRxErrCnt.OverflowErr, pMACRxErrCnt.sLongErr, \
 		pMACRxErrCnt.RxParErr, pMACRxErrCnt.sRxHalted) ;
	Print("\r +- BDMA Tx Err Count -+ ") ;
	Print("\r    BTxNull: %d, BTxNOwn: %d, BTxEmpty: %d ", \
 		pBDMATxErrCnt.BTxNLErr, pBDMATxErrCnt.BTxNOErr, \
 		pBDMATxErrCnt.BTxEmptyErr ) ;

	Print("\r +- BDMA Rx Err Count -+ ") ;
	Print("\r    BRxNull: %d, BRxNOwn: %d, BRxEmpty: %d ", \
 		pBDMARxErrCnt.BRxNLErr, pBDMARxErrCnt.BRxNOErr, \
 		pBDMARxErrCnt.BRxEmptyErr) ;
	Print("\r    BRxMaxOvr: %d, BRxEarly: %d ", \
	 	pBDMARxErrCnt.BRxMSOErr, pBDMARxErrCnt.sBRxSEarly) ;

	Print("\r +- Missed Error Count : %d",EMISSCNT) ;

	Print("\n >>>>> Error Occured Time <<<<< ") ;
	while (1)
	{
	sec = *ReadErrBase++ ;
	min = *ReadErrBase++ ;
	ErrType = *ReadErrBase++ ;
	ErrStatus = *ReadErrBase++ ;

	if ( ErrType == TypeIsMACTx )
    		sprintf(buff,"\r - MAC Tx Err : %x at %d:%d time", 
			ErrStatus,min,sec ) ;
	else if ( ErrType == TypeIsMACRx )
    		sprintf(buff,"\r - MAC Rx Err : %x at %d:%d time", 
			ErrStatus,min,sec ) ;
	else if ( ErrType == TypeIsBDMATx )
    		sprintf(buff,"\r - BDMA Tx Err : %x at %d:%d time", 
			ErrStatus,min,sec ) ;
	else if ( ErrType == TypeIsBDMARx )
    		sprintf(buff,"\r - BDMA Rx Err : %x at %d:%d time", 
			ErrStatus,min,sec ) ;
	else break ;
	
	// Print Error Report
        put_string(buff);

	// Clear ErrRpt Memory
	for (i=0; i<4 ; i++) *CrlErrRptBase++ = 0x0 ;
	}
}

/*
 * Function : ClearErrReport
 * Description : Clear Error Report Area
 */
void ClearErrReport(void) 
{
	// Clear Error Report Variable 
 	pMACTxErrCnt.ExCollErr = pMACTxErrCnt.TxDefferedErr = 0 ;
 	pMACTxErrCnt.sPaused = pMACTxErrCnt.UnderErr = 0 ; 
 	pMACTxErrCnt.DeferErr = pMACTxErrCnt.NCarrErr = 0 ;
 	pMACTxErrCnt.sSQE = pMACTxErrCnt.LateCollErr = 0 ;
 	pMACTxErrCnt.TxParErr = pMACTxErrCnt.sTxHalted = 0 ;
 	pMACRxErrCnt.sCtlRecd = pMACRxErrCnt.sRx10Stat = 0 ;
 	pMACRxErrCnt.AllgnErr = pMACRxErrCnt.sCRCErr = 0 ;
 	pMACRxErrCnt.OverflowErr = pMACRxErrCnt.sLongErr = 0 ;
 	pMACRxErrCnt.RxParErr = pMACRxErrCnt.sRxHalted = 0 ;
 	pBDMATxErrCnt.BTxNLErr =  pBDMATxErrCnt.BTxNOErr = 0 ;
 	pBDMATxErrCnt.BTxEmptyErr = 0 ;
	pBDMARxErrCnt.BRxNLErr = pBDMARxErrCnt.BRxNOErr = 0 ;
 	pBDMARxErrCnt.BRxMSOErr = pBDMARxErrCnt.BRxEmptyErr = 0 ;
 	pBDMARxErrCnt.sBRxSEarly = 0 ;
}

/*
 * Function : CheckTxOwnerAndStatus 
 * Description : In Interrupt Mode, Check Tx Status 
 *               And Change  Ownership of BDMA Descriptor to CPU 
 */
int CheckTxOwnerAndStatus(void)
{
	// ------------------------------------------------
  	// Step 4. 
	// ~~~~~~~
	// 1. Check Tx Ownership 
	// 2. Tx Fail Check
	// ------------------------------------------------
	sTxFrameDescriptor *pTxFDPtr ;
	U32 *pFDPtr;
	U32 status, CheckBdma ;

	// 1. Check Tx Ownership
	pTxFDPtr = (sTxFrameDescriptor *)gCTxFDPtr ;
	pFDPtr = (U32 *)&pTxFDPtr->NextFrameDescriptor ;
	gCTxFDPtr = (U32)*pFDPtr ;

	pFDPtr = (U32 *)&pTxFDPtr->FrameDataPtr ;
  	CheckBdma = *pFDPtr & fOwnership_BDMA ;

	if (!CheckBdma) // if BDMA Ownership is CPU
		status = (pTxFDPtr->StatusAndFrameLength >> 16) & 0xffff;
	else return (U32)MacTx_Fail ;
	
	// 2. Tx Fail Check (by Frame Descriptor)
	CheckBdma = ExColl | TxDeffer | Paused | Under | Defer | \
		 NCarr | SQErr | LateColl | TxPar | TxHalted  ;

	if ( status & CheckBdma ) return MacTx_Fail ;
	else return MacTx_Ok ;
}

/*
 * Function : BdmaRxForSingleIsr
 * Description : BDMA Rx Interrupt Service Routine for Single Receive
 */
void BdmaRxForSingleIsr(void) 
{
	sRxFrameDescriptor *pRxFDptr ;
	U32 *pFDPtr, RxStatus, FrameLength, IntBDMARxStatus ;

	IntBDMARxStatus = BDMASTAT;
	BDMASTAT = IntBDMARxStatus ;
	Clear_PendingBit(nBDMA_RX_INT) ;

	if (IntBDMARxStatus & S_BRxRDF) BDMARxCnt++ ;
	if (IntBDMARxStatus & S_BRxNL) pBDMARxErrCnt.BRxNLErr++ ;
	if (IntBDMARxStatus & S_BRxNO) pBDMARxErrCnt.BRxNOErr++ ;
	if (IntBDMARxStatus & S_BRxMSO) pBDMARxErrCnt.BRxMSOErr++ ;
	if (IntBDMARxStatus & S_BRxEmpty) pBDMARxErrCnt.BRxEmptyErr++ ;
	if (IntBDMARxStatus & S_BRxSEarly) 
		{
		pBDMARxErrCnt.sBRxSEarly++ ;
		Print("\r --- Early Notify: %x",BDMARXLSZ ) ;
		}

	// Write Error Report
	if (IntBDMARxStatus & (S_BRxNL | S_BRxNO | S_BRxMSO) )
		WriteErrReport(TypeIsBDMARx,IntBDMARxStatus) ;

	// Get Frame Buffer Pointer from Current BDMARXPTR's
	// Previous Ptr.
	pRxFDptr = (sRxFrameDescriptor *)BDMARXPTR ;
	pFDPtr = (U32 *)&pRxFDptr->PreviousFrameDescriptor ;
	pRxFDptr = (sRxFrameDescriptor *)*pFDPtr ;

	// Check BDMA Receive Ownership and Status
	pFDPtr = (U32 *)&pRxFDptr->FrameDataPtr ;

	if ( !(*pFDPtr & fOwnership_BDMA) )
		{
		RxStatus = (pRxFDptr->StatusAndFrameLength >> 16) & 0xffff;
		FrameLength = (pRxFDptr->StatusAndFrameLength) & 0xffff;
		Print("\n ->> %d'th Received Frame ",BDMARxCnt) ;
		PrintRxFrame((U32)*pFDPtr,FrameLength) ;
		}
	else Print("\r MAC Rx Fail") ;

}

/*
 * Function : PrintRxFrame
 * Description : Print MAC Received Data
 */
void PrintRxFrame(U32 gCRxPtr, U32 FrameLength)
{
	U8 *RxFrameData;
	U32 i,ColumnSize ;

  	RxFrameData = (U8 *)gCRxPtr ; 

	Print("\r -- Destin Address : ") ;
	for(i=0; i < 6; i++) Print("%02x ", *RxFrameData++);
	Print("\r -- Source Address : ") ;
	for(i=0; i < 6; i++) Print("%02x ", *RxFrameData++);
	Print("\r -- LLC Frame Length(Data+Length:%d) : ",FrameLength-16) ;
	for(i=0; i < 2; i++) Print("%02x ", *RxFrameData++);
	Print("\r -- LLC Frame Data : ") ;
	ColumnSize  = FrameLength - 18 ;
	do {
	for(i=0; i < 16 ; i++) 
		{
		Print("%02x ", *RxFrameData++);
		if (--ColumnSize == 0) break ;
		}
		Print("\r%19c",' ') ; 
	} while(ColumnSize) ; 

	Print("\r -- CRC Value      : ") ;
	for(i=0; i < 4 ; i++) Print("%x ", *RxFrameData++);
}

/*
 * Function : CheckRxStatus 
 * Description : In Interrupt Mode, Check Rx Status
 */
U32 CheckRxStatus(U32 pRxFramePointer)
{
	// ------------------------------------------------
  	// Step 5. 
	// ~~~~~~~
	// 1. Get Received Rx Frame Descriptor
	// 2. Check BDMA Receive Ownership
	// 3. Check BDMA Receive Status
	//    If No error, then Get receive data
	// 4. If All Frame is received, Save Global Current Frame Descriptor
	//    else Get Frame Data from Previous Frame Descriptor
	// ------------------------------------------------
	sRxFrameDescriptor 	*pRxFDptr;
	U32 			*pFDPtr, CheckOwnerShip, RxStatus ;
	U32 			nRxFDPtr ;

	nRxFDPtr = pRxFramePointer ;

	while (1) {
	// 1. Get Received Rx Frame Descriptor
	pRxFDptr = (sRxFrameDescriptor *)nRxFDPtr ;
	pFDPtr = (U32 *)&pRxFDptr->PreviousFrameDescriptor ;
	pRxFDptr = (sRxFrameDescriptor *)*pFDPtr ;
	pFDPtr = (U32 *)&pRxFDptr->FrameDataPtr ;

	// 2. Check BDMA Receive Ownership 
  	CheckOwnerShip = *pFDPtr & fOwnership_BDMA ;
	if (!CheckOwnerShip) 
		RxStatus = (pRxFDptr->StatusAndFrameLength >> 16) & 0xffff;
	else 	return (U32)MacRx_Fail ;

	// 3. Check BDMA Receive Status
	//    If No error, then Get receive data
	if(RxStatus & RxGood) GetRxFrameData((U32)pFDPtr) ;
	else return (U32)MacRx_Fail ;


	// 4. If All Frame is received, Save Global Current Frame Descriptor
	//    else Get Frame Data from Previous Frame Descriptor
	if ( gCRxFDPtr == (U32)pFDPtr ) 
		{
		gCRxFDPtr = pRxFramePointer ;
		break ;
		}
	else 	{
		pRxFDptr = (sRxFrameDescriptor *)nRxFDPtr ;
		pFDPtr 	= (U32 *)&pRxFDptr->PreviousFrameDescriptor ;
		nRxFDPtr = (U32)pFDPtr ;
		}

	} return (U32)MacRx_Ok ; // End of while loop
}


/*
 * Function : GetRxFrameData
 * Description : Move MAC Received Frame Data
 */
void GetRxFrameData(U32 RxFramePointer)
{
	sRxFrameDescriptor *pRxFDptr;
	U32 *pFDPtr, FrameLength, RxStatus, RxTime ;
	U32 *pFrameData;

	RxTime = tm0.tick | (tm0.min << 8) ;
	// 1. Get Frame Length and Status
	pRxFDptr = (sRxFrameDescriptor *)RxFramePointer ;
	FrameLength = pRxFDptr->StatusAndFrameLength & 0xffff ;
	RxStatus = (pRxFDptr->StatusAndFrameLength >> 16) & 0xffff ;


	pFDPtr = (U32 *)&pRxFDptr->FrameDataPtr ;
  	pFrameData = (U32 *)*pFDPtr;

	// Receoved Frame Data Store Format
	// +------------------+
	// | Preamble         |
	// | Sequence| Time   |
	// | Status  | Length |
	// | Rx Data          |
	// +------------------+

	if ( BaseofUserFrame > (U32 *)RxFrameUserAreaLimit) 
	{
		BaseofUserFrame = (U32 *)RxFrameUserAreaBase ;
		Print("\r*- Buffer is Full -*\r") ;
	}

	*BaseofUserFrame++ = RxFrameDataPreamble ; 
	*BaseofUserFrame++ = (RxFrameSequence++ << 16) | RxTime ;
	*BaseofUserFrame++ = (RxStatus << 16) | FrameLength  ; 

	// Move Rx Frame to User Area
	//dcopy((U32)BaseofUserFrame,(U32)pFrameData, FrameLength, TxBYTE) ;
	bcopy((U32 *)pFrameData,(U32 *)BaseofUserFrame,FrameLength) ;
	BaseofUserFrame += (U32)(FrameLength/4 + 1) ;

	// 2. Change Ownersip to BDMA for Next Use
   	*pFDPtr |= fOwnership_BDMA;

	// Clear Reserved & Status/Length Field
	pFDPtr = (U32 *)&pRxFDptr->Reserved;
	*pFDPtr = (U32)0x0 ;
	pFDPtr = (U32 *)&pRxFDptr->StatusAndFrameLength;
	*pFDPtr = (U32)0x0 ;
}

/*
 * Function : InterruptLoopBackTest
 * Description : MAC Loopback Test main function, in Interrupt mode
 */
int InterruptLoopBackTest(void)
{
	U32 result;
	U32 i, LoopCnt = MaxTxFrameDescriptors / 2 ;

	IOPDATA = 0x0F ;
	RxFrameMisMatch = 0 ;
	Print("\n --> MAC Loopback Test is Running (%d) ...  ",\
		MaxTxFrameDescriptors * MaxRxFrameDescriptors / 4 ) ;
	while(LoopCnt--) {
		for (i=0; i<MaxRxFrameDescriptors/2 ;i++)
		{
			TxFrameDataInit() ;
			MacTxGo() ;
			if (!CompareLoopBackData()) 
			{
			RxFrameMisMatch++ ;
			Print("\r     MAC Loopback Test is Running (%d) ...  ",\
				LoopCnt*MaxRxFrameDescriptors/2 ) ;
			result = 0 ;
			return result ;
			}
			else 	 result = 1 ;
		}
		PrintMemTestStatus((MaxTxFrameDescriptors/2-LoopCnt)%4) ;
	}
	Print(" Ok !!! \r  ** Total Mismatch Frame is %d ",RxFrameMisMatch) ;
	IOPDATA = 0x0F ;

	return result ;
}

/*
 * Function : PollingLoopBackCheck
 * Description : Send and Check Frame for Loopback Diagnostics (Polling)
 */
U32	PollingLoopBackCheck(void)
{
	U32 pRxFramePointer ;
	U32 status, result ; 

	// --- Transmit Frame Data ---
	// Initialize MAC Frame Data
	TxFrameDataInit() ; 

	// MAC Transfer
	MacTxGo() ;

	// Wait until MAC Tx Done, by check MAC Tx Status Register
	// Polling Transmit finished
	while(1) {
		status = MACTXSTAT;
		if (status & Comp) break; 
	}

	// Check BDMA Tx Owner Bit
	result = CheckTxOwnerAndStatus() ;

	Print("\n Polling Tx Done (%d)", result) ;

	// --- Receive Frame Data ---
	// Wait until BDMA Rx Done, by check BDMA Status Register
	// Polling Received
	while(1) {
		status = BDMASTAT;
		BDMASTAT = status ; 		// Clear Status Register
		if(status & S_BRxRDF) break; 	// Check BDMA Rx Done
	}

	pRxFramePointer = BDMARXPTR ;
	result = CheckRxStatus(pRxFramePointer) ;
	Print("\r Polling Rx Done (%d)",result) ;

	result = CompareLoopBackData() ;
	Print("\r Compare Loopback Data Done (%d)",result) ;

	return result;
}

/*
 * Function : CompareLoopBackData
 * Description : Compare Tx Frame Data and Rx Frame Data
 */
int CompareLoopBackData(void)
{
	sTxFrameDescriptor 	*psTxFD;
	volatile U32 		*pFDPtr, *pRxPtr ;
	U8  			*pTxFrameData, *pRxFrameData ;
	U32 			RxStatus, RxSequence ;
	int i , TxFrameLength=0, RxFrameLength=0 ;

	// Wait Untill Current Tx is Done
	// by Check Current BDMATXPTR's BDMA Ownership
	psTxFD = (sTxFrameDescriptor *)BDMATXPTR ;
	pFDPtr = (U32 *)&psTxFD->FrameDataPtr ;
	while (*pFDPtr & fOwnership_BDMA) ;

	// get TX Frame Descriptor start address to Compare
	psTxFD = (sTxFrameDescriptor *)gPrevTxFDPtr ;
	pFDPtr = (U32 *)&psTxFD->FrameDataPtr ;
	pTxFrameData = (U8 *)*pFDPtr ;

	// Destination & Source Address
	for (i=0;i<12;i++) pTxFrameData++ ;
	for (i=0;i<2;i++) 
		TxFrameLength = TxFrameLength | ( *pTxFrameData++ << i*8) ;

	// Wait untill Recevie Done
	// Receoved Frame Data Store Format
	// +------------------+
	// | Preamble         |
	// | Sequence| Time   |
	// | Status  | Length |
	// | Rx Data          |
	// +------------------+
	pRxPtr = (U32 *)RxFrameUserAreaBase ;
	BaseofUserFrame = (U32 *)RxFrameUserAreaBase ;
	while (*pRxPtr != RxFrameDataPreamble) ;
	*pRxPtr++ = 0 ; // Preamble Clear
	
	// Get Rx Frame Sequence  & Status
	RxSequence = (*pRxPtr++ >> 16) & 0xFFFF ;
	RxStatus = (*pRxPtr >> 16) & 0xFFFF ;
	
	// Destin + Source + CRC
	RxFrameLength = (*pRxPtr++ & 0xFFFF) - 14 ;
	pRxFrameData = (U8 *)pRxPtr ;
	for (i=0;i<14;i++) pRxFrameData++ ;
	//Print(" %d ",RxFrameLength) ;

	if(RxStatus & RxGood) 
		{
		if (!bcomp(pTxFrameData,pRxFrameData,TxFrameLength-2))
			{
			Print("\rgCRxFDPtr:%x, Status:%x, Length:%d",\
				gCRxFDPtr,RxStatus,RxFrameLength) ;
			return MacRx_Fail ;
			}
		
		else return MacRx_Ok ;
		}
	else return MacRx_Fail ;
}


/*
 * Function : MacAddrWrite
 * Description : MAC Address Write to IIC EEPROM
 */
void MacAddrWrite(void)
{
	U8 MacAddr[6], RcvData[12] ;
	int i = 0 , j = 0 ;

	Print("\r >>> Input MAC Address (00 00 F0 11 00 00) : ") ;
	while ( (RcvData[i] = get_byte() ) != CR ) 
	{
		if ( (RcvData[i] >= '0') && (RcvData[i] <= '9') )
			RcvData[i] -= '0' ;
		else if ( (RcvData[i] >= 'A') && (RcvData[i] <= 'F') )
			RcvData[i] = RcvData[i] - 'A' + 10 ;
		else if ( (RcvData[i] >= 'a') && (RcvData[i] <= 'f') )
			RcvData[i] = RcvData[i] - 'a' + 10 ;
		i++ ;
	}

	// Make Received Character to MAC Address Format
	i = 0 ;
	MacAddr[j] = RcvData[i++] << 4 ; MacAddr[j++] += RcvData[i++] ;
	MacAddr[j] = RcvData[i++] << 4 ; MacAddr[j++] += RcvData[i++] ;
	MacAddr[j] = RcvData[i++] << 4 ; MacAddr[j++] += RcvData[i++] ;
	MacAddr[j] = RcvData[i++] << 4 ; MacAddr[j++] += RcvData[i++] ;
	MacAddr[j] = RcvData[i++] << 4 ; MacAddr[j++] += RcvData[i++] ;
	MacAddr[j] = RcvData[i++] << 4 ; MacAddr[j++] += RcvData[i++] ;

	Print("\r     Write MAC Address : ") ;
	for (i=0;i<6;i++) Print("%02x ",MacAddr[i]) ;

	IICWrite((U32)iMacAddrPtr,MacAddr, sizeof(MacAddr)) ;
}

/*
 * Function : MacAddrRead
 * Description : MAC Address Read from IIC EEPROM
 */
void MacAddrRead(void)
{
	U8 MacAddr[6] ;
	int i, MacAddrSize ;
	
	MacAddrSize = sizeof(MacAddr) ;

	Print("\n >>> Read MAC Address : ") ;
	IICRead((U32)iMacAddrPtr,MacAddrSize,MacAddr) ;
	for (i=0;i<MacAddrSize;i++) Print(" %02x", MacAddr[i]) ;
}


/*
 * Function : CaptureRxFrame
 * Description : Capture Rx Frame Data
 */
void CaptureRxFrame(void)
{
	// Setup BDMA Rx Interrupt for Capturing
	// Disable  MAC Tx/Rx Interrupt and BDMA Tx Interrupt
 	MacInitialize() ; // Reinitialize MAC/BDMA Control
	SysSetInterrupt(nBDMA_RX_INT, BDMA_Rx_isr) ;	
	Disable_Int(nMAC_RX_INT) ;
	Disable_Int(nMAC_TX_INT) ;
	Disable_Int(nBDMA_TX_INT) ;	

	VPint(CAM_BaseAddr) = gCapture_Addr0 ;
	VPint(CAM_BaseAddr + 0x4) = gCapture_Addr1 ;


	// Initialize User Rx Frame Buffer Area
	RxFrameSequence = 0 ;
	BaseofUserFrame = (U32 *)RxFrameUserAreaBase ;

	Rtc_init(TIMER0) ;
        TimerStart(TIMER0);

	while(get_byte() != 'q') ;

	Print("\n\n >>> Received Frame Count is %d <<<",RxFrameSequence) ;

	// Disable MAC RX
	// Disable global interrupt mask
	MACRXCON &= ~RxEn ;
	Disable_Int(nGLOBAL_INT); 
	Timer0Stop() ;
	tm0.tick = 0 ; tm0.min = 0 ; tm0.hour = 0 ;
}

/*
 * Function : BdmaRxIsr
 * Description : EthernetReceive Function  Rx  Interrupt Service Routine
 *               BDMA Rx Interrupt Service Routine for Capturing
 */
void BdmaRxIsr(void) 
{
	U32 IntBDMARxStatus ;
	U32 pRxFramePointer ;

	IntBDMARxStatus = BDMASTAT;
	BDMASTAT = IntBDMARxStatus ;
	Clear_PendingBit(nBDMA_RX_INT) ;

	if (IntBDMARxStatus & S_BRxRDF) pRxFramePointer = BDMARXPTR ;
	if (!CheckCaptureStatus(pRxFramePointer)) 
			Print("\r MAC Rx Fail") ;
}

/*
 * Function : CheckCaptureStatus 
 * Description : Check Rx Status for Capturing Mode
 */
U32 CheckCaptureStatus(U32 pRxFramePointer)
{
	sRxFrameDescriptor *pRxFDptr;
	U32 *pFDPtr, CheckOwnerShip, RxStatus ;

	while (1) {
	// Get first of Frame Descriptor for check
	pRxFDptr = (sRxFrameDescriptor *)gCRxFDPtr ;

	pFDPtr = (U32 *)&pRxFDptr->NextFrameDescriptor ;
	gCRxFDPtr = (U32)*pFDPtr ;

	// 1. Check BDMA Receive Ownership and Status
	pFDPtr = (U32 *)&pRxFDptr->FrameDataPtr ;
  	CheckOwnerShip = *pFDPtr & fOwnership_BDMA ;

	if (!CheckOwnerShip) 
		RxStatus = (pRxFDptr->StatusAndFrameLength >> 16) & 0xffff;
	else 
		return (U32)MacRx_Fail ;

	if(RxStatus & RxGood) CopyCaptureFrame((U32)pFDPtr) ;
	else return (U32)MacRx_Fail ;

	// if All Received Frame is Checked
	if (gCRxFDPtr == pRxFramePointer) break ;

	} return (U32)MacRx_Ok ; // end of while loop
}


/*
 * Function : CopyCaptureFrame
 * Description : Copy Captured Rx Frame to User Area
 */
void CopyCaptureFrame(U32 RxFramePointer)
{
	sRxFrameDescriptor *pRxFDptr;
	U32 *pFDPtr, FrameLength, RxStatus, RxTime ;
	U32 *pFrameData;

	RxTime = tm0.tick | (tm0.min << 8) ;
	// 1. Get Frame Length and Status
	pRxFDptr = (sRxFrameDescriptor *)RxFramePointer ;
	FrameLength = pRxFDptr->StatusAndFrameLength & 0xffff ;
	RxStatus = (pRxFDptr->StatusAndFrameLength >> 16) & 0xffff ;

	pFDPtr = (U32 *)&pRxFDptr->FrameDataPtr ;
  	pFrameData = (U32 *)*pFDPtr;

	// Receoved Frame Data Store Format
	// +------------------+
	// | Preamble         |
	// | Sequence| Time   |
	// | Status  | Length |
	// | Rx Data          |
	// +------------------+

	if ( BaseofUserFrame > (U32 *)RxFrameUserAreaLimit) 
	{
		BaseofUserFrame = (U32 *)RxFrameUserAreaBase ;
		Print("\r*- Buffer is Full -*\r") ;
	}

	*BaseofUserFrame++ = RxFrameDataPreamble ; 
	*BaseofUserFrame++ = (RxFrameSequence++ << 16) | RxTime ;
	*BaseofUserFrame++ = (RxStatus << 16) | FrameLength  ; 

	// Move Rx Frame to User Area
	bcopy((U32 *)pFrameData,(U32 *)BaseofUserFrame,FrameLength) ;
	BaseofUserFrame += (U32)(FrameLength/4 + 1) ;

	// 2. Change Ownersip to BDMA for Next Use
   	*pFDPtr |= fOwnership_BDMA;

	// Clear Reserved & Status/Length Field
	pFDPtr = (U32 *)&pRxFDptr->Reserved;
	*pFDPtr = (U32)0x0 ;
	pFDPtr = (U32 *)&pRxFDptr->StatusAndFrameLength;
	*pFDPtr = (U32)0x0 ;
}

/*================================================================
 * View Received Frame Data  : Capture Frame Display
 * Used Function : ShowReceivedFrameData, PrintRxFrame
 *================================================================*/
/*
 * Function : ShowRecevedFrameData
 * Description : Show MAC Received Data
 */
void ShowRecevedFrameData(void)
{
	U32 *ViewAreaBase, *CurrentArea ;
	U32 FrameLength, RxStatus ;
	U32 RxSequence, RxTime , RxMin, RxSec ;
	U8 SrcAddr[6], DstAddr[6], LengthOrControlField[2] ;
	U8 *RxFrameData;
	U8 RcvData[12] ;
	U8 NextCheck ;
	char CheckViewFunction ;
 	int CheckSequence , CheckFrameLength ;
	int i,j, AddrMached=0, CheckPreamble=0 ;

	ViewAreaBase = (U32 *)RxFrameUserAreaBase ;

   do {
	Print("\n >>> Show Received Frame Data(%d Frame) <<<",\
					RxFrameSequence) ;
	Print("\n +--------------------------------------+") ;
	Print("\r | 1. Show All Received Frame  ---- [A] |") ;
	Print("\r | 2. Show by Rx Sequence  -------- [N] |") ;
	Print("\r | 3. Show by Rx Frame Length ----- [L] |") ;
	Print("\r | 4. Show by Source Address  ----- [S] |") ;
	Print("\r | 5. Show by Destination Address - [D] |") ;
	Print("\r | 6. Show Control Frame   -------- [C] |") ;
	Print("\r | 7. Quit       ------------------ [Q] |") ;
	Print("\r +--------------------------------------+") ;
	Print("\n >> Enter View Function ?") ;
	CheckViewFunction = get_byte() ;

	if ( (CheckViewFunction == 'N') || (CheckViewFunction == 'n') )
		{
		Print("\r $$ Enter Sequence Number (Digit) : ") ;
		CheckSequence = get_digit() ;
		}
	if ( (CheckViewFunction == 'L') || (CheckViewFunction == 'l') )
		{
		Print("\r $$ Enter Rx Frame Length (Digit) : ") ;
		CheckFrameLength = get_digit() ;
		}
	if ( (CheckViewFunction == 'S') || (CheckViewFunction == 's') )
		{
		Print("\r $$ Enter Source Address : ") ;
		i = 0 ;
		while ( (RcvData[i] = get_byte() ) != CR ) 
			{
			if ( (RcvData[i] >= '0') && (RcvData[i] <= '9') )
				RcvData[i] -= '0' ;
			else if ( (RcvData[i] >= 'A') && (RcvData[i] <= 'F') )
				RcvData[i] = RcvData[i] - 'A' + 10 ;
			else if ( (RcvData[i] >= 'a') && (RcvData[i] <= 'f') )
				RcvData[i] = RcvData[i] - 'a' + 10 ;
			i++ ;
			}
		}
	if ( (CheckViewFunction == 'D') || (CheckViewFunction == 'd') )
		{
		Print("\r $$ Enter Destination Address : ") ;
		i = 0 ;
		while ( (RcvData[i] = get_byte() ) != CR ) 
			{
			if ( (RcvData[i] >= '0') && (RcvData[i] <= '9') )
				RcvData[i] -= '0' ;
			else if ( (RcvData[i] >= 'A') && (RcvData[i] <= 'F') )
				RcvData[i] = RcvData[i] - 'A' + 10 ;
			else if ( (RcvData[i] >= 'a') && (RcvData[i] <= 'f') )
				RcvData[i] = RcvData[i] - 'a' + 10 ;
			i++ ;
			}
		}

   CurrentArea = ViewAreaBase  ;

   while (1) {
	CheckPreamble = 0 ;
	while (1) {
		if (*CurrentArea++ == RxFrameDataPreamble) break ;
		else if (CheckPreamble++ > 8) break ;
	}

	if (CheckPreamble > 8) { CheckPreamble = 0 ; break ; } 

	// Received Frame Data Store Format
	// +------------------+
	// | Preamble         |
	// | Sequence| Time   |
	// | Status  | Length |
	// | Rx Data          |
	// +------------------+
	RxSequence = (*CurrentArea >> 16) & 0xFFFF ;
	RxTime = *CurrentArea & 0xFFFF ; CurrentArea++ ;
	RxStatus = (*CurrentArea >> 16) & 0xFFFF ;
	FrameLength = *CurrentArea & 0xFFFF ; CurrentArea++ ;

	RxMin = (RxTime >> 8) & 0xFF ;
	RxSec = RxTime & 0xFF ;

  	RxFrameData = (U8 *)CurrentArea ; 
	for(i=0; i < 6; i++) DstAddr[i] = *RxFrameData++ ;
	for(i=0; i < 6; i++) SrcAddr[i] = *RxFrameData++ ;
	for(i=0; i < 2; i++) LengthOrControlField[i] = *RxFrameData++ ;

	if ( (CheckViewFunction == 'A') || (CheckViewFunction == 'a') ) {
	   Print("\n+++ %d'th Received Frame Data(%x)",RxSequence,RxStatus);
	   Print("\r -> Received Time  : %d min %d sec ",RxMin,RxSec ) ;
	   PrintRxFrame((U32)CurrentArea, FrameLength) ;
	}

	else if ( (CheckViewFunction == 'N') || (CheckViewFunction == 'n') ) {
	   if (RxSequence == CheckSequence)
	   {
	   Print("\n+++ %d'th Received Frame Data(%x)",RxSequence,RxStatus) ;
	   Print("\r -> Received Time  : %d min %d sec ",RxMin,RxSec ) ;
	   PrintRxFrame((U32)CurrentArea, FrameLength) ;
	   break ;
	   }
	}

	else if ( (CheckViewFunction == 'L') || (CheckViewFunction == 'l') ) {
	   if (FrameLength <= CheckFrameLength)
	   {
	   Print("\n+++ %d'th Frame is Less than %d Byte(%x)",
			RxSequence,CheckFrameLength,RxStatus) ;
	   Print("\r -> Received Time  : %d min %d sec ",RxMin,RxSec ) ;
	   PrintRxFrame((U32)CurrentArea, FrameLength) ;
	   }
	}

	else if ( (CheckViewFunction == 'S') || (CheckViewFunction == 's') ) {
	   i = 0 ; j = 0 ;
	   while(1)
	   {
	   	if (((SrcAddr[j]>>4) & 0xF) == RcvData[i++]) AddrMached = 1 ;
	   	else { AddrMached =0 ; break ; }
	   	if ((SrcAddr[j] & 0xF) == RcvData[i++]) AddrMached = 1 ;
	   	else { AddrMached =0 ; break ; }

	   	if (j==5) break ; else j++ ;
	   }

	   if ( AddrMached )
	   {
	   Print("\n+++ %d'th Frame is Source Address Mached(%x)",\
					RxSequence,RxStatus) ;
	   Print("\r -> Received Time  : %d min %d sec ",RxMin,RxSec ) ;
	   PrintRxFrame((U32)CurrentArea, FrameLength) ;
	   }	
	}

	else if ( (CheckViewFunction == 'D') || (CheckViewFunction == 'd') ) {
	   i = 0 ; j = 0 ;
	   while(1)
	   {
	   	if (((DstAddr[j]>>4) & 0xF) == RcvData[i++]) AddrMached = 1 ;
	   	else { AddrMached =0 ; break ; }
	   	if ((DstAddr[j] & 0xF) == RcvData[i++]) AddrMached = 1 ;
	   	else { AddrMached =0 ; break ; }

	   	if (j==5) break ; else j++ ;
	   }

	   if ( AddrMached )
	   {
	   Print("\n+++ %d'th Frame is Destination Address Mached(%x)",\
					RxSequence,RxStatus) ;
	   Print("\r -> Received Time  : %d min %d sec ",RxMin,RxSec ) ;
	   PrintRxFrame((U32)CurrentArea, FrameLength) ;
	   }	
	}

	else if ( (CheckViewFunction == 'C') || (CheckViewFunction == 'c') ) {
		if ( ( LengthOrControlField[0] == 0x88 ) && \
			( LengthOrControlField[1] == 0x08 ) )
		{
	   	Print("\n+++ %d'th Frame is Control Frame(%x)", \
				RxSequence, RxStatus) ;
	   	Print("\r -> Received Time  : %d min %d sec ",RxMin,RxSec ) ;
	   	PrintRxFrame((U32)CurrentArea, FrameLength) ;
		}
	}

	// For Next Frame
	CurrentArea += FrameLength/4 ; 
   }
	
   Print("\n Press Any Key to Continue") ;
   NextCheck = get_byte() ;
   if ((CheckViewFunction=='Q')||(CheckViewFunction == 'q')) 
	{
	ClearUserFrameArea() ;
	MACRXCON &= ~RxEn ;
	Disable_Int(nGLOBAL_INT); 
	break ;
	}
	
   } while  ( (NextCheck != 'Q') && (NextCheck != 'q') ) ;

}

/*
 * Function : ClearUserFrameArea
 * Description : Clear MAC Received Data
 */
void ClearUserFrameArea(void) 
{
	U32 *ClearArea ; 
	U32 FrameLength, CheckPreamble = 0;

	ClearArea = (U32 *)RxFrameUserAreaBase ;

   	while (1) {
		while (1) {
			if (*ClearArea == RxFrameDataPreamble) 
			{
			*ClearArea++ = 0x0 ; break ;
			}
			else {
			if (CheckPreamble++ > 8) break ;
			*ClearArea++ = 0x0 ;
			}
		}
		if (CheckPreamble > 8) break ; CheckPreamble = 0 ;

		*ClearArea++ = 0x0 ;
		FrameLength = *ClearArea & 0xFFFF ; 
		FrameLength = FrameLength / 4 - 1 ;
		while(FrameLength--) *ClearArea++ = 0x0 ;
   	}
}
