/*  ks32c5000/ES0 : d:/ks32c5000/program/BDTest/cache.c     98/02/13  */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/cache.c                                           */
/*   DATE:    98/02/13                                                 */
/*   PURPOSE: Cache & internal SRAM test module.                       */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by Young-sun kim                                    */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*   The functions in this file are:                                   */
/*                                                                     */
/*   CacheTest      cache test top module                              */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include "ks5000.h"
#include "pollio.h"
#include "memory.h"
#include "uart.h"
#include "system.h"
#include "math.h"
#include "sysconf.h"


/***********************************************************************
 *           CACHE & INTERNAL STRAM TEST TOP MODULE                    *
 ***********************************************************************/
void CacheTest(void)
{
     char items;

     IOPDATA= ~0x5;  //in Cache test mode

     do {
           PrintCacheItems(); // Main memu for cache test

           do { 
                 Print("\rSelect Number?_");
                 items = get_byte();
           }while(is_space(items));

           if(is_xdigit(to_upper(items))) {

               switch(to_upper(items)) {
                      case '1': TagMarch10();            break;
                      case '2': SramMarch10_4k();        break;
                      case '3': SramMarch10_8k();        break;
                      case '4': Set0March10();        break;
                      case '5': Set1March10();        break;
                      case '6': InterSramTest();         break;
                      case '7': CacheStartUpDialog();    break;
                      case '8': CacheConfig();  break;
                      case '9': CpuClockCon();  break;
                      case 'A': CacheClear();   break;
                      case 'B': SearchPattern();   break;
                      default : break;
              }
           }
           Print("\rPress any key to continue.\r");
           while(!is_space(get_byte()));

     } while((items!='q')&&(items!='Q'));
}


/* 
 * Display Cache & SRAM test items 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void PrintCacheItems(void)
{
        Print("\n\n");
	Print("+---------------------------------------------------+\r") ;
        Print("|       KS32C5000 CACHE & SRAM TEST ITEMS           |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| 1. Tag RAM(256words) 10N march test.              |\r") ;
	Print("| 2. Cache SRAM(4kbytes) 10N march test.            |\r") ;
	Print("| 3. Cache SRAM(8kbytes) 10N march test.            |\r") ;
	Print("| 4. Cache Set0(4kbytes) 10N march test.            |\r") ;
	Print("| 5. Cache Set1(4kbytes) 10N march test.            |\r") ;
	Print("| 6. Internal SRAM R/W test at 4k or 8k mode.       |\r") ;
	Print("|    ->SRAM address used data pattern.              |\r") ;
	Print("| 7. Cache start up dialog for setup cache mode.    |\r") ;
	Print("| 8. Cache configurations.                          |\r") ;
	Print("| 9.Change CPU Clock use by clkcon register.        |\r") ;
	Print("| A.Cache Memory(SET0[4k]/SET1[4k]) Clear function. |\r") ;
	Print("| B.Pattern search in cache memory region.          |\r") ;
	Print("| Q.QUIT - Return main menu.                        |\r") ;
	Print("+---------------------------------------------------+\r") ;
}


void SearchPattern()
{
	unsigned long int pattern;
	unsigned long int *Set0Pointer;
	unsigned long int *Set1Pointer;
	unsigned long int *Dram0Pointer;
	unsigned long int i;

        Print("Press any key\r");
        get_byte();
        Print("\rEntern Search Pattern."); 
	pattern = get_num();
	Set0Pointer = (unsigned long int *)(Set0BaseAddress);
	Set1Pointer = (unsigned long int *)(Set1BaseAddress);
	Dram0Pointer = (unsigned long int *)(0x1000000);

	for (i=0; i<1024; i++) {
		if (pattern == Set0Pointer[i]) {
			Print("Set 0 : Address = %08x, Data = %08x\r",
				Set0Pointer + i, Set0Pointer[i]);
		}
	}

	for (i=0; i<1024; i++) {
		if (pattern == Set1Pointer[i]) {
			Print("Set 1 : Address = %08x, Data = %08x\r",
				Set1Pointer + i, Set1Pointer[i]);
		}
	}

	for (i=0; i<1024*1024; i++) {
		if (pattern == Dram0Pointer[i]) {
			Print("DRAM 0 : Address = %08x, Data = %08x\r",
				Dram0Pointer + i, Dram0Pointer[i]);
		}
	}
}


/* 
 *   Internal SRAM read & write, compare test 
 *   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void InterSramTest(void)
{
     unsigned int *base;   // Internal SRAM Base Address
     char ch;

     base =  (unsigned int *)InterSramArea;

     /* get SRAM R&W test pattern */
     //Print("\n$Enter SRAM R&W test pattern[8 hexa digit].\r");
     //pat = gethex2dec(8); // get test pattern
     //Print("$Entered Test pattern is 0x%08x\r",pat);
     Print("\r$ 8K SRAM test? If NO,default 4K SRAM tested?[Y/N]"); 
     ch = get_byte();
           
     if((ch=='Y')||(ch=='y')) {
   
         SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_10;
         Print("\r\r$SRAM size is 4K-bytes.\r");
         Print("[Cache Mode]SYSCFG=0x%08x.\r",SYSCFG);
         Print("$SRAM Address:0x%08x~0x%08x\r",base,base+2048); 
         Print("====================================\r\r",base,base+2048); 
         SelfTestMem(base,2048);
     }
     else {
            SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_00;
            Print("\r\r$SRAM size is 4K-bytes.\r");
            Print("\r[Cache Mode]SYSCFG=0x%08x.\r",SYSCFG);
            Print("$SRAM Address:0x%08x~0x%08x\r",base,base+1024); 
            Print("====================================\r\r",base,base+1024); 
            SelfTestMem(base,1024);
     }
}



void SelfTestMem(unsigned int *src, int unit)
{
    unsigned int *tempAddr;
    int i;
    int j;


    tempAddr = src;

    for(i=0;i<unit;i++) {
        *tempAddr = (unsigned int)tempAddr;     //write memory data
        tempAddr++;
    }


    tempAddr = src;

    for(i=0;i<unit;i+=j) {
            Print("\r[%08x]= ",tempAddr);
            for(j=0;j<4;j++) {
                 Print(" %08x", *tempAddr);
                 tempAddr++;
            }
    }
}

/*
 *  Setup System manager dialog for Cache & SRAM test
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void CacheStartUpDialog(void)
{
     char ch;
     unsigned int n;
     REG32  Reg;

     Print("\n$ Start System Configurations....\r\r"); 

     /**********************************/
     /* Select Stall enable or disable */  
     /**********************************/
     do {
           Print("\n>>Stall Enable?[Y/N]_"); 
           ch=get_byte(); 
     } while((ch!='N')&&(ch!='n')&&(ch!='Y')&&(ch!='y'));


     if((ch=='Y')||(ch=='y')) 
           SYSCFG |= STALL;
     else
           SYSCFG &= ~STALL;
     

     /**************************************/
     /* Select write buffer enable/disable */  
     /**************************************/
     do {
           Print("\r>>Write buffer Enable?[Y/N]_"); 
           ch=get_byte(); 
     } while((ch!='N')&&(ch!='n')&&(ch!='Y')&&(ch!='y'));

     if((ch=='Y')||(ch=='y')) 
           SYSCFG |= WRITE_BUFF; 
     else 
           SYSCFG &= ~WRITE_BUFF; 


     /**************************************/
     /*       Select Cache mode            */
     /**************************************/

     SYSCFG &= ~CACHE;      // Disable cache before cache mode change  
     Reg = (~CACHE_MODE & SYSCFG); // clear previous CM bit
     do {
           
           Print("\r\r>>>>>>>>>>  Cache Mode <<<<<<<<<<<<<\r"); 
           Print("=====================================\r\r"); 
           Print("0. 4K-bytes SRAM, 4K-bytes Cache.\r"); 
           Print("1. 0K-bytes SRAM, 8K-bytes Cache.\r"); 
           Print("2. 8K-bytes SRAM, 0K-bytes Cache.\r"); 
           Print("=====================================\r\r"); 
           Print("$ Select number.>");
           n = get_number();
      
           if(n < 3) 
               Reg |= (CACHE_MODE & (n<<4)); 
      } while(n > 2);

      SYSCFG = Reg;


     /*******************************/
     /* Select Cache enable/disable */  
     /*******************************/
     do {
           Print("\r>>Cache Enable?[Y/N]_"); 
           ch=get_byte(); 
     } while((ch!='N')&&(ch!='n')&&(ch!='Y')&&(ch!='y'));


     CacheFlush();   // Tag ram clear before cache enable

     if((ch=='Y')||(ch=='y')) 
           SYSCFG |= CACHE; 
     else 
           SYSCFG &= ~CACHE; 

}



/* 
 *  Cache flush function for re-configuration cache mode
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void CacheFlush(void)
{
    int i;
    unsigned int *tagram; 

    tagram = (unsigned int *)TagRAM;
    SYSCFG &= ~CACHE;  // Disable cache before cache flush

    for(i=0; i < 256; i++) {
        *tagram = 0x00000000;   // Clear tag ram 
        tagram += 1; 
    }

    //Print("\nCache Flushed!!\r");
}



/* 
 *  Set0, Set1 Cache memory clear function
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void CacheClear(void)
{
    unsigned int *set0;
    unsigned int *set1;
    int i;

    set0 = (unsigned int *)Set0CacheRAM;
    set1 = (unsigned int *)Set1CacheRAM;


    Print("\nSET0(4k),SET1(4K) Cache memory clear Start!\r");

    SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_01;
    Print("SYSCFG=0x%08x: 8k Cache.\r",SYSCFG);

    for(i=0; i < 1024; i++) {
        *set0 = 0x00000000;   // Clear tag ram 
        set0++; 
    }

    Print("\nSET0 Cache Momory Cleared!!\r");

    for(i=0; i < 1024; i++) {
        *set1 = 0x00000000;   // Clear tag ram 
        set1++; 
    }

    Print("\nSET1 Cache Momory Cleared!!\r");
}


/*
 *  Cache Memory(256 word Tag RAM,8k-bytes SRAM 
 *  Cache memory test use by 10N march algorithm
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  - Cache memory address range
 *     Tag  = 0x11000000~0x11000400 (256word)
 *  
 *     . at Cache Mode : 8kb SRAM, 0kb Cache
 *        SRAM = 0x3fe0000~0x3fe02000  (8k bytes)
 */
void TagMarch10(void)
{
    unsigned int *tag;
    int tagflag = 0;  // Tag memory error flag for march test int delay = 0;

        tag  = (unsigned int *)TagRAM;

        /* Cache off,wr buf off,stall off */
        SYSCFG &= ~(WRITE_BUFF|CACHE|STALL);


        Print("\nTag(256words) Test by 10N march algorithm!\r");
        Print("[Cache Mode]SYSCFG=0x%08x.\r",SYSCFG);

        //Print("\n>>Enter memory read/write delay._");
        //delay = get_number();
        //Print("\r\r");
        Print("\r=====================================================");
        Print("\r  TAG[Address]= READ Pattern:EXPECTED Pattern");
        Print("\r=====================================================\r");

        /* tag(256 words) test by 10N march */
        if(March10N32(tag,256,0x00000000,0)) tagflag =1; 
        if(March10N32(tag,256,0x0000ffff,0)) tagflag =1; 
        if(March10N32(tag,256,0x00ff00ff,0)) tagflag =1; 
        if(March10N32(tag,256,0x0f0f0f0f,0)) tagflag =1; 
        if(March10N32(tag,256,0x33333333,0)) tagflag =1; 
        if(March10N32(tag,256,0x55555555,0)) tagflag =1; 

        if(tagflag) {
             tagflag = 0; // clear march error flag
             Print("[FAIL] 256word Tag memory 10N march test!!!!\r\r");
        }
        else
             Print("[PASS] OK! 256word Tag memory 10N march test!!!!\r\r");
}

void SramMarch10_4k(void)
{
    unsigned int *sram;
    int sramflag = 0;  // sram flag for march error
    //int delay = 0;     // memory read & write cycle delay

       sram = (unsigned int *)InterSramArea;


        Print("\nSRAM(4K bytes) test by 10N march algorithm!\r");

        SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_00;
        Print("SYSCFG=0x%08x:4Kb SRAM,4Kb Cache.\r",SYSCFG);

        //Print("\n>>Enter memory read/write delay._");
        //delay = get_number();
        //Print("\r\r");
        Print("\r     SRAM[Address]= READ Pattern:EXPECTED Pattern");
        Print("\r=====================================================");

        /* SRAM(2048 words) test by 10N march */
        if(March10N32(sram,1024,0x00000000,0)) sramflag =1; 
        if(March10N32(sram,1024,0x0000ffff,0)) sramflag =1; 
        if(March10N32(sram,1024,0x00ff00ff,0)) sramflag =1; 
        if(March10N32(sram,1024,0x0f0f0f0f,0)) sramflag =1; 
        if(March10N32(sram,1024,0x33333333,0)) sramflag =1; 
        if(March10N32(sram,1024,0x55555555,0)) sramflag =1; 

        if(sramflag) {
             sramflag = 0; // clear march error flag
             Print("\r[FAIL] SRAM 4kbyte 10N march test!!!!\r\r");
        }
        else {
               Print("\r[PASS] OK! SRAM 4kbyte 10N march test!!!!\r\r");
               sramflag =0;
        }
}



void SramMarch10_8k(void)
{
    unsigned int *sram;
    int sramflag = 0;  // sram flag for march error
    //int delay = 0;     // memory read & write cycle delay

       sram = (unsigned int *)InterSramArea;


        Print("\nSRAM(8K bytes) test by 10N march algorithm!\r");

        SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_10;
        Print("SYSCFG=0x%08x:8Kb SRAM,0Kb Cache.\r",SYSCFG);

        //Print("\n>>Enter memory read/write delay._");
        //delay = get_number();
        //Print("\r\r");
        Print("\r     SRAM[Address]= READ Pattern:EXPECTED Pattern");
        Print("\r=====================================================");

        /* SRAM(2048 words) test by 10N march */
        if(March10N32(sram,2048,0x00000000,0)) sramflag =1; 
        if(March10N32(sram,2048,0x0000ffff,0)) sramflag =1; 
        if(March10N32(sram,2048,0x00ff00ff,0)) sramflag =1; 
        if(March10N32(sram,2048,0x0f0f0f0f,0)) sramflag =1; 
        if(March10N32(sram,2048,0x33333333,0)) sramflag =1; 
        if(March10N32(sram,2048,0x55555555,0)) sramflag =1; 

        if(sramflag) {
             sramflag = 0; // clear march error flag
             Print("\r[FAIL] SRAM 8kbyte 10N march test!!!!\r\r");
        }
        else {
               Print("\r[PASS] OK! SRAM 8kbyte 10N march test!!!!\r\r");
               sramflag =0;
        }
}



void Set1March10(void)
{
    unsigned int *set1;
    int setflag = 0;  // set0 flag for march error
    //int delay = 0;    // memory read & write cycle delay

       set1 = (unsigned int *)Set1CacheRAM;


        Print("\nCache[SET0]4K bytes tested by 10N march algorithm!\r");

        SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_01;
        Print("SYSCFG=0x%08x: 8k Cache.\r",SYSCFG);

        //Print("\n>>Enter memory read/write delay._");
        //delay = get_number();
        //Print("\r\r");
        Print("\r      SET1[Address]= READ Pattern:EXPECTED Pattern");
        Print("\r=====================================================");

        /* SET1(1024 words) test by 10N march */
        if(March10N32(set1,1024,0x00000000,0)) setflag =1; 
        if(March10N32(set1,1024,0x0000ffff,0)) setflag =1; 
        if(March10N32(set1,1024,0x00ff00ff,0)) setflag =1; 
        if(March10N32(set1,1024,0x0f0f0f0f,0)) setflag =1; 
        if(March10N32(set1,1024,0x33333333,0)) setflag =1; 
        if(March10N32(set1,1024,0x55555555,0)) setflag =1; 

        if(setflag) {
             setflag = 0; // clear march error flag
             Print("\r[FAIL] SET1 4kbyte 10N march test!!!!\r\r");
        }
        else {
               setflag = 0;
               Print("\r[PASS] OK! SET1 4kbyte 10N march test!!!!\r\r");
        }
}



void Set0March10(void)
{
    unsigned int *set0;
    int setflag = 0;  // set0 flag for march error
    //int delay = 0;    // memory read & write cycle delay

       set0 = (unsigned int *)Set0CacheRAM;


        Print("\nCache Set0,4K bytes,tested by 10N march algorithm!\r");

        SYSCFG = (SYSCFG & ~CACHE_CONF_ON)|CACHE_MODE_01;
        Print("SYSCFG=0x%08x: 8k Cache.\r",SYSCFG);

        //Print("\n>>Enter memory read/write delay._");
        //delay = get_number();
        //Print("\r\r");

        Print("\r        SET0[Address]= READ Pattern:EXPECTED Pattern");
        Print("\r=====================================================");
        /* SET0(1024 words) test by 10N march */
        if(March10N32(set0,1024,0x00000000,0)) setflag =1; 
        if(March10N32(set0,1024,0x0000ffff,0)) setflag =1; 
        if(March10N32(set0,1024,0x00ff00ff,0)) setflag =1; 
        if(March10N32(set0,1024,0x0f0f0f0f,0)) setflag =1; 
        if(March10N32(set0,1024,0x33333333,0)) setflag =1; 
        if(March10N32(set0,1024,0x55555555,0)) setflag =1; 

        if(setflag) {
             setflag = 0; // clear march error flag
             Print("\r[FAIL] SET0 4kbyte 10N march test!!!!\r\r");
        }
        else {
               Print("\r[PASS] OK! SET0 4kbyte 10N march test!!!!\r\r");
               setflag = 0;
        }
}
/*
 *  Main 10N March Algorithm
 *  ~~~~~~~~~~~~~~~~~~~~~~~~
 *  March10N32(base address, word size, pattern)
 */
int March10N32(unsigned int *addr,int unit,unsigned int pattern,int rwdelay)
{
    int i;
    int flag4march = 0; // error flag for march test
    unsigned int *tempAddr;
    
    tempAddr = addr;  
   
    /* Write pattern on memory */
    for(i=0;i<unit;i++)
    {
        *tempAddr++ = pattern;
    }
   
    /* Sub Algorithm for Read & Write compare in memory 
     * _MarchSub1(end or base address,memory word size,pattern,addr dir); 
     *1 means increment, -1 is decrement address 
     */
     if(MarchSub1(addr,unit,~pattern,1,rwdelay)) flag4march = 1;
     if(MarchSub1(addr,unit,pattern,1,rwdelay)) flag4march = 1;
     if(MarchSub1(addr+unit,unit,~pattern,-1,rwdelay)) flag4march = 1;
     if(MarchSub1(addr+unit,unit,pattern,-1,rwdelay)) flag4march = 1;      
    
     tempAddr = addr;

    /* Read & test memory */
    for(i=0;i<unit;i++)
    {
        if(*tempAddr != pattern){
             Print("\r[ERROR] Cache[0x%08x]=0x%08x:0x%08x",tempAddr,*tempAddr,pattern);
             flag4march = 1;
        }
        tempAddr++;
    }

    return(flag4march);  //return error code
}



/*
 *  This function is parts of 10N march Algorithms
 *  Memory  read[pattern] & write[~pattern] or 
 *          read[~pattern] & write[~pattern] 
 *  on increment or decrement address.
 *
 *  MarchSub1(base or end address, word size, pattern, address direction)
 */
int MarchSub1(unsigned int *addr,int unit,unsigned int pattern,int dir,int d)
{
    unsigned int i;
    unsigned int j =0;
    unsigned int flag4march = 0;
    unsigned int rdata;
    unsigned int *tempAddr;

    tempAddr = addr;

    for(i=0;i<unit;i++)
    {
        j++; 
        j++; 
        rdata = *addr;     //read memory data
        //CycleDelay(d);
 
        //Print(">Read data[0x%08x] is 0x%08x\r", addr,rdata);
        *addr = pattern;   //write data pattern
        //CycleDelay(d);
        //Print(">write pattern is 0x%08x\n", ~pattern);
        
        if(rdata!=(~pattern)) {
             Print("\r[ERROR] Cache[0x%08x]=0x%08x:0x%08x", addr,*addr,~pattern);
             flag4march =1; // set flag for error
        }

        addr += dir;
    }

    return(flag4march); // return error code
}


/* 
 * delay time function with CPU clock cycle
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void CycleDelay(int d)
{
  int i;

  for(i=0; i < d; i++);
}

/* 
 *  Cache & SRAM control Status
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void CacheConfig(void)
{
        Print("\n\r>> System Manager Status.\r") ;
        Print("=============================\r") ;
        Print("SYSCFG[0x%08x]=0x%08x\r",&SYSCFG,SYSCFG);

	if (SYSCFG & STALL) 
	      Print(">Stall Enabled\r") ;
	else 
	      Print(">Stall Disabled\r") ;

	if (SYSCFG & CACHE) 
	      Print(">Cache Enabled\r") ;
	else 
	      Print(">Cache Disabled\r") ;

	if (SYSCFG & WRITE_BUFF) 
	      Print(">Write Buffer Enabled\r") ;
	else 
	      Print(">Write Buffer Disabled\r") ;

	if ((SYSCFG & CACHE_MODE) == 0x00 ) 
	      Print(">4K SRAM,4K CACHE\r") ;
	else if ((SYSCFG & CACHE_MODE) == 0x10 ) 
	           Print(">0K SRAM,8K CACHE\r") ;
	     else if ((SYSCFG & CACHE_MODE) == 0x20 ) 
			Print(">8K SRAM,0K CACHE\r") ;

        Print("=============================\r\r") ;
}


/**********************************************************************
 *                    CLOCK CONTROL FUNCTION                          *
 **********************************************************************/

/* 
 * Calculate CPU Clock from arqument divider value
 */
unsigned int fCPU_c(int devider)
{
     unsigned int CpuClock; 

     CpuClock = fMCLK/(devider+1); 
     return(CpuClock); 
}


/* 
 * Get CPU Clock from CLKCON divider value
 */
unsigned int fCPU_r(void)
{
     unsigned int CpuClock; 
     unsigned long Devider; 

     Devider = CLKCON & 0x0000ffff;
     CpuClock = fMCLK/(Devider+1); 
     return(CpuClock); 
}


/* CPU clock control function */
void CpuClockCon(void)
{
     unsigned int devider;
     int i;
     char ch;
     unsigned int CpuClk; 
     unsigned int BaudRate; 
     unsigned long BuffReg; 
     unsigned long RefCycle;  //for refresh cycle


     do {
           Print("\r$ Enter Clock divider value for new CPU clock![0~0xffff]_");
           devider = gethex2dec(4);  //argument value is the number of hexa digit.
          
           CpuClk = fCPU_c(devider);
           Print("\r CPU clock frequency is %d Mhz\r", CpuClk/1000000); 

           Print("\r$Reset clkcon register?[Y/N]_"); 
 
     } while(((ch=get_byte()) != 'N')&& (ch != 'n'));          

     /* Write devider value to clkcon register for change CPU clock  */
     CLKCON = devider;                  // Set CPU clock will be fMCLK/(devider+1)
     for(i=0; i < Delay4ClkCange; i++); //delay for clock stabilizations


     /* Change UART Board Rate */
     BaudRate = fMCLK/((UARTBRD0+1)*16);            // Get original board rate
     UARTBRD0 = (fCPU_r()/(16* BaudRate)-1); // Set new board rate value
     BaudRate = fMCLK/((UARTBRD1+1)*16);            // Get original board rate
     UARTBRD1 = (fCPU_r()/(16* BaudRate)-1); // Set new board rate value


     /* Change DRAM refresh cycle */
     BuffReg = (REFEXTCON & 0xffe00000) >> 21;                //Get refresh count value 
     RefCycle = (pow(2,11) - BuffReg +1)/fMCLK;               //Get original refresh cycle  
     BuffReg =   pow(2,11) +1 - (RefCycle * fCPU_r()); //Calculate refresh cycle 
     REFEXTCON = (REFEXTCON & ~0xffe00000)|(BuffReg << 21);   //Set refresh cycle

     Print("\rUart Board Rate & DRAM refresh cycle were changed automaitcally.\r");
     Print("\rCLKCON=0x%08x,",CLKCON);
     Print("UARTBRD0=0x%08x,",UARTBRD0);
     Print("UARTBRD1=0x%08x,",UARTBRD1);
     Print("REFEXTCON=0x%08x\r",REFEXTCON);
}

/* 
 *   Setup DRAM refresh cycle  
 *   where refcyc's unit is ms.
 */
void RefreshCycle(unsigned int refcyc)
{
     unsigned long RefCnt;

     RefCnt = pow(2,11)+1-(refcyc * (fMCLK/1000));

     /*  Set refresh cycle  */
     REFEXTCON = (REFEXTCON & ~0xffe00000)|(RefCnt << 21);   

     Print("\r\r\rRefresh Count value is 0x%08x.\r",(REFEXTCON &0xffe00000));
}


void RefCycleCal(void)
{
     U32 RefCnt;
     U32 RefCntReg;
     float RefCycle;
      
     RefCnt = ((REFEXTCON&0xffe00000)>>21);
     RefCntReg =  REFEXTCON;

     RefCycle = (pow(2,11)-RefCnt +1)/(fMCLK/1000000);

     dbg_out("\r\r\r> Refresh Count Register value is 0x%x.", RefCntReg);
     dbg_out("\r> Refresh cycle is %fus.\r",RefCycle);
}

