/*  ks32c5000/ES0 : d:/ks32c5000/program/BDTest/timer.c     98/02/045  */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/timer.c                                           */
/*   DATE:    98/02/05                                                 */
/*   PURPOSE: timer0, timer1 functional modules.                       */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by Young-sun kim                                    */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*                                                                     */
/*   The functions in this file are:                                   */
/*                                                                     */
/*                                                                     */
/*    TimerTest      Timer test top module                             */
/*    Rtc_init       Initialize the timer with 1 sec unit              */
/*    tmReset        Clear timer counter and interrupt pending,        */
/*                   Restart timer                                     */
/*    tmCntr         Change the timer mode,periode,interrupt           */
/*    tmFreq         Read current timer frequency                      */
/*    tmPeriod       Return time periode[ms], argument value is        */
/*                   TDATA from stdin                                  */
/*                                                                     */
/***********************************************************************/

#include <stdio.h>
#include "ks5000.h"
#include "uart.h"
#include "pic.h"
#include "timer.h"
#include "pollio.h"
#include "sysconf.h"


TIME tm0 = {0,0,0,0,0};
TIME tm1 = {0,0,0,0,0};

/* Timer Test main routine */
void TimerTest(void)
{
        unsigned char ch;   // variable for io consol 
        unsigned char dev;  // for to select timer device 0,1 

        /* -------------------------*/
        /*  Setup interrupt vector */   
        /* -------------------------*/
	SysSetInterrupt(nTIMER0_INT, tm0isr);
	SysSetInterrupt(nTIMER1_INT, tm1isr);


        /* -------------------------*/
        /* --- use for debugging ---*/
        /* -------------------------*/
	IOPMOD |= 0x1F;        // Use output port for LED display
        IOPDATA = ~0x6;        // In timer mode


        /* ************************************** */
        /* Select timer device. Timer0 or Timer 1 */
        /* ************************************** */
        do {
             Print("\r\r$ Select timer[0/1]_");
             dev = get_byte();
        } while(dev != '1' && dev != '0');


        /* ************************************** */
        /* Choose timer test methode              */
        /* ************************************** */
        do {
              Print("\r$Use startup dialog?[Y/N]_");
              ch=get_byte();
        } while((ch!='N')&&(ch!='n')&&(ch!='Y')&&(ch!='y'));


        if((ch=='Y')||(ch=='y')) {
              Print("\r***************************************\r");
              Print(">> You select timer startup dialog <<\r");
              Print("***************************************\r\r");
           
              TimerStartUpDialog(dev);    // Dialog for Timer parameter set  
        }
        else {
              Print("\n*****************************************\r");
              Print(">> You select timer default value for test <<\r");
              Print("*****************************************\r\r");
        
              if(dev=='1') {
                        Rtc_init(TIMER1);  // Timer initialize & start  
                        TimerStart(TIMER1);
              }
              else {
                        Rtc_init(TIMER0);  // Timer initialize & start  
                        TimerStart(TIMER0);
              }
        }

       Enable_Int(nGLOBAL_INT); //enable global interrupt mask
 
        /* -------------------------*/
        /* Now Start timer for test */
        /* -------------------------*/

       Print("\r\r>> Timer is Running......\r\r");
       IOPDATA = 0x1f;     // All LED OFF

       while(1) DisplayTime(dev);
}



/* 
 *  Timer initialize function 
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void Rtc_init(int TMn)
{
   if(TMn) {
              TDATA1 = tmDATA(1); // will be 1sec at fMCLK=33Mhz
              TCNT1 = 0x0;
              TMOD = TM1_OUT_1; // default interval, Tout high 
              IntMode |= (1<<nTIMER1_INT); // default FIQ mode
              IntMask &= ~TIMER1_INT; // Timer interrupt enable
   }
   else {
              TDATA0 = tmDATA(1);  // will be 1sec at fMCLK=33Mhz
              TCNT0 = 0x0;
              TMOD = TM0_OUT_1; // default interval, Tout high 
              IntMode |= (1<<nTIMER0_INT); // default FIQ mode
              IntMask &= ~TIMER0_INT; // Timer interrupt enable
   } 
}


void tm_init_ms(int TMn)
{
   if(TMn) {
              TDATA1 = (U32)tmDATA(tmScale_ms(1)); // will be 1ms at fMCLK=33Mhz
              //TDATA1 = 0x80e7; // will be 1ms at fMCLK=33Mhz
              TCNT1 = 0x0;
              TMOD = TM1_OUT_1; // default interval, Tout high 
              IntMode |= (1<<nTIMER1_INT); // default FIQ mode
              IntMask &= ~TIMER1_INT; // Timer interrupt enable
   }
   else {
              TDATA0 = (U32)tmDATA(tmScale_ms(1)); // will be 1ms at fMCLK=33Mhz
              //TDATA0 = 0x80e7; // will be 1ms at fMCLK=33Mhz
              TCNT0 = 0x0;
              TMOD = TM0_OUT_1; // default interval, Tout high 
              IntMode |= (1<<nTIMER0_INT); // default FIQ mode
              IntMask &= ~TIMER0_INT; // Timer interrupt enable
   } 
}


/* 
 * Timer reset function 
 * ~~~~~~~~~~~~~~~~~~~~
 */
void tmReset(int TMn)
{
    if(TMn) {
              IntPend &= TIMER1_INT;  //Clear timer pending register
              TCNT1 = 0x0;            //Clear timer counter register
              TMOD |= TM1_RUN;        //Start Timer
    }
    else {
              IntPend &= TIMER0_INT;  //Clear timer pending register
              TCNT0 = 0x0;            //Clear timer counter register
              TMOD |= TM0_RUN;        //Start Timer
    }
}


/*
 *  Timer mode setup control function
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void tmCntr(char TMn,TM_PARAM *iopb)
{
   if(TMn == '1') {
             if(iopb->tmtoggle) TMOD |= TM1_TOGGLE; 
             if(iopb->toutvalue) TMOD |= TM1_OUT_1;
             TDATA1 = iopb->tmdata;             
             if(iopb->toen) IOPCON |= BIT31; // enable Timer1 out
             if(!iopb->tmintmask) IntMask &= ~TIMER1_INT;
             if(iopb->tmintmode) IntMode |= (1<<nTIMER1_INT);
   }
   else {
             if(iopb->tmtoggle) TMOD |= TM0_TOGGLE; 
             if(iopb->toutvalue) TMOD |= TM0_OUT_1;
             TDATA0 = iopb->tmdata;             
             if(iopb->toen) IOPCON |= BIT30; //enable Timer0 out
             if(!iopb->tmintmask) IntMask &= ~TIMER0_INT;
             if(iopb->tmintmode) IntMode |= (1<<nTIMER0_INT);
   }
} 


float tmPeriod(U32 tdata)
{
   return((float)((tdata)*tMCLK));  //:unit[ns]
}

/*  
 *  =============================================
 *  Timer Start For Dhrystone Program: Use No Interrupt
 *  =============================================
 */
unsigned long TimeStopCheck(int TMn)
{
	float TIME ;

	if (TMn) { 
		TIME = (float)TCNT1 ;
		Timer1Stop() ; 
		 }
	else { 
		TIME = (float)TCNT0 ;
		Timer0Stop() ; 
		}

	TIME = (float)(0xFFFFFFFF - TIME) ;

	//Print("\r +------ Time Stop : %d\r",(int)TIME);

	return (TIME-1);
}



void DisplayTime(char timer)
{
    char buff[100];
    int i;

    if(timer == '1') {
        sprintf(buff,"TIMER1: %d month,%d day,time is %d:%d:%d.",tm1.day,tm1.month,tm1.hour,tm1.min,tm1.tick);
        put_string(buff);
        for(i=0;i < sizeof(buff);i++)
            put_byte('\b'); 
    }
    else { 
        sprintf(buff,"TIMER0: %d month,%d day,time is %d:%d:%d.",tm0.month,tm0.day,tm0.hour,tm0.min,tm0.tick);
            put_string(buff);
            for(i=0;i < sizeof(buff);i++)
                     put_byte('\b'); 
    }
}

char *getTimeSec(char timer)
{
    char buff[250];

    if(timer == '1') {
        sprintf(buff,"Transfer time is %d:%d:%d.",tm1.hour,tm1.min,tm1.tick);
        return(buff);
    }
    else { 
           sprintf(buff,"Transfer time is %d:%d:%d.",tm0.hour,tm0.min,tm0.tick);
            return(buff);
    }
}



char *getTimeMs(char timer)
{
    char buff[100];

    if(timer == '1') {
        sprintf(buff,"Transfer time is %dms.",(tm1.min*60+tm1.tick));
        return(buff);
    }
    else { 
           sprintf(buff,"Transfer time is %dms.",(tm0.min*60+tm0.tick));
           return(buff);
    }
}


void ClrTimeVar(void)
{
   tm0.tick = 0;
   tm0.min = 0;
   tm0.hour = 0;
   tm0.day = 0;
   tm0.month = 0;

   tm1.tick = 0;
   tm1.min = 0;
   tm1.hour = 0;
   tm1.day = 0;
   tm1.month = 0;
}


/***********************************/
/* timer interrupt service routine */
/***********************************/

void tm0isr(void)
{
    Clear_PendingBit(nTIMER0_INT);

    tm0.tick++;

    /* Count minute */
    if(tm0.tick > 59) {
         tm0.min++;
         tm0.tick = 0;
    }

    /* Count hour */
    if(tm0.min > 59) {
          tm0.hour++;
          tm0.min = 0;
    }

    /* Count hour */
    if(tm0.hour > 23) {
           tm0.hour = 0;
           tm0.day++;
    }

    /* Count day */
    if(tm0.day > 30) {
           tm0.day = 0;
           tm0.month++;
    }

    /* Reset month */
    if(tm0.month > 11) {
           tm0.month =0;
    }

/*
#ifdef  DEBUG
    IOPDATA = ~(1<<tm0.tick%5); // 4 means digit number for LED display
#endif
*/

}


void tm1isr(void)
{
    Clear_PendingBit(nTIMER1_INT);

    tm1.tick++;

    /* Count minute */
    if(tm1.tick > 59) {
         tm1.min++;
         tm1.tick = 0;
    }

    /* Count hour */
    if(tm1.min > 59) {
          tm1.hour++;
          tm1.min = 0;
    }

    /* Count hour */
    if(tm1.hour > 23) {
           tm1.hour = 0;
           tm1.day++;
    }

    /* Count day */
    if(tm1.day > 30) {
           tm1.day = 0;
           tm1.month++;
    }

    /* Reset month */
    if(tm1.month > 11) {
           tm1.month =0;
    }

//#ifdef  DEBUG
    //IOPDATA = ~(1<<tm1.tick%5); // 4 means digit number for LED display
//#endif
 
}



/*******************************************************
 *  Start up dialog function for timer parameter setup
 *  & Timer start and stop control.
 *
 *******************************************************/
void TimerStartUpDialog(char device)
{
     char       ch; 
     U32        i;
     TM_PARAM   tmpb;  // Define Timer 1 parameter              


     /* ************************************** */
     /* Stop nonselected timer                 */
     /* ************************************** */
     do {
           Print("\rDirect write timer data?[Y/N]");
           ch = get_byte();
           ch = to_upper(ch);

           if(ch == 'Y') {
                  Print("\r$ Enter timer data register[8hexdigit]_");
                  i = gethex2dec(8); get_byte(); //dummy for carrige return
                  tmpb.tmdata = i;

                  Print("\r $Input timer data = 0x%x.\r", tmpb.tmdata); 
                  dbg_out(" $Time period will be %fns.\r",tmPeriod(tmpb.tmdata));
           }
           else {
                   Print("\r$ Enter time step[sec]");
                   i = get_digit(); get_byte();
                   tmpb.tmdata = (U32)tmDATA(i);

                   Print("\r - Time step is %d[sec]",i);
                   Print("\r - Timer DATA Register value = 0x%x\r", tmpb.tmdata); 
          }

          Print("\r$Resetting timer data value?[Y/N]"); 
 
     } while(((ch=get_byte()) != 'N')&& (ch != 'n'));          

              
     /* ************************************** */
     /* Select timer interval or toggle mode   */
     /* ************************************** */
     do {
           Print("\r$ Select mode(interval/toggle)?[0/1]_");      
           ch = get_byte();
     } while(ch != '1' && ch != '0');
           
     if(ch == '1') {
              tmpb.tmtoggle = 1;
              Print("\r$Selected mode is toggle mode.\r");
     }
     else {
              tmpb.tmtoggle = 0;
              Print("\r$Selected mode is interval mode.\r");
     }


     /* ************************************** */
     /* Select timer initial TOUT value?       */
     /* ************************************** */
     do {
           Print("\r$ Select initial TOUT value?[0/1]_");      
           ch = get_byte();
     } while(ch != '1' && ch != '0');
           
     if(ch == '1') {
              tmpb.toutvalue = 1;
              Print("\r$Timer TOUT value will be 1.\r");
     }
     else {
              tmpb.toutvalue = 0;
              Print("\r$Timer TOUT value will be 0.\r");
     }
     
     /* ************************************** */
     /* Timer pulse port output control        */
     /* ************************************** */
     do {
           Print("\r$ Select timer output disable/enable?[0/1]_");      
           ch = get_byte();
     } while(ch != '1' && ch != '0');
           
     if(ch == '1') {
               tmpb.toen = 1;
               Print("\r$Timer output to port is enabled.\r");
     }
     else {
               tmpb.toen = 0;
               Print("\r$Timer output to port is disabled.\r");
     }


     /* ************************************** */
     /* Timer interrupt masking control        */
     /* ************************************** */
     do {
           Print("\r$ Timer interrupt masking?[0:enable/1:disable]_");      
           ch = get_byte();
     } while(ch != '1' && ch != '0');
           
     if(ch == '1') {
              tmpb.tmintmask = 1;
              Print("\r$Timer interrupt disabled.\r");
     }
     else {
              tmpb.tmintmask = 0;
              Print("\r$Timer interrupt enabled.\r");
     }

     /* ************************************** */
     /* Timer interrupt masking control        */
     /* ************************************** */
     do {
           Print("\r$ Timer interrupt mode?[0:IRQ/1:FIQ]_");      
           ch = get_byte();
     } while(ch != '1' && ch != '0');
           
     if(ch == '1') {
              tmpb.tmintmode = 1;
              Print("\r$Timer interrupt mode is FIQ.\r");
     }
     else {  
              tmpb.tmintmode = 0;
              Print("\r$Timer interrupt mode is IRQ.\r");
     }


     tmCntr(device, &tmpb);  // Timer control function 
   
     if(device == '1') 
           Timer1Start();
     else
           Timer0Start();
}


/*  =============================================
 *  Monitoring the timer related system register
 *  =============================================
 */
void tmConfig(void)
{


    Print("\r<<---------- TIMER 0 STATUS ----------->>\r");

    if(TMOD & TM0_RUN)
         Print(">>TMOD[0]: Timer0,Start enabled.\r");
    else
         Print(">>TMOD[0]: Timer0,Disabled.\r");

    if(TMOD & TM0_TOGGLE)
         Print(">>TMOD[1]: Timer0,Toggle mode.\r");
    else
         Print(">>TMOD[1]: Timer0,Interval mode.\r");

    if(TMOD & TM0_OUT_1)
         Print(">>TMOD[2]: Timer0,Initial TOUT0 is 1.\r");
    else
         Print(">>TMOD[2]: Timer0,Initial TOUT0 is 0.\r");

    Print(">>TDATA0 = 0x%08x\r",TDATA0);
    Print(">>TCNT0= 0x%08x \r",TCNT0);

    if((IOPCON & 0x40000000) && (IOPMOD & 0x10000))
         Print(">>Timer0,TOUT0 will be monitored at PORT16.\r");
    else
         Print(">>Timer0,TOUT0 output disabled.\r");

    if(IntMode & TIMER0_INT)
          Print(">>Timer0 is in FIQ mode.\r");
    else
          Print(">>Timer0 is in IRQ mode.\r");

    if(~IntMask & TIMER0_INT)
          Print(">>Timer0 interrupt enabled.\r");
    else
          Print(">>Timer0 interrupt disabled.\r");


    Print("-----------------------------------------\r");
    Print("\r<<---------- TIMER 1 STATUS ----------->>\r");


    if(TMOD & TM1_RUN)
         Print(">>TMOD[1]: Timer1,Start enabled.\r");
    else
         Print(">>TMOD[1]: Timer1,Disabled.\r");

    if(TMOD & TM1_TOGGLE)
         Print(">>TMOD[1]: Timer1,Toggle mode.\r");
    else
         Print(">>TMOD[1] Timer1,Interval mode.\r");

    if(TMOD & TM1_OUT_1)
         Print(">>TMOD[2]: Timer1,Initial TOUT1 is 1.\r");
    else
         Print(">>TMOD[2]: Timer1,Initial TOUT1 is 0.\r");

    Print(">>TDATA1 = 0x%08x\r",TDATA1);
    Print(">>TCNT1= 0x%08x\r",TCNT1);

    if((IOPCON & 0x80000000) && (IOPMOD & 0x20000))
         Print(">>Timer1,TOUT1 will be monitored at PORT17.\r");
    else
         Print(">>Timer1,TOUT1 output disabled.\r");

    if(IntMode & TIMER1_INT)
          Print(">>Timer1 is in FIQ mode.\r");
    else
          Print(">>Timer1 is in IRQ mode.\r");

    if(~IntMask & TIMER1_INT)
          Print(">>Timer1 interrupt enabled.\r");
    else
          Print(">>Timer1 interrupt disabled.\r");
    Print("-----------------------------------------\r\r");
}

