/********************************************************************/
/*                                                                  */
/*      UART Test Program for KS32C5000                             */
/*                                                                  */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*                                                                  */
/*      Programmed by YOUNG-SUN KIM 			            */
/*                                                                  */
/*      Description : 1998-1-25 first edited for KS32C5000          */
/********************************************************************/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "apdialog.h"
#include "ks5000.h"
#include "memory.h"
#include "uart.h"
#include "dma.h"
#include "pic.h"
#include "pollio.h"
#include "lcd.h"
#include "sysconf.h"

/********************************************************************/
/*               INITIALIZE UART GLOBAL VARIABLE                    */
/********************************************************************/

//define uart imitation register, Default value setting
UART_REG UART = {  
     ULCON_WL8,  //8bit word length, one stop bit,no parity
     (UCON_RXM_INTREQ|UCON_TXM_INTREQ|UCON_RXSTAT_INT),
     (fMCLK/(16* BAUDRATE)-1), //Boud rate register
     0           //Flag to control dialog or direct function test
};


QUE RxQue0,TxQue0; //UART tx,rx que for interrupt method, & Initialize 
QUE RxQue1,TxQue1; //UART tx,rx que for interrupt method, & Initialize 

/********************************************************************/
/* T1            UART TEST TOP MODULE                               */
/********************************************************************/

void UartTest(void)
{

     void PrintUartItems(void);
     char items;

     IOPDATA= ~0xc;  //in UART test mode
     TxQueInit(); // Tx Que0,1 initialize
     RxQueInit(); // Rx Que0,1 initialize
     Enable_Int(nGLOBAL_INT);  //Global interrupt enable

     /* Default all channel is polling mode */
     UARTPollInit(SERIAL_DEV0); 
     UARTPollInit(SERIAL_DEV1); 

     do {
           PrintUartItems(); // Main memu for cache test

           do { 
                 Print("\rSelect Number?_");
                 items = get_byte();
           }while(is_space(items));

           if(is_xdigit(to_upper(items))) {

               switch(to_upper(items)) {
                      case '1': UARTStartUpDialog();    break;
                      case '2': UARTConfigView();  break;
                      case '3': UARTAutoLoopBack();  break;
                      case '4': UARTExtLoopBack();  break;
                      case '5': SetBaudRate();  break;
                      case '6': BaudRateTable();  break;
                      default : break;
              }
           }
           Print("\rPress any key to continue.\r");
           while(!is_space(get_byte()));

     } while((items!='q')&&(items!='Q'));

}

/********************************************************************/
/* T2            UART TEST MAIN MODULE                              */
/********************************************************************/

void SetBaudRate(void)
{
    char ch;
    unsigned int i;

        /* Baud rate Change*/
        do {
             Print("\rChange  Baud Rate ?[Y/N]");
             ch = get_byte();
             ch = to_upper(ch);
        }while((ch!='Y')&&(ch!='N'));

        if(ch == 'Y') {
            do {
                 Print("\rSet direct Baud Rate Register value?[Y/N] ");
                 ch = get_byte();
                 ch = to_upper(ch);
            }while((ch!='Y')&&(ch!='N'));

            if(ch == 'Y') {
                Print("\r\rEnter Baudrate Register value. > 0x");
                i = gethex2dec(4);
                Print("\rBaud rate will be %dbps.\r",BaudRate(i));
                if(getchannel()) 
                     UARTBRD1 = i;
                else UARTBRD0 = i;

            }
            else {
                   do {
                        Print("\r\rBaud Rate:1200,2400,4800,9600,19200,38400,57600,115200"); 
                        Print("\r\r>Enter Baud Rate?[default: 19200bps] ");
                        i = get_number();
                   }while ((i<110)&&(i>921600));
                   
                   Print("\rBaud rate register value will be 0x%04x\r",BaudRateVal(i));
                   if(getchannel()) 
                        UARTBRD1 = BaudRateVal(i);
                   else UARTBRD0 = BaudRateVal(i);
            }
        }  // case of not baudrate change
}

/* 
 *  Receive data using interrupt method.
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  This function receive data from receive user defined QUE and write
 *  receive data to receive buffer memory. 
 *  At the end of receive, The function return receive data byte size.
 */
void UARTIntRead(int channel, void *RxbuffAddr, int rxsize)
{
    char ch;
    U8    *rxbuf;          //Receive buffer memory address 

    rxbuf = (U8 *)RxbuffAddr;   //Copy receive buffer memory address
    UARTRxIntOn(channel); //Rx interrupt ON for interrupt mode

    while(rxsize--) {
          if((ch=RxQueRead(SERIAL_DEV1))!='\0')  *rxbuf++ = ch;
          else  rxsize = 0;  //for escape loop
    }

    UARTRxIntOff(channel);   //Rx interrupt OFF for return to polling mode.
}

/*
 *   Transmit data using interrupt method.
 *   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *   This function transmit data write to transmit circular que
 *   Transmit data in circular que will be transmitted by interrupt
 *   service routine.
 *
 *     Revised 1998.5.13
 */
void UARTIntWrite(int channel, void *TxbufAddr, int txsize)
{
    U8 *txbuf;           //Transmit buffer memory address

    txbuf = (U8 *)TxbufAddr;     //Copy transmit buffer address

    UARTTxIntOn(channel);  //Tx interrupt on for interrupt mode

    // Write txdata to Transmit Que
    while(txsize--) TxQueWrite(*txbuf++, channel); 

    UARTTxIntOff(channel); //Tx interrupt off for return to polling mode
}



/*
 *  UART AUTO INTERNAL LOOPBACK TEST
 */
int UARTAutoLoopBack(void)
{
	U32 ascii;
	int LoopBackFlag0=0, LoopBackFlag1=0 ; 
	char ch;

    	/* Set Loopback test mode */
	UARTCONT0 |= UCON_LOOPBACK;
	UARTRxIntOff(SERIAL_DEV0); UARTTxIntOff(SERIAL_DEV0);
	get_byte() ; get_byte() ;
    	for(ascii = 48; ascii < 123 ; ascii++) {
        	put_char((char)ascii, SERIAL_DEV0);
        	ch = get_char(SERIAL_DEV0);
        	if(ch != (char)ascii) LoopBackFlag0 = 1;
	}

        UARTCONT0 &= ~UCON_LOOPBACK; //clear loopback for print
	if (LoopBackFlag0) Print("\r $$$ UART Channel 0 FAIL !!!") ;
	else Print("\r $$$ UART Channel 0 PASS !!!") ;

    	/* Set Loopback test mode */
	UARTCONT1 |= UCON_LOOPBACK;
	UARTRxIntOff(SERIAL_DEV1); UARTTxIntOff(SERIAL_DEV1);
    	for(ascii = 48; ascii < 123 ; ascii++) {
        	put_char((char)ascii, SERIAL_DEV1);
        	ch = get_char(SERIAL_DEV1);
        	if(ch != (char)ascii) LoopBackFlag1 = 1;
    	}
	
        UARTCONT1 &= ~UCON_LOOPBACK; //clear loopback for print
	if (LoopBackFlag1) Print("\r $$$ UART Channel 1 FAIL !!!") ;
	else Print("\r $$$ UART Channel 1 PASS !!!") ;

	if (!(LoopBackFlag0 || LoopBackFlag1)) return 1 ;
	else return 0 ;

}


/*
 *  UART external loopback function(User Interrupt mode)
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  External plag(Null modem) for loopback test must be installed.
 *  This driver perform data transfer with null modem. hardware 
 *  controll signal(DTR,DSR) also will be used for external loopback
 *  test for assigned channel.
 *  Case of external loopback test fail, return 0. Case of Pass, return 1.
 */
void UARTExtLoopBack(void)
{
    char ch;

    //For Hardware flow control, DSR and DTR can be used.
    // DTE: DTR  <-------> DSR :DCE 

    do {
         Print("\nExternal loopback test mode for channel 1.\n");
         Print("External loopback plug is installed?[Y/N]");
         ch = get_byte();  ch = to_upper(ch);
    }while(ch != 'Y');

    if(!ExtLoopBacksub(SERIAL_DEV1)) 
          Print("\nUART1 External Loopback test FAIL!!!\n"); 
    else  Print("\nUART1 External Loopback test PASS!!!\n");  
}


/*
 * UART External loopback main sub module.
 * Data transmit & receive use by interrupt mode.
 * Transmit data will be written to transmit QUE. This QUE will be
 * accessed by interrupt service.
 * Receive interrupt occured whenever the receive buffer register
 * have data. Receive QUE also will be written by receive interrupt
 * and accessed by user program to read receive data.
 * External Tx/Rx, DTR/DSR will be connected for external loopback test.
 */
unsigned int ExtLoopBacksub(int channel)
{
    U32 *txbuf;
    U32 *rxbuf;

    txbuf = (U32 *)UartTxbuff;
    rxbuf = (U32 *)UartRxbuff;

    UARTIntInit(channel); //Change to interrupt mode 

    /*Initialize source memory with random pattern */
    MemTestInit(txbuf,(int)UartTestSize); 

    /* Transmit data use by interrupt mode */
    UARTIntWrite(channel,txbuf,(int)UartTestSize);

    /* Receive data use by interrupt mode */
    UARTIntRead(channel,rxbuf,(int)UartTestSize);

    UARTPollInit(channel);  //Return to polling mode 

    if (!bcomp((U32 *)UartTxbuff,(U32 *)UartRxbuff,UartTestSize)) 
         return 0; //External loopback test FAIL 
    else  return 1; //External loopback test PASS

}
/********************************************************************/
/* T2            UART INITIALIZE FUNCTIONS                          */
/********************************************************************/


/*
 * UART DEFAULT INITIALIZE
 */
void UARTPollInit(unsigned int channel) 
{
    //U32 rUARTBRD;

    UARTRxIntOff(channel);
    UARTTxIntOff(channel);

    if(channel) {
        UARTLCON1 = ULCON_WL8; 
        UARTCONT1 = (UCON_RXM_INTREQ|UCON_TXM_INTREQ);

        //rUARTBRD = UARTBRD1;
        //if(!rUARTBRD)  UARTBRD1=(fMCLK/(16* BAUDRATE)-1); 
        UARTBRD1 = (fMCLK/(16* BAUDRATE)-1); 
    }
    else {
          UARTLCON0 = ULCON_WL8; 
          UARTCONT0 = (UCON_RXM_INTREQ|UCON_TXM_INTREQ);
          //rUARTBRD = UARTBRD0;
          //if(!rUARTBRD)  UARTBRD0=(fMCLK/(16* BAUDRATE)-1); 
          UARTBRD0 = (fMCLK/(16* BAUDRATE)-1); 
    }
}

void UARTIntInit(unsigned int channel) 
{
    U32 rUARTBRD;

    if(channel) {
          /* Interrupt service routine setup */
        SysSetInterrupt(nUART1_TX_INT, Uart1TxIsr);
        SysSetInterrupt(nUART1_RX_INT, Uart1RxIsr);
        SysSetInterrupt(nUART1_ERROR_INT, Uart1ErrIsr);

          /* UART mode, default baud rate setup */
        UARTLCON1 = ULCON_WL8; 
        UARTCONT1 = (UCON_RXM_INTREQ|UCON_TXM_INTREQ|UCON_RXSTAT_INT);

        //Board rate register.BRD
        rUARTBRD = UARTBRD1;
        if(!rUARTBRD)  UARTBRD1=(fMCLK/(16* BAUDRATE)-1); 

        //Initialize Transmit register, 
        //Initialize character will be not transfered.
        UARTTXH1 = '0';
    }
    else {
          /* Interrupt service routine setup */
          SysSetInterrupt(nUART0_TX_INT, Uart0TxIsr);
          SysSetInterrupt(nUART0_RX_INT, Uart0RxIsr);
          SysSetInterrupt(nUART0_ERROR_INT, Uart0ErrIsr);
   
          /* UART mode, default baud rate setup */
          UARTLCON0 = ULCON_WL8; 
          UARTCONT0 = (UCON_RXM_INTREQ|UCON_TXM_INTREQ|UCON_RXSTAT_INT);

          //Board rate register.BRD
          rUARTBRD = UARTBRD0;
          if(!rUARTBRD)  UARTBRD0=(fMCLK/(16* BAUDRATE)-1); 

          //Initialize Transmit register, 
          //Initialize character will be not transfered.
          UARTTXH0 = '0';
    }

    UARTTxIntOn(channel);
    UARTRxIntOn(channel);
}


void UARTRegWrite(int channel)
{
    if(channel) {
          UARTLCON1 = UART.LCON; 
          UARTCONT1 = UART.CON;
          UARTBRD1 = UART.BRD;
    }
    else {
            UARTLCON0 = UART.LCON; 
            UARTCONT0 = UART.CON;
            UARTBRD0 = UART.BRD;
    }
}
/********************************************************************/
/* T3            UART INTERRUPT FUNCTIONS                           */
/********************************************************************/
void Uart0TxIsr(void)
{
    Clear_PendingBit(nUART0_TX_INT);
    UARTTXH0=TxQueRead(SERIAL_DEV0);
}

void Uart0RxIsr(void)
{ 
     Clear_PendingBit(nUART0_RX_INT);
     RxQueWrite(UARTRXB0,SERIAL_DEV0);
}

void Uart0ErrIsr(void)
{
     Clear_PendingBit(nUART0_ERROR_INT);

     if(UARTSTAT0&USTAT_OVERRUN) {
        Print("\r[ERROR] OVER RUN ERROR OCCURRED!\r");  
     }

     if(UARTSTAT0&USTAT_PARITY) {
        Print("\r[ERROR] PARITY ERROR OCCURRED!\r");  
        UARTCONT0 |= UCON_RXM_OFF; //UART RCV Disabled 
     }

     if(UARTSTAT0&USTAT_FRAME) {
        Print("\r[ERROR] FRAME ERROR OCCURRED!\r");  
        UARTCONT0 |= UCON_RXM_OFF; //UART RCV Disabled 
     }

     if(UARTSTAT0&USTAT_BREAK) {
        Print("\r[ERROR] BREAK SIGNAL HAS BEEN RECEIVED!\r");  
        UARTCONT0 |= UCON_RXM_OFF; //UART RCV Disabled 
     }
}


void Uart1TxIsr(void)
{
    Clear_PendingBit(nUART1_TX_INT);
    UARTTXH1=TxQueRead(SERIAL_DEV1);
}


void Uart1RxIsr(void)
{
     Clear_PendingBit(nUART1_RX_INT);
     RxQueWrite(UARTRXB1,SERIAL_DEV1);
}


void Uart1ErrIsr(void)
{
     Clear_PendingBit(nUART1_ERROR_INT);

     if(UARTSTAT1&USTAT_OVERRUN) {
        Print("\r[ERROR] OVER RUN ERROR OCCURRED!\r");  
     }

     if(UARTSTAT1&USTAT_PARITY) {
        Print("\r[ERROR] PARITY ERROR OCCURRED!\r");  
        UARTCONT1 |= UCON_RXM_OFF; //UART RCV Disabled 
     }

     if(UARTSTAT1&USTAT_FRAME) {
        Print("\r[ERROR] FRAME ERROR OCCURRED!\r");  
        UARTCONT1 |= UCON_RXM_OFF; //UART RCV Disabled 
     }

     if(UARTSTAT1&USTAT_BREAK) {
        Print("\r[ERROR] BREAK SIGNAL HAS BEEN RECEIVED!\r");  
        UARTCONT1 |= UCON_RXM_OFF; //UART RCV Disabled 
     }
}


void UARTTxIntOn(unsigned int channel)
{
     if(channel) {
	/* Enable Interrupt */
	Enable_Int(nUART1_TX_INT);
     }
     else {
	/* Enable Interrupt */
	Enable_Int(nUART0_TX_INT);
     }
}

void UARTRxIntOn(unsigned int channel)
{
     if(channel) {
	/* Enable Interrupt */
	Enable_Int(nUART1_RX_INT);
	Enable_Int(nUART1_ERROR_INT);
     }
     else {
	/* Enable Interrupt */
	Enable_Int(nUART0_RX_INT);
	Enable_Int(nUART0_ERROR_INT);
     }
}


void UARTTxIntOff(unsigned int channel)
{
     if(channel) {
	/* Enable Interrupt */
	Disable_Int(nUART1_TX_INT);
     }
     else {
	/* Enable Interrupt */
	Disable_Int(nUART0_TX_INT);
     }
}

void UARTRxIntOff(unsigned int channel)
{
     if(channel) {
	/* Enable Interrupt */
	Disable_Int(nUART1_RX_INT);
	Disable_Int(nUART1_ERROR_INT);
     }
     else {
           /* Enable Interrupt */
	   Disable_Int(nUART0_RX_INT);
	   Disable_Int(nUART0_ERROR_INT);
     }
}
/********************************************************************/
/* T4            UART FUNCTION FOR INTERRUPT MODE                   */
/********************************************************************/

/* QUE, RX, TX QUE CONTROL FUNCTIONS */
/* WILL BE USED IN INTERRUPT MODE */

/* Transmit Que write function */
void TxQueWrite(char data,unsigned int channel)
{
    if(channel) {
        if(TxQue1.wptr+1 == TxQue1.rptr) 
            return; //ring buffer full state

        TxQue1.buff[TxQue1.wptr] = data;
        TxQue1.wptr++;

        if(TxQue1.wptr==MAXEVENT) TxQue1.wptr=0;
    }
    else {
           if(TxQue0.wptr+1 == TxQue0.rptr) 
               return; //ring buffer full state

           TxQue0.buff[TxQue0.wptr] = data;
           TxQue0.wptr++;

           if(TxQue0.wptr==MAXEVENT) TxQue0.wptr=0;
    }
}


/* Transmit Que read function */
char TxQueRead(unsigned int channel)
{
     if(channel) {
         if(TxQue1.rptr==MAXEVENT) TxQue1.rptr=0; /*loop back*/
         if(TxQue1.rptr==TxQue1.wptr) 
            return('\0'); //ring buffer is empty state
         TxQue1.rptr++;
         return(TxQue1.buff[TxQue1.rptr-1]);
     }
     else {
            if(TxQue0.rptr==MAXEVENT) TxQue0.rptr=0; /*loop back*/
            if(TxQue0.rptr==TxQue0.wptr) 
               return('\0'); //ring buffer is empty state
            TxQue0.rptr++;
            return(TxQue0.buff[TxQue0.rptr-1]);
     }
}



/* Receive Que write function */
void RxQueWrite(char data,unsigned int channel)
{
    if(channel) {
        if(RxQue1.wptr+1 == RxQue1.rptr) 
            return; //ring buffer full state

        RxQue1.buff[RxQue1.wptr] = data;
        RxQue1.wptr++;

        if(RxQue1.wptr==MAXEVENT) RxQue1.wptr=0;
    }
    else {
           if(RxQue0.wptr+1 == RxQue0.rptr) 
               return; //ring buffer full state

           RxQue0.buff[RxQue0.wptr] = data;
           RxQue0.wptr++;

           if(RxQue0.wptr==MAXEVENT) RxQue0.wptr=0;
    }
}


/* Receive Que read function */
char RxQueRead(unsigned int channel)
{
     if(channel) {
         if(RxQue1.rptr==MAXEVENT) RxQue1.rptr=0; /*loop back*/
         if(RxQue1.rptr==RxQue1.wptr) 
            return('\0');
         RxQue1.rptr++;
         return(RxQue1.buff[RxQue1.rptr-1]);
     }
     else {
            if(RxQue0.rptr==MAXEVENT) RxQue0.rptr=0; /*loop back*/
            if(RxQue0.rptr==RxQue0.wptr) 
               return('\0');
            RxQue0.rptr++;
            return(RxQue0.buff[RxQue0.rptr-1]);
     }
}


/* Transmit Que initialize */
void TxQueInit(void)
{
    int i;

    for(i=0; i<MAXEVENT; i++) {
          TxQue0.buff[i] = '\0';
          TxQue1.buff[i] = '\0';
    }

    TxQue0.wptr = 0;
    TxQue0.rptr = MAXEVENT;
    TxQue1.wptr = 0;
    TxQue1.rptr = MAXEVENT;
}

/* Receive Que initialize */
void RxQueInit(void)
{
    int i;

    for(i=0; i<MAXEVENT; i++) {
          RxQue0.buff[i] = '\0';
          RxQue1.buff[i] = '\0';
    }

    RxQue0.wptr = 0;
    RxQue0.rptr = 0;
    RxQue1.wptr = 0;
    RxQue1.rptr = 0;
}


/* See what's in the que */
void TxQueView(void)
{
     int i;
     
     /* Transmit Que for UART0 */
     Print("\rTransmit Que0 contents\r\r.");
     for(i=TxQue0.rptr; i < TxQue0.wptr;i++)
         Print("%c",TxQue0.buff[i]);

     /* Transmit Que for UART1 */
     Print("\r\rTransmit Que1 contents\r\r.");
     for(i=TxQue1.rptr; i < TxQue1.wptr;i++)
         Print("%c",TxQue1.buff[i]);
}

void RxQueView(void)
{
     int i;

     /* Receive Que for UART0 */
     Print("\r\rReceive Que0 contents.\r\r");
     for(i=RxQue0.rptr; i < RxQue0.wptr;i++)
         Print("%c",RxQue0.buff[i]);

     /* Receive Que for UART1 */
     Print("\r\rReceive Que1 contents.\r\r");
     for(i=RxQue1.rptr; i < RxQue1.wptr;i++)
         Print("%c",RxQue1.buff[i]);
}

/********************************************************************/
/* T5            UART FUNCTION FOR POLLING MODE                     */
/********************************************************************/

/* RECEIVE & TRANSMIT FUNCTIONS */
/* WILL BE USED IN POLLING MODE */
//void put_byte(char ch,int channel) 
void put_char(char ch,unsigned int channel) 
{
    if(channel) {
        WaitXmitter(UARTSTAT1);
        UARTTXH1 = ch;
    }
    else {
        WaitXmitter(UARTSTAT0);
        UARTTXH0 = ch;
    }
}

//char get_byte(int channel) 
char get_char(unsigned int channel) 
{ 
  char ch;
 
    if(channel) { WaitRcver(UARTSTAT1);
                  ch = UARTRXB1;
    }
    else { WaitRcver(UARTSTAT0);
           ch = UARTRXB0;
    }
    return ch;
}

void put_string(char *ptr ) 
{
    while(*ptr )  put_byte(*ptr++ );
}

//get_string is in pollio.h

/*
 *   Get hexa string to decimal value. 
 *   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *   As long as the character in *hex is a valid hexadecimal
 *   value
 *
 *   arqument value is the number of hexadecimal digit.
 */
int gethex2dec(unsigned int digitcnt)
{
    int decimal = 0;
    char hex;
    int  i;

    put_string("0x");

    hex = get_byte();

    for(i=0;(i<digitcnt)&&(!is_control(hex));i++) {
         hex = to_upper(hex);

         if(is_xdigit(hex)) {

              if(is_digit(hex))  /* '0' - '9' */
                   decimal = decimal * 16 + hex - '0';
              else               /* 'A' - 'F' */
                   decimal = decimal * 16 + 10 + hex - 'A';
         }
         else {
                Print("\r[ERROR] Re-enter %d hexdigit\r",digitcnt);
                i = 0; // clear character counter
                decimal = 0;
                put_string("0x");
         }

         hex = get_byte(); //get next character
    }

    put_byte('\r');
    return(decimal);
}


U32 get_num(void)
{
	char RcvData[10] ;
	unsigned RcvNum=0 ;
	int i = 0 ;

	while ( (RcvData[i] = get_byte() ) != CR )
	{
		if ( (RcvData[i] >= '0') && (RcvData[i] <= '9') )
			RcvNum=(RcvNum<<4) + (RcvData[i]-'0') ;
		else if ( (RcvData[i] >= 'A') && (RcvData[i] <= 'F') )
			RcvNum=(RcvNum<<4) + (RcvData[i]-'A'+ 10) ;
		else if ( (RcvData[i] >= 'a') && (RcvData[i] <= 'f') )
			RcvNum=(RcvNum<<4) + (RcvData[i]-'a'+ 10) ;
		i++ ;
	}

	return RcvNum ;
}


/*------------------------------------------------------------------------
 * Function : get_digit
 * Description : get decimal number
 *------------------------------------------------------------------------*/
int get_digit(void)
{
	char RcvData[10] ;
	int RcvNum=0 ;
	int RcvDataSize=0 ;
	int i ;

	while ((RcvData[RcvDataSize] = get_byte()) != CR) RcvDataSize++ ;

	for (i=0;i < RcvDataSize ; i++)
		RcvNum = RcvNum*10 + (RcvData[i]-'0') ; 

	return RcvNum ;
}

// get_key : get character from uart or Key-pad
char get_key(void) 
{
    	char ch ;
	U32 KeyInValue ;

	// Key in value : 0
	while(1) {

	KeyInValue = (IOPDATA>>5) & 0x7 ;

	// For Key Pad Input
	if (KeyInValue != 7 )
	{
		switch(KeyInValue)
		{
			case 0x6 : put_byte(ch = 'F') ; break ; // P5
			case 0x5 : put_byte(ch = 'P') ; break ; // P6
			case 0x3 : put_byte(ch = 'O') ; break ; // P7
			default : break ;
		}
		break ;
	}

	// For UART Input
    	#ifdef PORT1
	else if (UARTSTAT1 & USTAT_RCV_READY) 
		{
		ch = UARTRXB1; break ;
		}
    	#else
	else if (UARTSTAT0 & USTAT_RCV_READY) 
		{
		ch = UARTRXB0; break ;
		}
    	#endif

	} // End of while

	lPut_byte(ch) ;
 
    	return ch ;
}

// kbd_hit
int kbd_hit(void) 
{
    int KbdHit ;
    char ch ;

    #ifdef PORT1
	if (UARTSTAT1 & USTAT_RCV_READY) 
		{
		KbdHit = 1 ;
        	ch = UARTRXB1;
		}
	else KbdHit = 0 ;
    #else
	if (UARTSTAT0 & USTAT_RCV_READY) 
		{
		KbdHit = 1 ;
        	ch = UARTRXB0;
		}
	else KbdHit = 0 ;
    #endif

    return KbdHit ;
}


/********************************************************************/
/* T6            UART START UP DIALOG FUNCTION                      */
/********************************************************************/
void UARTStartUpDialog(void)
{
    char c;
    int i;

    UART.DIALOG = 1;   // set flag
    
     Print("\r\r>>> STARTUP DIALOG FOR UART CONFIGURATION  <<<\r\r"); 

     do {
           Print(" >  WORD LENGTH PER FRAME  <\r");
           Print(" --------------------------\r");
           Print(" 0. 5 bits.\r");
           Print(" 1. 6 bits.\r");
           Print(" 2. 7 bits.\r");
           Print(" 3. 8 bits.\r");
           Print(" --------------------------\r");
           Print("$ Select Number.> ");
           i = get_number();
           get_byte(); //dummy for carrige return
     } while(i > 3);

     switch(i) {
        case 1:  UART.LCON = ULCON_WL6; break;
        case 2:  UART.LCON = ULCON_WL7; break;
        case 3:  UART.LCON = ULCON_WL8; break;
        default:  UART.LCON = ULCON_WL5;
     }

    
     /* NUMBER OF STOP BIT END OF FRAME */
     do {
           Print("\r$ No of stop bit?[0:1bit/1:2bit]");      
           c = get_byte();
     } while(c!='1' && c!='0');
           
     if(c == '1') UART.LCON |= ULCON_STOP_2;

     /* PARITY MODE */
     do {
           Print("\r\r>  PARITY MODE   <\r");
           Print(" --------------------\r");
           Print(" 0. No Parity.\r");
           Print(" 1. Odd Parity.\r");
           Print(" 2. Even Parity.\r");
           Print(" 3. Parity forced/checked as 1.\r");
           Print(" 4. Parity forced/checked as 0.\r");
           Print(" --------------------\r");
           Print("$ Select Number.> ");
           i = get_number();
           get_byte(); //dummy for carrige return
     } while(i > 4);

     switch(i) {
        case 1:  UART.LCON |= ULCON_PMD_ODD; break;
        case 2:  UART.LCON |= ULCON_PMD_EVEN; break;
        case 3:  UART.LCON |= ULCON_PMD_CHK1; break;
        case 4:  UART.LCON |= ULCON_PMD_CHK0; break;
        default: UART.LCON |= ULCON_PMD_NO; 
     }

     /* INFRA-RED MODE */
     do {
           Print("\r$ Infra-Red Mode?[1/0]");      
           c = get_byte();
     } while(c!='1' && c!='0');
           
     if(c == '1') UART.LCON |= ULCON_INFRA_RED;
    
     /* RECEIVE MODE */
     do {
           Print("\r>  RECEIVE MODE   <\r");
           Print(" --------------------\r");
           Print(" 0. Disable.\r");
           Print(" 1. Interrupt Request.\r");
           Print(" 2. GDMA Channel0 Request.\r");
           Print(" 3. GDMA Channel1 Request.\r");
           Print(" --------------------\r");
           Print("$ Select Number.> ");
           i = get_number();
           get_byte(); //dummy for carrige return
     } while(i > 3);

     switch(i) {
        case 1:  UART.CON |= UCON_RXM_INTREQ; break;
        case 2:  UART.CON |= UCON_RXM_GDMA0REQ; break;
        case 3:  UART.CON |= UCON_RXM_GDMA1REQ; break;
        default: UART.CON |= UCON_RXM_OFF; 
     }

     /* RX STATUS INTERRUPT ENABLE */
     do {
           Print("\r$ Generate Receive Status Interrupt?[1/0]");      
           c = get_byte();
     } while(c!='1' && c!='0');
           
     if(c == '1') UART.CON |= UCON_RXSTAT_INT;

     /* TRANSMIT MODE */ 
     do { 
           Print("\r>  TRANSMIT MODE   <\r"); 
           Print(" --------------------\r"); 
           Print(" 0. Disable.\r"); 
           Print(" 1. Interrupt Request.\r");
           Print(" 2. GDMA Channel0 Request.\r");
           Print(" 3. GDMA Channel1 Request.\r");
           Print(" --------------------\r");
           Print("$ Select Number.> ");
           i = get_number();
           get_byte(); //dummy for carrige return
     } while(i > 3);

     switch(i) {
        case 1:  UART.CON |= UCON_TXM_INTREQ; break;
        case 2:  UART.CON |= UCON_TXM_GDMA0REQ; break;
        case 3:  UART.CON |= UCON_TXM_GDMA1REQ; break;
        default: UART.CON |= UCON_TXM_OFF; 
     }

     /* LOOPBACK ENABLE */
     do {
           Print("\r$ Enable Loopback enable?[1/0]");      
           c = get_byte();
     } while(c!='1' && c!='0');
           
     if(c == '1') UART.CON |= UCON_LOOPBACK;

     if(getchannel()) 
         UARTRegWrite(SERIAL_DEV1);
     else 
         UARTRegWrite(SERIAL_DEV0);
}


unsigned getchannel(void)
{
  char ch;

     do { 
           Print("\rSelect UART Channel[0/1]?_");
           ch = get_byte();
     }while((ch!= '1') && (ch != '0'));
    
     if(ch == '1') return(1);
     else  return(0);
}

/********************************************************************/
/*  T7           UART DISPLAY FUNCTIONS                             */
/********************************************************************/

void UARTConfigView(void)
{
   char channel;
 
   if(getchannel()) channel = '1';
   else  channel = '0';

   if(channel=='1') {
        Print("\r------------------------------");
        Print("\r <<< UART1 REGISTER STATUS >>>");
        Print("\r------------------------------");
        Print("\rUARTLCON1[0x%08x] = 0x%08x",&UARTLCON1,UARTLCON1);
        Print("\rUARTCONT1[0x%08x] = 0x%08x",&UARTCONT1,UARTCONT1);
        Print("\rUARTSTAT1[0x%08x] = 0x%08x",&UARTSTAT1,UARTSTAT1);
        Print("\rUART1 BAUDRATE = %d[bps]",BaudRate(UARTBRD1));
   }
   else {
           Print("\r------------------------------");
           Print("\r <<< UART0 REGISTER STATUS >>>");
           Print("\r------------------------------");
           Print("\rUARTLCON0[0x%08x] = 0x%08x",&UARTLCON0,UARTLCON0);
           Print("\rUARTCONT0[0x%08x] = 0x%08x",&UARTCONT0,UARTCONT0);
           Print("\rUARTSTAT0[0x%08x] = 0x%08x",&UARTSTAT0,UARTSTAT0);
           Print("\rUART0 BAUDRATE = %d[bps]",BaudRate(UARTBRD0));
   }
}



void dbg_out(char *fmt, ...)
{
    va_list argptr;
    char temp_buf[256];

    va_start(argptr, fmt);
    vsprintf(temp_buf, fmt, argptr);
    put_string(temp_buf);
    va_end(argptr);
}


//Print Test Status
void PrintStatus(int Ldisp, char Udisp)
{
        put_byte(Udisp);
        put_byte('\b') ;
        IOPDATA = Ldisp ;
}


/* 
 * Display GDMA test items 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
void PrintUartItems(void)
{
        Print("\n\n");
	Print("+---------------------------------------------------+\r") ;
        Print("|            KS32C5000 UART TEST ITEMS              |\r") ;
	Print("+---------------------------------------------------+\r") ;
	Print("| 1. UART start up dialog for setup DMA mode.       |\r") ;
	Print("| 2. UART configurations viewer.                    |\r") ;
	Print("| 3. Internal Loopback test.(Polling mode)          |\r") ;
	Print("| 4. External Loopback test.(Interrupt mode)        |\r") ;
	Print("| 5. Change Baud Rate.                              |\r") ;
	Print("| 6. Baud rate Table.                               |\r") ;
	Print("| Q. QUIT - Return to main menu.                    |\r") ;
	Print("+---------------------------------------------------+\r") ;
}

/********************************************************************/
/*  T8             UART MACRO FUNCTIONS                             */
/********************************************************************/
void SendBreak(char channel,int mode)
{
     if(channel=='1') {
         if(mode) UARTCONT1 |= UCON_SEND_BREAK;
         else     UARTCONT1 &= ~UCON_SEND_BREAK;
     }
     else {
            if(mode) UARTCONT0 |= UCON_SEND_BREAK;
            else     UARTCONT0 &= ~UCON_SEND_BREAK;
     }
}


void SetDSR(char channel,int mode)
{
     if(channel=='1') {
         if(mode) UARTCONT1 |= UCON_DSR;
         else     UARTCONT1 &= ~UCON_DSR;
     }
     else {
            if(mode) UARTCONT0 |= UCON_DSR;
            else     UARTCONT0 &= ~UCON_DSR;
     }
}

void SetDTR(char channel, int mode)
{
     if(channel=='1') {
         if(mode) UARTSTAT1 |= USTAT_DTR_LOW;
         else     UARTSTAT1 &= ~USTAT_DTR_LOW;
     }
     else {
            if(mode) UARTSTAT1 |= USTAT_DTR_LOW;
            else     UARTSTAT1 &= ~USTAT_DTR_LOW;
     }
}


unsigned BaudRateVal(unsigned br)
{
     return((fMCLK/(16* br)-1)); //Retun baud rate register value
}


unsigned BaudRate(unsigned div)
{
     return(fMCLK/((div+1)*16)); //return Board rate of argument(register)
}

void BaudRateTable(void)
{
     int baud[10] = {1200,2400,4800,9600,19200,38400,57600,115200,230400,460860}; 
     int StartBaudVal = 0x06b5;  //from 1200bps
     int CalBaud;
     int CalBaudValue;
     int i;
  
     Print("\r\r     << BAUD RATE TABLE >>     \r\r");
     Print("========+==========+=======+==================\r");
     Print("  SPEC  -   TYP.   =  DIF.    UBRDIVn\r");
     Print("========+==========+=======+==================\r");

     for(i=0; i<10; i++) {
          CalBaudValue = BaudRateVal(baud[i]);
          CalBaud = BaudRate(CalBaudValue);
          Print("%6dbps %6dbps %6d     0x%04x\n",baud[i],CalBaud,(baud[i]-CalBaud),CalBaudValue);
     }


     Print("\r\r     << BAUD RATE TABLE >>     \r");
     Print("========+==========+=======+======================\r");
     Print("  UBRDIVn  Baudrate(TYP.) Baudrate(SPEC) Variation\r");
     Print("========+==========+=======+======================\r");

     while(StartBaudVal--) 
          Print("0x%04x    %6dbps \n", StartBaudVal, BaudRate(StartBaudVal));  

}
