/********************************************************************/
/*                                                                  */
/*       UART Header for KS32C5000                                  */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*      Modified, programmed by YOUNG-SUN KIM                       */
/*                                                                  */
/*      Description : 1998-1-25 first edited for KS32C5000          */ 
/********************************************************************/

#ifndef _BSP_UART_H_
#define _BSP_UART_H_

#include "std.h"
/***********************************************************
 *	SET SERIAL DEVICE PARAMETER                        *
 ***********************************************************/
#define SERIAL_DEV0  0
#define SERIAL_DEV1  1

#define CONSOLE      SERIAL_DEV0  //default console channel


/* Can be used in polling mode */
#define put_byte(ch) put_char(ch, CONSOLE)
extern void (put_byte)(char /*ch*/);

#define get_byte()   get_char(CONSOLE)
extern char (get_byte)(void);

/***********************************************************
 *	REDEFINE C-TYPE HEADER                             *
 ***********************************************************/
#define is_digit(x)     ((x >= '0' && x <= '9') ? 1:0)
#define is_lowercase(x) ((x >= 'a' && x <= 'z') ? 1:0)
#define is_xdigit(x)    ((is_digit(x) || (x >= 'A' && x <= 'F'))? 1:0)
#define to_upper(x)     ((is_lowercase(x)) ? x - 'a' + 'A': x)
#define is_space(x)     (((x==' ')||(x==0x0D)) ? 1:0)
#define is_control(x)   ((x > 0 && x < 32) ? 1:0)

/***********************************************************
 *	LINE CONTROL REGISTER BIT DEFINITIONS              *
 ***********************************************************/
/* bit 0-1 */
#define	ULCON_WL5               0x00
#define	ULCON_WL6               0x01
#define	ULCON_WL7               0x02
#define	ULCON_WL8               0x03

/* bit 2 */
#define	ULCON_STOP_2            0x04

/* bit 3-5 */
#define	ULCON_PMD_NO            0x00
#define	ULCON_PMD_ODD           0x20
#define	ULCON_PMD_EVEN          0x28
#define	ULCON_PMD_CHK1          0x30
#define	ULCON_PMD_CHK0          0x38

/* bit 6 */
/* bit 7 */
#define	ULCON_INFRA_RED         0x80
#define	ULCON                   0xff

/***********************************************************
 *       CONTROL REGISTER BIT DEFINITIONS                  *
 ***********************************************************/
/* bit 0-1 */
#define	UCON                    0xff
#define	UCON_RXM_OFF            0x00
#define	UCON_RXM_INTREQ         0x01
#define	UCON_RXM_GDMA0REQ       0x02
#define	UCON_RXM_GDMA1REQ       0x03

/* bit 2 */
#define	UCON_RXSTAT_INT         0x04

/* bit 3-4 */
#define	UCON_TXM_OFF            0x00
#define	UCON_TXM_INTREQ         0x08
#define	UCON_TXM_GDMA0REQ       0x10
#define	UCON_TXM_GDMA1REQ       0x18

/* bit 5 */
#define	UCON_DSR                0x20

/* bit 6 */
#define	UCON_SEND_BREAK         0x40

/* bit 7 */
#define	UCON_LOOPBACK           0x80


/***********************************************************
 *	Status Register Bit Definitions                    *
 ***********************************************************/
#define	USTAT                   0xff
#define	USTAT_OVERRUN           0x01
#define	USTAT_PARITY            0x02
#define	USTAT_FRAME             0x04
#define	USTAT_BREAK             0x08
#define	USTAT_DTR_LOW           0x10
#define	USTAT_RCV_READY         0x20
#define	USTAT_TXB_EMPTY         0x40
#define	USTAT_TX_COMPLET        0x80

#define	USTAT_ERROR	    (USTAT_OVERRUN|USTAT_PARITY|UART_FRAME)


/***********************************************/
/*          UART MACRO FUNCTIONS               */
/***********************************************/

/* Used In Polling Mode */
// Waiting for Xmitter Empty 
#define	WaitXmitter(UARTSTAT)						\
	while(!(UARTSTAT & USTAT_TXB_EMPTY))

// Waiting for Receive Data 
#define	WaitRcver(UARTSTAT)						\
	while(!(UARTSTAT & USTAT_RCV_READY))

#define UART0          '0'   
#define UART1          '1'   
#define VALID           1
#define ERROR           0

/* UART Registers */
typedef struct {
        unsigned long LCON;     // Uart Line Control register
        unsigned long CON;      // Uart Control register
        unsigned long BRD;      // Uart baud rate divisior register
        int DIALOG;   //flag to control dialog or direct function test
} UART_REG;


#define MAXEVENT 300
typedef struct
{
    char buff[MAXEVENT];     /* data buffer */
    unsigned wptr;                    /* write pointer */
    unsigned rptr;                    /* read pointer */
} QUE;


/********************************************************************/
/* T1            UART TEST TOP MODULE                               */
/********************************************************************/
extern void UartTest(void);


/********************************************************************/
/* T2            UART TEST MAIN MODULE                              */
/********************************************************************/
extern void SetBaudRate(void);
extern void  UARTIntRead(int /*channel*/,void */*RxbuffAddr*/,int /*txsize*/);
/* 
 *  Receive data using interrupt method.
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  This function receive data from receive user defined QUE and write
 *  receive data to receive buffer memory. 
 *  At the end of receive, The function return receive data byte size.
 */

extern void UARTIntWrite(int /*channel*/,void */*TxbufAddr*/,int/*txsize*/);
/*
 *   Transmit data using interrupt method.
 *   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *   This function transmit data write to transmit circular que
 *   Transmit data in circular que will be transmitted by interrupt
 *   service routine.
 *
 *     Revised 1998.5.13
 */


extern void UARTExtLoopBack(void);
/*
 *  UART external loopback test(Use Interrupt mode)
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  External plag(Null modem) for loopback test must be installed.
 *  This driver perform data transfer with null modem. hardware 
 *  controll signal(DTR,DSR) also will be used for external loopback
 *  test for assigned channel.
 */

extern unsigned int ExtLoopBacksub(int /*channel*/);
/*
 * UART External loopback main sub module.
 * Data transmit & receive use by interrupt mode.
 * Transmit data will be written to transmit QUE. This QUE will be
 * accessed by interrupt service.
 * Receive interrupt occured whenever the receive buffer register
 * have data. Receive QUE also will be written by receive interrupt
 * and accessed by user program to read receive data.
 * External Tx/Rx, DTR/DSR will be connected for external loopback test.
 */

extern int UARTAutoLoopBack(void); 
/*
 * All Uart channel internal loopback
 */
/********************************************************************/
/* T2            UART TEST SUB MODULE                               */
/********************************************************************/

extern void UARTPollInit(unsigned int /*channel*/); 
extern void UARTIntInit(unsigned int /*channel*/); 
/*
 * UART DEFAULT INITIALIZE
 * 19200 bps, 8 word, all interrupt select(tx,rx,err)
 */

extern void UARTRegWrite(int /*channel*/);
 
/********************************************************************/
/* T3            UART INTERRUPT FUNCTIONS                           */
/********************************************************************/
extern void UARTTxIntOn(unsigned int /*channel*/);
extern void UARTTxIntOff(unsigned int /*channel*/);
extern void UARTRxIntOn(unsigned int /*channel*/);
extern void UARTRxIntOff(unsigned int /*channel*/);
extern void Uart0TxIsr(void);
extern void Uart0RxIsr(void);
extern void Uart0ErrIsr(void);
extern void Uart1TxIsr(void);
extern void Uart1RxIsr(void);
extern void Uart1ErrIsr(void);

/********************************************************************/
/* T4            UART FUNCTION FOR INTERRUPT MODE                   */
/********************************************************************/

/* QUE, RX, TX QUE CONTROL FUNCTIONS */
/* WILL BE USED IN INTERRUPT MODE */

extern void TxQueWrite(char /*data*/, unsigned int /*channel*/);
/* Transmit Que write function */

extern char TxQueRead(unsigned int /*channel*/);
/* Transmit Que read function */

extern void RxQueWrite(char /*data*/,unsigned int /*channel*/);
/* Receive Que write function */

extern char RxQueRead(unsigned int /*channel*/);
/* Receive Que read function */

extern void TxQueInit(void);
/* Transmit Que initialize */

extern void RxQueInit(void);
/* Receive Que initialize */

extern void TxQueView(void);
extern void RxQueView(void);
/* See what's in the que */


/********************************************************************/
/* T5            UART FUNCTION FOR POLLING MODE                     */
/********************************************************************/

/* RECEIVE & TRANSMIT FUNCTIONS */
/* WILL BE USED IN POLLING MODE */

extern void put_char(char /*ch*/,unsigned int /*channel*/);
//void put_byte(char ch,int channel) 
/* 
   Write byte data to console through serial port. This function were 
   implemented use by polling method
*/

extern char get_char(unsigned int /*channel*/); 
//char get_byte(int channel) 
/* 
   read byte data from keyboard through serial port. This function were
   implemented use by polling method
*/

extern void put_string(char * /*ptr*/); 
//get_string is in pollio.h
/* 
   Write string data to console through serial port. This function were
   implemented use by polling method
*/


extern int gethex2dec(unsigned int /*No of digit*/);
/*
 * Get hexa value from keyboard and convert to decimal value
 * Return value will be decimal value
 */

extern int get_digit(void);
/*
 * Function : get_digit
 * Description : get decimal number
 */

extern char get_key(void) ;
extern int kbd_hit(void) ;

extern U32 get_num(void) ;
/*
 * get decimal value from hexa string
 */


/********************************************************************/
/* T6            UART START UP DIALOG FUNCTION                      */
/********************************************************************/
extern void UARTStartUpDialog(void);
extern unsigned getchannel(void);
/*
 * get Uart channel
 */

/********************************************************************/
/*  T7           UART DISPLAY FUNCTIONS                             */
/********************************************************************/

extern void UARTConfigView(void);

extern void dbg_out(char */*fmt*/, ...);
/* 
    Write string data to console through serial port with formated type.
    This function also were implemented use by polling method.
*/

extern void PrintStatus(int /*LED_disp*/, char /*Consol_disp*/);

extern void PrintUartItems(void);
/* 
 * Display GDMA test items 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */


/********************************************************************/
/*  T8             UART MACRO FUNCTIONS                             */
/********************************************************************/
extern void SendBreak(char /*channel*/,int /*mode*/);
/* Send break */

extern void SetDSR(char /*channel*/,int /*mode*/);
/* set Data Set Ready */

extern void SetDTR(char /*channel*/, int /*mode*/);
/* Set Data Terminal Ready */

extern unsigned BaudRateVal(unsigned /*br*/);
/*
 * arqument value is real baud rate(ex. 19200bps, 9600...)
 * Return value is Baud rate register write value for baud rate
 */

extern unsigned BaudRate(unsigned /*div*/);
/*
 * Arquement value is baud rate register value
 * return value is baud rate of baud rate register value.
 */

extern void BaudRateTable(void);

#endif  /* _BSP_UART_H_ */

